unit Main;

{$ifdef fpc}
  {$MODE Delphi}
{$endif}

interface

uses
  {$ifdef Win32}
    Windows,
  {$endif}
  {$ifndef fpc}
    Spin,
  {$endif}
  {$ifndef Ver80}
    ComCtrls,
  {$endif}
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  DelTime,
  WolfUtil;

type
  {$ifdef fpc}
    TSpinButton        = TUpDown;
  {$endif}

  {$ifdef Ver80}
    TUDBtnType         = (btNext, btPrev);
  {$endif}

  { TMainForm }

  TMainForm            = class(TForm)
    Panel              : TPanel;
    Label1             : TLabel;
    Label2             : TLabel;
    Label3             : TLabel;
    Label4             : TLabel;
    LblDay1            : TLabel;
    LblMonth1          : TLabel;
    LblYear1           : TLabel;
    LblHour1           : TLabel;
    LblMinute1         : TLabel;
    LblSecond1         : TLabel;
    LblJulianDay       : TLabel;
    LblIncrement       : TLabel;
    LblDay2            : TLabel;
    LblHour2           : TLabel;
    LblMinute2         : TLabel;
    LblFormat          : TLabel;
    LblOutput          : TLabel;
    LblHolidays        : TLabel;
    RadioGroup         : TRadioGroup;
    DayEdit            : TEdit;
    MonthEdit          : TEdit;
    YearEdit           : TEdit;
    HourEdit           : TEdit;
    MinuteEdit         : TEdit;
    SecondEdit         : TEdit;
    Format             : TEdit;
    JulianDayEdit      : TEdit;
    Output             : TEdit;
    DayUpDown          : TSpinButton;
    MonthUpDown        : TSpinButton;
    YearUpDown         : TSpinButton;
    HourUpDown         : TSpinButton;
    MinuteUpDown       : TSpinButton;
    SecondUpDown       : TSpinButton;
    JulianDayUpDown    : TSpinButton;
    JulianHourUpDown   : TSpinButton;
    JulianMinuteUpDown : TSpinButton;
    HolidayMemo        : TMemo;
    MoonsPhaseMemo     : TMemo;
    procedure            FormShow(Sender: TObject);
    procedure            JulianDayUpDownClick(Sender: TObject;
                                              Button: TUDBtnType);
    procedure            JulianHourUpDownClick(Sender: TObject;
                                               Button: TUDBtnType);
    procedure            JulianMinuteUpDownClick(Sender: TObject;
                                                 Button: TUDBtnType);
    procedure            SecondEditChange(Sender: TObject);
    procedure            FormCreate(Sender: TObject);
    procedure            JulianDayEditChange(Sender: TObject);
    procedure            DateChange(Sender: TObject);
    procedure            RadioGroupClick(Sender: TObject);
    procedure            SecondUpDownClick(Sender: TObject; Button: TUDBtnType);
    procedure            TimeChange(Sender: TObject);
    procedure            DayUpDownDownClick(Sender: TObject);
    procedure            DayUpDownUpClick(Sender: TObject);
    procedure            MonthUpDownDownClick(Sender: TObject);
    procedure            MonthUpDownUpClick(Sender: TObject);
    procedure            YearUpDownClick(Sender: TObject; Button: TUDBtnType);
    procedure            YearUpDownDownClick(Sender: TObject);
    procedure            YearUpDownUpClick(Sender: TObject);
    procedure            HourUpDownDownClick(Sender: TObject);
    procedure            HourUpDownUpClick(Sender: TObject);
    procedure            MinuteUpDownDownClick(Sender: TObject);
    procedure            MinuteUpDownUpClick(Sender: TObject);
    procedure            YearEditChange(Sender: TObject);
    procedure            JulianDayUpDownDownClick(Sender: TObject);
    procedure            JulianDayUpDownUpClick(Sender: TObject);
    procedure            SecondUpDownUpClick(Sender: TObject);
    procedure            SecondUpDownDownClick(Sender: TObject);
    procedure            JulianHourUpDownDownClick(Sender: TObject);
    procedure            JulianHourUpDownUpClick(Sender: TObject);
    procedure            FormResize(Sender: TObject);
    procedure            JulianMinuteUpDownDownClick(Sender: TObject);
    procedure            JulianMinuteUpDownUpClick(Sender: TObject);
    procedure            FormatChange(Sender: TObject);
  private
    Start,
    UpdateFields       : Boolean;
    procedure            StructToJulianDay;
    procedure            JulianDayToStruct;
    procedure            JulianDayToOutput;
    procedure            FillStruct(Year : Integer; Month : Word; d : Double);
    procedure            JulianDayToMoonsPhase;
    procedure            SetJulianDay(d : Double);
    procedure            SetSecond(d : Double);
  end;

var
  MainForm : TMainForm;

implementation

{$ifdef fpc}
  {$R *.lfm}
{$else}
  {$R *.dfm}
{$endif}

const
  MinHeight = 415;
  MinWidth  = 580;


function GetIntValue(Edit : tEdit; Min, Max : Integer) : Integer;
  var    i, e : Integer;
  begin
    val(Edit.Text, i, e);
    if (e <> 0) or (i < Min) or (i > Max) then
      GetIntValue := Min
    else
      GetIntValue := i;
  end;

function GetDoubleValue(Edit : tEdit) : Double;
  var    e : Integer;
         d : Double;
  begin
    val(Edit.Text, d, e);
    if e <> 0 then
      GetDoubleValue := 0
    else
      GetDoubleValue := d;
  end;

procedure DownClick(Edit : tEdit; Min, Max: Integer);
  var     i : Integer;
  begin
    i := GetIntValue(Edit, Min, Max);
    if i > Min then
      Edit.Text := IntToStr(i - 1)
    else
      Edit.Text := IntToStr(i);
  end;

procedure UpClick(Edit : tEdit; Min, Max: Integer);
  var     i : Integer;
  begin
    i := GetIntValue(Edit, Min, Max);
    if i < Max then
      Edit.Text := IntToStr(i + 1)
    else
      Edit.Text := IntToStr(i);
  end;

procedure TMainForm.FillStruct(Year : Integer; Month : Word; d : Double);
  begin
    YearEdit.Text   := IntToStr(Year);
    MonthEdit.Text  := IntToStr(Month);
    DayEdit.Text    := IntToStr(trunc(d));
    d               := frac(d) * 24;
    HourEdit.Text   := IntToStr(trunc(d));
    d               := frac(d) * 60;
    MinuteEdit.Text := IntToStr(trunc(d));
    SetSecond(frac(d) * 60);
  end;

procedure TMainForm.SecondEditChange(Sender: TObject);
  begin
    if UpdateFields then
      begin
        UpdateFields := False;
        StructToJulianDay;
        JulianDayToOutput;
        UpdateFields := True;
      end;
  end;

procedure TMainForm.JulianDayUpDownClick(Sender: TObject; Button: TUDBtnType);
  begin
    if Button = btNext then
      JulianDayUpDownUpClick(Sender)
    else
      JulianDayUpDownDownClick(Sender);
  end;

procedure TMainForm.FormShow(Sender: TObject);
  begin
    if Start then
      begin
        Start := False;
        { On Linux the non-associated UpDowns look different
          without the following lines }
        YearUpDown.Height         := DayUpDown.Height;
        SecondUpDown.Height       := DayUpDown.Height;
        JulianDayUpDown.Height    := DayUpDown.Height;
        JulianHourUpDown.Height   := DayUpDown.Height;
        JulianMinuteUpDown.Height := DayUpDown.Height;
      end;
  end;

procedure TMainForm.JulianHourUpDownClick(Sender: TObject; Button: TUDBtnType);
  begin
    if Button = btNext then
      JulianHourUpDownUpClick(Sender)
    else
      JulianHourUpDownDownClick(Sender);
  end;

procedure TMainForm.JulianMinuteUpDownClick(Sender: TObject; Button: TUDBtnType);
  begin
    if Button = btNext then
      JulianMinuteUpDownUpClick(Sender)
    else
      JulianMinuteUpDownDownClick(Sender);
  end;

procedure TMainForm.JulianDayToStruct;
  var     Year   : Integer;
          Month  : Word;
          d      : Double;
  begin
    Year  := 0;
    Month := 0;
    d     := GetDoubleValue(JulianDayEdit)
             + 0.5 / 24 / 60 / 60 / 1000;
    if RadioGroup.ItemIndex = 0 then
      GMTJulianDate(d, Year, Month, d)
    else
      LocalJulianDate(d, Year, Month, d);
    FillStruct(Year, Month, d);
  end;

procedure TMainForm.JulianDayToOutput;
  var     d : Double;
  begin
    d := GetDoubleValue(JulianDayEdit);
    if RadioGroup.ItemIndex = 0 then
      Output.Text := GMTJulianFormatDateTime(Format.Text, d)
    else
      Output.Text := LocalJulianFormatDateTime(Format.Text, d);
  end;

procedure TMainForm.JulianDayToMoonsPhase;
  var     d     : tJulianDay;
          Phase,
          i     : tMoonsPhase;
          s     : String;
          Moon  : Array[tMoonsPhase] of tJulianDay;
  begin
    d := GetDoubleValue(JulianDayEdit);
    MoonsPhaseMemo.Hide;
    MoonsPhaseMemo.Clear;
    for Phase := Low(tMoonsPhase) to High(tMoonsPhase) do
      Moon[Phase] := MoonsPhase(Phase, d);
    Phase := Low(tMoonsPhase);
    for i := Succ(Low(tMoonsPhase)) to High(tMoonsPhase) do
      if Moon[i] < Moon[Phase] then
        Phase := i;
    for i := Phase to High(tMoonsPhase) do
      begin
        if RadioGroup.ItemIndex = 0 then
          s := GMTJulianFormatDateTime(
                 GetShortDateFormat + ' ' + GetLongTimeFormat, Moon[i])
        else
          s := LocalJulianFormatDateTime(
                 GetShortDateFormat + ' ' + GetLongTimeFormat, Moon[i]);
        MoonsPhaseMemo.Lines.Add(s + ' ' + MoonsPhaseName[i]);
      end;
    if Phase > Low(tMoonsPhase) then
      for i := Low(tMoonsPhase) to Pred(Phase) do
        begin
          if RadioGroup.ItemIndex = 0 then
            s := GMTJulianFormatDateTime(
                   GetShortDateFormat + ' ' + GetLongTimeFormat, Moon[i])
          else
            s := LocalJulianFormatDateTime(
                   GetShortDateFormat + ' ' + GetLongTimeFormat, Moon[i]);
          MoonsPhaseMemo.Lines.Add(s + ' ' + MoonsPhaseName[i]);
        end;
    MoonsPhaseMemo.SelStart := 0;
    MoonsPhaseMemo.Show;
  end;

procedure TMainForm.FormCreate(Sender: TObject);
  var     Year,
          Month,
          Day    : Word;
          t      : tDateTime;
  begin
    {$ifndef Ver80}
      Constraints.MinWidth  := MinWidth;
      Constraints.MinHeight := MinHeight;
    {$endif}
    case SysLocale.PriLangID of
      Lang_German :
        begin
          Caption              := 'Zeit Umrechnungen mit Jean Meeus Funktionen';
          RadioGroup.Caption   := 'Datum / Zeit';
          RadioGroup.Items[1]  := '&lokal';
          LblJulianDay.Caption := 'Julianischer Tag';
          LblIncrement.Caption := 'Inkrement um';
          LblDay2.Caption      := 'T';
          LblHour2.Caption     := 'H';
          LblMinute2.Caption   := 'M';
          LblDay1.Caption      := 'Tag';
          LblMonth1.Caption    := 'Monat';
          LblYear1.Caption     := 'Jahr';
          LblHour1.Caption     := 'Stunde';
          LblMinute1.Caption   := 'Minute';
          LblSecond1.Caption   := 'Sekunde';
          LblHolidays.Caption  := 'Feiertage';
          LblFormat.Caption    := 'Format';
          LblOutput.Caption    := 'Ausgabe';
        end;
    end;
    LblIncrement.Left := JulianDayEdit.Left
                         + JulianDayEdit.Width
                         - Canvas.TextWidth(LblIncrement.Caption + 'n');
    Start             := True;
    HolidayMemo.Align := alClient;
    UpdateFields      := False;
    t                 := Now;
    DecodeDate(t, Year, Month, Day);
    SetJulianDay(LocalJulianDay(Year, Month, Day + Frac(t)));
    JulianDayToStruct;
    JulianDayToOutput;
    JulianDayToMoonsPhase;
    UpdateFields := True;
  end;

procedure TMainForm.StructToJulianDay;
  var     Year  : Integer;
          Month : Word;
          d     : Double;
  begin
    Year  := GetIntValue(YearEdit, MinYear, 9999);
    Month := GetIntValue(MonthEdit,    1, 12);
    d     := GetIntValue(DayEdit,      1, 31)
             + GetIntValue(HourEdit,   0, 23) / 24
             + GetIntValue(MinuteEdit, 0, 59) / 24 / 60
             + GetDoubleValue(SecondEdit) / 24 / 60 / 60;
    if RadioGroup.ItemIndex = 0 then
      d := GMTJulianDay(Year, Month, d)
    else
      d := LocalJulianDay(Year, Month, d);
    SetJulianDay(d);
  end;

procedure TMainForm.JulianDayEditChange(Sender: TObject);
  begin
    if UpdateFields then
      begin
        UpdateFields := False;
        JulianDayToStruct;
        JulianDayToMoonsPhase;
        JulianDayToOutput;
        UpdateFields := True;
      end
  end;

procedure TMainForm.DateChange(Sender: TObject);
  begin
    if UpdateFields then
      begin
        UpdateFields := False;
        StructToJulianDay;
        JulianDayToMoonsPhase;
        JulianDayToOutput;
        UpdateFields := True;
      end;
  end;

procedure TMainForm.RadioGroupClick(Sender: TObject);
  var     Year  : Integer;
          Month : Word;
          d     : Double;
  begin
    Year  := GetIntValue(YearEdit, MinYear, 9999);
    Month := GetIntValue(MonthEdit,    1, 12);
    d     :=   GetIntValue(DayEdit,    1, 31)
             + GetIntValue(HourEdit,   0, 23) / 24
             + GetIntValue(MinuteEdit, 0, 59) / 24 / 60
             + GetDoubleValue(SecondEdit) / 24 / 60 / 60;
    if RadioGroup.ItemIndex = 0 then
      LocalDateToGMTDate(Year, Month, d)
    else
      GMTDateToLocalDate(Year, Month, d);
    FillStruct(Year, Month, d);
    JulianDayToMoonsPhase;
  end;

procedure TMainForm.SecondUpDownClick(Sender: TObject; Button: TUDBtnType);
  begin
    if Button = btNext then
      SecondUpDownUpClick(Sender)
    else
      SecondUpDownDownClick(Sender);
  end;

procedure TMainForm.TimeChange(Sender: TObject);
  begin
    if UpdateFields then
      begin
        UpdateFields := False;
        StructToJulianDay;
        JulianDayToOutput;
        UpdateFields := True;
      end;
  end;

procedure TMainForm.DayUpDownDownClick(Sender: TObject);
  begin
    DownClick(DayEdit, 1, 31);
  end;

procedure TMainForm.DayUpDownUpClick(Sender: TObject);
  begin
    UpClick(DayEdit, 1, 31);
  end;

procedure TMainForm.MonthUpDownDownClick(Sender: TObject);
  begin
    DownClick(MonthEdit, 1, 12);
  end;

procedure TMainForm.MonthUpDownUpClick(Sender: TObject);
  begin
    UpClick(MonthEdit, 1, 12);
  end;

procedure TMainForm.YearUpDownClick(Sender: TObject; Button: TUDBtnType);
  begin
    if Button = btNext then
      YearUpDownUpClick(Sender)
    else
      YearUpDownDownClick(Sender);
  end;

procedure TMainForm.YearUpDownDownClick(Sender: TObject);
  begin
    DownClick(YearEdit, MinYear, 9999);
    if YearEdit.Text = '0' then
      DownClick(YearEdit, MinYear, 9999);
  end;

procedure TMainForm.YearUpDownUpClick(Sender: TObject);
  begin
    UpClick(YearEdit, MinYear, 9999);
    if YearEdit.Text = '0' then
      UpClick(YearEdit, MinYear, 9999);
  end;

procedure TMainForm.HourUpDownDownClick(Sender: TObject);
  begin
    DownClick(HourEdit, 0, 23);
  end;

procedure TMainForm.HourUpDownUpClick(Sender: TObject);
  begin
    UpClick(HourEdit, 0, 23);
  end;

procedure TMainForm.MinuteUpDownDownClick(Sender: TObject);
  begin
    DownClick(MinuteEdit, 0, 59);
  end;

procedure TMainForm.MinuteUpDownUpClick(Sender: TObject);
  begin
    UpClick(MinuteEdit, 0, 59);
  end;

procedure TMainForm.YearEditChange(Sender: TObject);
  var     l : tLanguageList;
          i : Integer;
  begin
    DateChange(Sender);
    HolidayMemo.Hide;
    HolidayMemo.Clear;
    l := tLanguageList.Create;
    CalcSpecialDays(GetIntValue(YearEdit, MinYear, 9999), l);
    l.Free;
    for i := 0 to SpecialDayList.Count - 1 do
      HolidayMemo.Lines.Add(LocalJulianFormatDateTime(
        'ddd ' + GetShortDateFormat, SpecialDayList[i].JDay)
        + ' ' + SpecialDayName[SpecialDayList[i].SpecialDay]);
    HolidayMemo.SelStart := 0;
    HolidayMemo.Show;
  end;

procedure TMainForm.SetJulianDay(d : Double);
  var     s : String;
  begin
    str(d : 1 : 9, s);
    JulianDayEdit.Text := s;
  end;

procedure TMainForm.SetSecond(d : Double);
  var     s : String;
  begin
    Str(trunc(d * 1000) / 1000 : 1 : 3, s);
    SecondEdit.Text := LNull(s, 6);
  end;

procedure TMainForm.JulianDayUpDownDownClick(Sender: TObject);
  var d : Double;
  begin
    d := GetDoubleValue(JulianDayEdit) - 1;
    if d < 0 then
      d := 0;
    SetJulianDay(d);
  end;

procedure TMainForm.JulianDayUpDownUpClick(Sender: TObject);
  begin
    SetJulianDay(GetDoubleValue(JulianDayEdit) + 1);
  end;

procedure TMainForm.SecondUpDownUpClick(Sender: TObject);
  var     d : Double;
  begin
    d := GetDoubleValue(SecondEdit) + 1;
    if d >= 60 then
      d := 59.999;
    SetSecond(d);
  end;

procedure TMainForm.SecondUpDownDownClick(Sender: TObject);
  var     d : Double;
  begin
    d := GetDoubleValue(SecondEdit) - 1;
    if d < 0 then
      d := 0;
    SetSecond(d);
  end;

procedure TMainForm.JulianHourUpDownDownClick(Sender: TObject);
  var d : Double;
  begin
    d := GetDoubleValue(JulianDayEdit) - 1 / 24;
    if d < 0 then
      d := 0;
    SetJulianDay(d);
  end;

procedure TMainForm.JulianHourUpDownUpClick(Sender: TObject);
  begin
    SetJulianDay(GetDoubleValue(JulianDayEdit) + 1 / 24);
  end;

procedure TMainForm.FormResize(Sender: TObject);
  begin
    { Delphi 1 has no form constraints }
    if Height < MinHeight then
      Height := MinHeight;
    if Width < MinWidth then
      Width := MinWidth;
  end;

procedure TMainForm.JulianMinuteUpDownDownClick(Sender: TObject);
  var d : Double;
  begin
    d := GetDoubleValue(JulianDayEdit) - 1 / 24 / 60;
    if d < 0 then
      d := 0;
    SetJulianDay(d);
  end;

procedure TMainForm.JulianMinuteUpDownUpClick(Sender: TObject);
  begin
    SetJulianDay(GetDoubleValue(JulianDayEdit) + 1 / 24 / 60);
  end;

procedure TMainForm.FormatChange(Sender: TObject);
  begin
    JulianDayToOutput;
  end;

end.
