unit Main;

{$ifdef fpc}
  {$MODE Delphi}
{$endif}

interface

uses
  {$ifdef fpc}
    ComCtrls,
  {$else}
    {$ifdef Win32}
      ComCtrls,
    {$endif}
    Spin,
  {$endif}
  {$ifdef Ver80}
    WinTypes,
    WinProcs,
  {$endif}
  {$ifdef Win32}
    Windows,
  {$endif}
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  WolfUtil;

type
  {$ifdef fpc}
    TSpinButton    = TUpDown;
  {$endif}

  {$ifdef Ver80}
    TUDBtnType     = (btNext, btPrev);
  {$endif}

  { TMainForm }

  TMainForm        = class(TForm)
    Label1         : TLabel;
    Label2         : TLabel;
    Label3         : TLabel;
    Label4         : TLabel;
    LblDay         : TLabel;
    LblMonth       : TLabel;
    LblYear        : TLabel;
    LblHour        : TLabel;
    LblMinute      : TLabel;
    LblSecond      : TLabel;
    LblFormat      : TLabel;
    LblFmtDT       : TLabel;
    LblStrFmtToDT  : TLabel;
    DayEdit        : TEdit;
    MonthEdit      : TEdit;
    YearEdit       : TEdit;
    HourEdit       : TEdit;
    MinuteEdit     : TEdit;
    SecondEdit     : TEdit;
    Format         : TEdit;
    OutFmtDT       : TEdit;
    OutStrFmtToDT  : TEdit;
    DayUpDown      : TSpinButton;
    MonthUpDown    : TSpinButton;
    YearUpDown     : TSpinButton;
    HourUpDown     : TSpinButton;
    MinuteUpDown   : TSpinButton;
    SecondUpDown   : TSpinButton;
    Label5         : TLabel;
    OutDateTime    : TEdit;
    procedure        FormCreate(Sender: TObject);
    procedure        FormShow(Sender: TObject);
    procedure        Change(Sender: TObject);
    procedure        DayUpDownDownClick(Sender: TObject);
    procedure        DayUpDownUpClick(Sender: TObject);
    procedure        MonthUpDownDownClick(Sender: TObject);
    procedure        MonthUpDownUpClick(Sender: TObject);
    procedure        YearUpDownDownClick(Sender: TObject);
    procedure        YearUpDownUpClick(Sender: TObject);
    procedure        HourUpDownDownClick(Sender: TObject);
    procedure        HourUpDownUpClick(Sender: TObject);
    procedure        MinuteUpDownDownClick(Sender: TObject);
    procedure        MinuteUpDownUpClick(Sender: TObject);
    procedure        SecondUpDownDownClick(Sender: TObject);
    procedure        SecondUpDownUpClick(Sender: TObject);
    procedure        SecondUpDownClick(Sender: TObject; Button: TUDBtnType);
  private
    Start          : Boolean;
    DayTimeFormat,
    TimeFormat     : String;
    function         GetSecond : Double;
    procedure        SetSecond(Sec : Double);
  end;

var
  MainForm : TMainForm;

implementation

{$ifdef fpc}
  {$R *.lfm}
{$else}
  {$R *.dfm}
{$endif}

function GetWord(Edit : tEdit; Min, Max : Word) : Word;
  var    i, e : Integer;
  begin
    val(Edit.Text, i, e);
    if (e <> 0) or (i < Min) or (i > Max) then
      i := Min;
    GetWord := i;
  end;

function TMainForm.GetSecond : Double;
  var    f : Double;
         e : Integer;
  begin
    val(SecondEdit.Text, f, e);
    f := (Round(f * 1000) + 0.2) / 1000;
    if (e <> 0) or (f < 0) or (f >= 60) then
      f := 0;
    GetSecond := f;
  end;

procedure TMainForm.SetSecond(Sec : Double);
  var s : String;
  begin
    str(Sec : 1 : 3, s);
    if Sec < 10 then
      s := '0' + s;
    SecondEdit.Text := s;
  end;

procedure TMainForm.FormCreate(Sender: TObject);
  var     day,
          month,
          year,
          hour,
          min,
          sec,
          MSec : Word;
          t    : tDateTime;
  begin
    Start         := True;
    DayTimeFormat := 'dddd dd.mm.yyyy hh:nn:ss';
    TimeFormat    := 'hh:nn:ss';
    {$ifndef Ver80}
      DayTimeFormat := DayTimeFormat + ', z "msec"';
      TimeFormat    := TimeFormat    + ', z "msec"';
    {$endif}
    case SysLocale.PriLangID of
      Lang_German :
        begin
          LblDay.Caption    := 'Tag';
          LblMonth.Caption  := 'Monat';
          LblYear.Caption   := 'Jahr';
          LblHour.Caption   := 'Stunde';
          LblMinute.Caption := 'Minute';
          LblSecond.Caption := 'Sekunde';
          LblFormat.Caption := 'Format';
          LblFmtDT.Caption  := 'formatierte Ausgabe';
        end;
    end;
    t := Now;
    DecodeDate(t, Year, Month, Day);
    DecodeTime(t, Hour, Min, Sec, MSec);
    DayEdit.Text    := IntToStr(Day);
    MonthEdit.Text  := IntToStr(Month);
    YearEdit.Text   := IntToStr(Year);
    HourEdit.Text   := IntToStr(Hour);
    MinuteEdit.Text := IntToStr(Min);
    SetSecond(Sec + MSec / 1000);
    Format.Text     := 'dddd c';
    {$ifndef Ver80}
      Format.Text := Format.Text + ', z "msec"';
    {$endif}
  end;

procedure TMainForm.Change(Sender: TObject);
  var     day,
          month,
          year,
          hour,
          min,
          sec,
          MSec : Word;
          f    : Double;
          t    : tDateTime;
          s    : String;
  begin
    day   := GetWord(DayEdit,    1,   31);
    Month := GetWord(MonthEdit,  1,   12);
    Year  := GetWord(YearEdit,   1, 9999);
    Hour  := GetWord(HourEdit,   0,   23);
    Min   := GetWord(MinuteEdit, 0,   59);
    f     := GetSecond;
    Sec   := Trunc(f);
    MSec  := Trunc(Frac(f) * 1000);
    try
      t := EncodeDate(Year, Month, Day);
      if t < 0 then
        t := t - EncodeTime(Hour, Min, Sec, MSec)
      else
        t := t + EncodeTime(Hour, Min, Sec, MSec);
      str(t : 1 : 8, s);
      OutDateTime.Text := s;
      OutFmtDT.Text := FormatDateTime(Format.Text, t);
      try
        t := StrWithFormatToDateTime(OutFmtDT.Text, Format.Text);
        if trunc(t) = 0 then
          OutStrFmtToDt.Text := FormatDateTime(TimeFormat, t)
        else
          OutStrFmtToDt.Text := FormatDateTime(DayTimeFormat, t);
      except
        on e : exception do
          OutStrFmtToDt.Text := e.Message;
      end;
    except
      on e : exception do
        OutFmtDT.Text := e.Message;
    end;
  end;

procedure UpDownDownClick(Edit : tEdit; Min, Max : Word);
  var     i : LongestInt;
  begin
    i := GetWord(Edit, Min, Max);
    if i > Min then
      Edit.Text := IntToStr(i - 1)
    else
      Edit.Text := IntToStr(Min);
  end;

procedure UpDownUpClick(Edit : tEdit; Min, Max : Word);
  var     i : LongestInt;
  begin
    i := GetWord(Edit, Min, Max);
    if i < Max then
      Edit.Text := IntToStr(i + 1)
    else
      Edit.Text := IntToStr(Max);
  end;

procedure TMainForm.DayUpDownDownClick(Sender: TObject);
  begin
    UpDownDownClick(DayEdit, 1, 31);
  end;

procedure TMainForm.DayUpDownUpClick(Sender: TObject);
  begin
    UpDownUpClick(DayEdit, 1, 31);
  end;

procedure TMainForm.FormShow(Sender: TObject);
  begin
    if Start then
      begin
        Start := False;
        { without the following line SecondUpDown is smaller than the others }
        SecondUpDown.Height := DayUpDown.Height;
      end;
  end;

procedure TMainForm.MonthUpDownDownClick(Sender: TObject);
  begin
    UpDownDownClick(MonthEdit, 1, 12);
  end;

procedure TMainForm.MonthUpDownUpClick(Sender: TObject);
  begin
    UpDownUpClick(MonthEdit, 1, 12);
  end;

procedure TMainForm.YearUpDownDownClick(Sender: TObject);
  begin
    UpDownDownClick(YearEdit, 1, 9999);
  end;

procedure TMainForm.YearUpDownUpClick(Sender: TObject);
  begin
    UpDownUpClick(YearEdit, 1, 9999);
  end;

procedure TMainForm.HourUpDownDownClick(Sender: TObject);
  begin
    UpDownDownClick(HourEdit, 0, 23);
  end;

procedure TMainForm.HourUpDownUpClick(Sender: TObject);
  begin
    UpDownUpClick(HourEdit, 0, 23);
  end;

procedure TMainForm.MinuteUpDownDownClick(Sender: TObject);
  begin
    UpDownDownClick(MinuteEdit, 0, 59);
  end;

procedure TMainForm.MinuteUpDownUpClick(Sender: TObject);
  begin
    UpDownUpClick(MinuteEdit, 0, 59);
  end;

procedure TMainForm.SecondUpDownDownClick(Sender: TObject);
  var     s : Double;
  begin
    s := GetSecond - 1;
    if s < 0 then
      s := 0;
    SetSecond(s);
  end;

procedure TMainForm.SecondUpDownUpClick(Sender: TObject);
  var     s : Double;
  begin
    s := GetSecond + 1;
    if s >= 60 then
      s := 59.999;
    SetSecond(s);
  end;

procedure TMainForm.SecondUpDownClick(Sender: TObject; Button: TUDBtnType);
  begin
    if Button = btNext then
      SecondUpDownUpClick(Sender)
    else
      SecondUpDownDownClick(Sender);
  end;

end.
