{ $Id: deltime.pas 131 2012-04-04 17:15:35Z Heiner $

 *****************************************************************************

                                 deltime.pas
                                 -----------
                    Initial revision : approx. year 1999
        This is a library for astronomical time calculations in Pascal.
                     Keywords are NOT case-sensitive.

 *****************************************************************************

 This file is distributed under the Library GNU General Public License
 (see the file COPYING.LGPL) with the following modification:

 As a special exception, the copyright holder of this file gives you
 permission to link this file with independent modules to produce an
 executable, regardless of the license terms of these independent modules,
 and to copy and distribute the resulting executable under terms of your choice,
 provided that you also meet, for each linked independent module, the terms
 and conditions of the license of that module. An independent module is a
 module which is not derived from or based on this file. If you modify this
 file, you may extend this exception to your version of the file, but
 you are not obligated to do so. If you do not wish to do so, delete this
 exception statement from your version.


 In order to receive a copy of the file COPYING.LGPL, contact:
      Free Software Foundation, Inc.,
      675 Mass Ave
      Cambridge, MA  02139
      USA

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 *****************************************************************************

deltime.pas for Lazarus or Delphi
Copyright (C) 2011 Heinrich Wolf.
http://www.Wolf-Fuerth.de/
}

Unit deltime;

Interface

Uses
  SysUtils,
  Classes,
  {$ifdef Unix}
    LibC,
    Unix,
    UnixUtil,
  {$else}
    {$ifdef FPC}
      Windows,
    {$else}
      {$ifdef Ver80}
        WinTypes,
        IniFiles,
      {$else}
        Windows,
      {$endif}
    {$endif}
  {$endif}
  WolfUtil;

Const
  MinYear               = -4713;

  { Output of DayOfWeek() }
  Sunday                = 1;
  Monday                = 2;
  Tuesday               = 3;
  Wednesday             = 4;
  Thursday              = 5;
  Friday                = 6;
  Saturday              = 7;

{$ifdef Win32}
  {$ifdef Ver100}
    {$define NeedTimeZone}
  {$endif}
{$else}
  {$define NeedTimeZone}
{$endif}

{$ifdef NeedTimeZone}
  TIME_ZONE_ID_UNKNOWN  = 0;
  TIME_ZONE_ID_STANDARD = 1;
  TIME_ZONE_ID_DAYLIGHT = 2;
{$endif}

Type
  tSpecialDay     =  (NormalDay,
                      Newyear,
                      Epiphany         { Heilige 3 Koenige },
                      SanAntonio,
                      PatrickDay       { Ireland },
                      ItalianFreedom,
                      NetherlandQueen,
                      MondayBeforeLent { Rosenmontag },
                      SanJose,
                      PalmSunday       { Palmsonntag },
                      MaundyThursday   { Gruendonnerstag },
                      GoodFriday       { Karfreitag },
                      EasterSunday,
                      EasterMonday,

                      ChristAscension,
                      LabourDay        { Tag der Arbeit },
                      MothersDay,
                      Armistice1945,
                      NorwegianConstitution,
                      WhitSunday       { Pfingstsonntag },
                      WhitMonday       { Pfingstmontag },
                      CorpusChristi    { Fronleichnam },
                      Santiago,
                      SwissDay         { Schweizer Bundesfeiertag },
                      AsumptionDay     { Mariae Himmelfahrt },
                      AustrianDay      { Oesterreichischer
                                         Nationalfeiertag,
                                         last occupying soldier in
                                         2nd world war is leaving },
                      USAindependence,
                      GuadalupeVirgin,
                      RussianConstitution,
                      SthAfrReconc,
                      JapaneseEmperor,
                      BelgiumDay,
                      FrenchDay,
                      GermanUnity,
                      Hispanidad,
                      AllHallows,
                      Armistice1918,
                      DayOfPrayer,
                      ImmaculateConception { Mariae Empfaengnis},
                      SpanishConstitution,
                      Advent1,
                      Advent2,
                      Advent3,
                      Advent4,
                      ChristmasEve,
                      ChristmasDay1,
                      ChristmasDay2);

const
  SpecialDayId      : Array[tSpecialDay] of String
                    = ('NormalDay',
                       'Newyear',
                       'Epiphany',
                       'SanAntonio',
                       'PatrickDay',
                       'ItalianFreedom',
                       'NetherlandQueen',
                       'MondayBeforeLent',
                       'SanJose',
                       'PalmSunday',
                       'MaundyThursday',
                       'GoodFriday',
                       'EasterSunday',
                       'EasterMonday',
                       'ChristAscension',
                       'LabourDay',
                       'MothersDay',
                       'Armistice1945',
                       'NorwegianConstitution',
                       'WhitSunday',
                       'WhitMonday',
                       'CorpusChristi',
                       'Santiago',
                       'SwissDay',
                       'AsumptionDay',
                       'AustrianDay',
                       'USAindependence',
                       'GuadalupeVirgin',
                       'RussianConstitution',
                       'SouthAfricanReconciliation',
                       'JapaneseEmperor',
                       'BelgiumDay',
                       'FrenchDay',
                       'GermanUnity',
                       'Hispanidad',
                       'AllHallows',
                       'Armistice1918',
                       'DayOfPrayer',
                       'ImmaculateConception',
                       'SpanishConstitution',
                       'Advent1',
                       'Advent2',
                       'Advent3',
                       'Advent4',
                       'ChristmasEve',
                       'ChristmasDay1',
                       'ChristmasDay2');

Type
  tMoonsPhase       = (NewMoon,
                       FirstQuarter { zunehmender Mond },
                       FullMoon,
                       LastQuarter  { abnehmender Mond } );

  tJulianDay        = double; { a float type with double precision }

  tLanguage         = record
                        PriLangID,
                        SubLangID  : word;
                      end;

  tLanguageArray    = Array[0..0] of tLanguage;
  pLanguageArray    = ^tLanguageArray;

  tLanguageList     = class
  private
    fList           : pLanguageArray;
    fCount          : LongInt;
    function          GetLanguage(Pos : LongInt) : tLanguage;
  public
    constructor       Create;
    destructor        Destroy; override;
    property          Count : LongInt read fCount;
    property          Items[Pos : LongInt] : tLanguage
                        read GetLanguage; default;
    procedure         Clear;
    procedure         Add(PriLangID, SubLangID : Word);
    procedure         Assign(l : tLanguageList);
  end;

  tRSpecialDay      = record
                        SpecialDay : tSpecialDay;
                        JDay       : tJulianDay;
                        IsHoliday  : Boolean;
                        Languages  : tLanguageList;
                      end;

  tSpecialDayArray  = Array[0..0] of tRSpecialDay;
  pSpecialDayArray  = ^tSpecialDayArray;

  tSpecialDayList   = class
  private
    fList           : pSpecialDayArray;
    fCount          : LongInt;
    function          GetSpecialDay(Pos : LongInt) : tRSpecialDay;
  public
    constructor       Create;
    destructor        Destroy; override;
    property          Count : LongInt read fCount;
    property          Items[Pos : LongInt] : tRSpecialDay
                        read GetSpecialDay; default;
    procedure         Clear;
    procedure         Add(NewVal : tRSpecialDay);
    procedure         Swap(a, b : LongInt);
  end;

  tHolidaySort      = class(tSort)
    function          MayContinue : Boolean; override;
    function          Compare(a, b : LongInt) : Integer; override;
    procedure         Swap(a, b : LongInt); override;
  end;

  tTZif                = array[0 ..  3] of char;
  ttzhead              = packed record
    tzh_TZif           : tTZif;
    tzh_reserved       : array[0 .. 15] of byte;
    tzh_ttisgmtcnt,
    tzh_ttisstdcnt,
    tzh_leapcnt,
    tzh_timecnt,
    tzh_typecnt,
    tzh_charcnt        : longint;
  end;
  ptzhead              = ^ttzhead;

  ttransitions         = array[0 .. 0] of longint;
  ptransitions         = ^ttransitions;

  tByteArray           = array[0 .. 0] of byte;
  pByteArray           = ^tByteArray;

  tByteBoolArray       = array[0 .. 0] of ByteBool;
  pByteBoolArray       = ^tByteBoolArray;

  tttinfo              = packed record
    tt_gmtoff          : longint;
    tt_isdst           : ByteBool;
    tt_abbrind         : byte;
  end;
  attinfo              = array[0 .. 0] of tttinfo;
  pttinfo              = ^attinfo;

  tleap                = record
    transition         : longint;
    change             : longint;
  end;
  aleap                = array[0 .. 0] of tleap;
  pleap                = ^aleap;

{$ifndef Win32}
  {$ifdef Ver80}
    WCHAR              = Word;
    DWORD              = LongInt;
  {$endif}

  TSystemTime          = record
    wYear              : Word;
    wMonth             : Word;
    wDayOfWeek         : Word;
    wDay               : Word;
    wHour              : Word;
    wMinute            : Word;
    wSecond            : Word;
    wMilliseconds      : Word;
  end;

  TimeZoneName         = Array[0 .. 31] of WCHAR;
  TTimeZoneInformation = record
    Bias               : Longint;
    StandardName       : TimeZoneName;
    StandardDate       : TSystemTime;
    StandardBias       : Longint;
    DaylightName       : TimeZoneName;
    DaylightDate       : TSystemTime;
    DaylightBias       : Longint;
  end;
{$endif}

var
  CurrentTimeNames     : tSysLocale;
  SpecialDayName       : Array[tSpecialDay] of String;
  MoonsPhaseName       : Array[tMoonsPhase] of String;
  SpecialDayList       : tSpecialDayList;

  tzhead               : ptzhead;
  transitions          : ptransitions;
  ttinfoind            : pByteArray;
  ttinfo               : pttinfo;
  leap                 : pleap;
  std_wall             : pByteBoolArray;
  utc_local            : pByteBoolArray;
  tzMoreData           : pByteArray;
  tzMoreLen            : LongestInt;
  tznames              : tStringList;
  { Each pointer may be nil. Only on Unix the pointers are not all nil.
    See man tzfile.
    tzMoreLen is the byte count of tzMoreData at the end of the tzfile.
    The byte order is already adjusted.
  }

procedure CalcSpecialDays(Year : Integer; Languages : tLanguageList);
            { Fills SpecialDayList and sorts it ascending by date. }

procedure GetHolidays(JD : tJulianDay; Languages : tLanguageList;
                      var l : tSpecialDayList);
            { Input Languages for correctly calculating
              SpecialDayList[].IsHoliday .
              empty SpecialDayList[].Languages means valid in all Languages,
              otherwise valid only in the given Languages.

              Examples:
              GetHolidays(Aug 15th 2001, [(LANG_GERMAN, SUBLANG_GERMAN)])
                returns AsumptionDay with
                IsHoliday = false and
                Languages = [];
              GetHolidays(Aug 15th 2001, [(LANG_GERMAN, SUBLANG_GERMAN_AUSTRIAN)])
                returns AsumptionDay with
                IsHoliday = true and
                Languages = [];
              GetHolidays(Oct 26th 2001,
                          [(LANG_GERMAN, SUBLANG_GERMAN),
                           (LANG_GERMAN, SUBLANG_GERMAN_AUSTRIAN)])
                returns AustrianDay with
                IsHoliday = true and
                Languages = [(LANG_GERMAN, SUBLANG_GERMAN_AUSTRIAN)];
            }

function  GetSpecialDay(SpecialDay    : tSpecialDay;
                        Year          : Integer;
                        var JD        : tJulianDay;
                        var Languages : tLanguageList) : Boolean;
            { GetSpecialDay returns true, if the day is a holiday.
              Input: SpecialDay, Year, Languages for correctly calculating
              the boolean return value on holidays.
              Output: JD (-1, if out of bounds),
              check the output Languages for limitations on national holidays.
            }

function  LanguageInLanguages(Language : tLanguage;
                              List : tLanguageList) : Boolean;

function  LanguagesInLanguages(Languages, List : tLanguageList) : Boolean;

procedure SetTimeNames(s : tSysLocale);

function  GMTJulianFormatDateTime(Format : String;
                                  Datum : tJulianDay) : String;

function  LocalJulianFormatDateTime(Format : String;
                                    Datum : tJulianDay) : String;

function  YearText(Year : Integer) : String;

function  Week0(Year : Integer) : tJulianDay;
          { The weeks start with monday.
            Week 1 contains January 4th.
            Week0 returns the julian day of monday in week 0.
          }

function  FirstWeek(Year : Integer) : Integer;
          { If Week 1 of the Year starts with Jan 2nd or later,
            then FirstWeek returns 0, otherwise 1
          }

function  LastWeek(Year : Integer) : Integer;
          { If Week 52 of the Year ends with Dec 30th or before,
            then LastWeek returns 53, otherwise 52
          }

procedure GetDayLightDates(Year                  : Integer;
                           var DaylightDate1,
                               DoubleSummerDate,
                               DaylightDate2,
                               StandardDate      : tJulianDay;
                           var DaylightBias,
                               DoubleSummerBias,
                               StandardBias      : Double);

function  DateTimeToJulianDay(DateTime : tDateTime) : tJulianDay;

function  JulianDayToDateTime(JulianDay : tJulianDay) : tDateTime;

procedure LocalDateToGMTDate(var Year  : Integer;
                             var Month : Word;
                             var Day   : double);
          { Fraction of Day = Time since midnight }

Procedure GMTDateToLocalDate(var Year  : Integer;
                             var Month : Word;
                             var Day   : double);
          { Fraction of Day = Time since midnight }

Function  LocalJulianDay(Year : Integer; Month: Word; Day : double)
            : tJulianDay;

Function  LocalJulDayOfWeek(JD : tJulianDay) : Word;

Procedure LocalJulianDate(JD        : tJulianDay;
                          var Year  : Integer;
                          var Month : Word;
                          var Day   : double);

function  GMTSameDate(Date1, Date2 : tJulianDay) : Boolean;

function  LocalSameDate(Date1, Date2 : tJulianDay) : Boolean;

{ The following functions are created according to
  "Astronomical Formulae for Calculators, third edition"
  from Jean Meeus, Willman-Bell publishing.
}
Function  GMTJulianDay(Year : Integer; Month: Word; Day : double)
            : tJulianDay;
          { Month 1 = January up to 12 = December.
            Fraction of Day = GMT since midnight.
            Fraction of GMTJulianDay = GMT since noon.
            Calculation is valid before and after Gregorian Calendar.
            Oct 4th 1582 + 1 = Oct 15th 1582.
            Years are astronomical:
               2 = 2 after christ
               1 = 1 after christ
               0 = 1 before christ
              -1 = 2 before christ
              -2 = 3 before christ
          }

Procedure GMTJulianDate(JD        : tJulianDay;
                        var Year  : Integer;
                        var Month : Word;
                        var Day   : double);

Function  GMTJulDayOfWeek(JD : tJulianDay) : Word;

Function  MoonsPhase(Phase : tMoonsPhase; JDay : tJulianDay) : tJulianDay;
          { JDay is the julian day for starting to search for the nearest Phase.
            The function returns the julian day of the exact Phase.
          }

Function  Easter(Year : Integer) : tJulianDay;

{$ifndef Win32}
  function GetTimeZoneInformation(var TimeZoneInformation :
                                  TTimeZoneInformation) : DWORD;
{$endif}

{$ifdef Ver80}
  function WideCharToString(Source: TimeZoneName): string;
{$endif}

implementation

var       SavedTimeZoneInfo        : tTimeZoneInformation;
          SpecialDaysYear,
          DaylightYear             : Integer;
          SavedDaylightDate1,
          SavedDoubleSummerDate,
          SavedDaylightDate2,
          SavedStandardDate        : tJulianDay;
          SavedDoubleSummerBias    : LongInt;
          LanguageList             : tLanguageList;

constructor tLanguageList.Create;
  begin
    fList  := Nil;
    fCount := 0;
  end;

destructor tLanguageList.Destroy;
  begin
    Clear;
  end;

function tLanguageList.GetLanguage(Pos : LongInt) : tLanguage;
  begin
    if (Pos < 0) or (Pos >= fCount) then
      raise Exception.Create('tLanguageList.GetLanguage(' + IntToStr(Pos)
                             + ' out of range)')
    else
      {$R-}
        GetLanguage := fList^[Pos];
      {$R+}
  end;

procedure tLanguageList.Clear;
  begin
    if fList <> Nil then
      FreeMem(fList, fCount * sizeof(tLanguage));
    fList  := Nil;
    fCount := 0;
  end;

procedure tLanguageList.Add(PriLangID, SubLangID : Word);
  var     NewList : pLanguageArray;
          i       : LongInt;
  begin
    GetMem(NewList, (fCount + 1) * sizeof(tLanguage));
    if NewList = Nil then
      Raise Exception.Create('out of memory in tLanguageList.Add')
    else
      begin
        for i := 0 to fCount - 1 do
          {$R-}
            NewList^[i] := fList^[i];
          {$R+}
        if fList <> Nil then
          FreeMem(fList, fCount * sizeof(tLanguage));
        fList := NewList;
        inc(fCount);
        {$R-}
          fList^[fCount - 1].PriLangID := PriLangID;
          fList^[fCount - 1].SubLangID := SubLangID;
        {$R+}
      end;
  end;

procedure tLanguageList.Assign(l : tLanguageList);
  var     i : Integer;
  begin
    Clear;
    for i := 0 to l.Count - 1 do
      Add(l[i].PriLangID, l[i].SubLangID);
  end;

constructor tSpecialDayList.Create;
  begin
    fList  := Nil;
    fCount := 0;
  end;

destructor tSpecialDayList.Destroy;
  begin
    Clear;
  end;

function tSpecialDayList.GetSpecialDay(Pos : LongInt) : tRSpecialDay;
  begin
    if (Pos < 0) or (Pos >= fCount) then
      raise Exception.Create('tSpecialDayList.GetSpecialDay(' + IntToStr(Pos)
                             + ' out of range)')
    else
      {$R-}
        GetSpecialDay := fList^[Pos];
      {$R+}
  end;

procedure tSpecialDayList.Clear;
  begin
    if fList <> Nil then
      FreeMem(fList, fCount * sizeof(tRSpecialDay));
    fList  := Nil;
    fCount := 0;
  end;

procedure tSpecialDayList.Add(NewVal : tRSpecialDay);
  var     NewList : pSpecialDayArray;
          i       : LongInt;
  begin
    GetMem(NewList, (fCount + 1) * sizeof(tRSpecialDay));
    if NewList = Nil then
      Raise Exception.Create('out of memory in tSpecialDayList.Add')
    else
      begin
        for i := 0 to fCount - 1 do
          {$R-}
            NewList^[i] := fList^[i];
          {$R+}
        if fList <> Nil then
          FreeMem(fList, fCount * sizeof(tRSpecialDay));
        fList := NewList;
        inc(fCount);
        {$R-}
          fList^[fCount - 1] := NewVal;
        {$R+}
      end;
  end;

procedure tSpecialDayList.Swap(a, b : LongInt);
  var     t : tRSpecialDay;
  begin
    {$R-}
      t         := fList^[a];
      fList^[a] := fList^[b];
      fList^[b] := t;
    {$R+}
  end;

procedure SetTimeNames(s : tSysLocale);
  begin
    CurrentTimeNames                      := s;
    SpecialDayName[NormalDay]             := '';
    SpecialDayName[Newyear]               := 'New year';
    SpecialDayName[Epiphany]              := 'Epiphany';
    SpecialDayName[SanAntonio]            := 'St. Antonio';
    SpecialDayName[PatrickDay]            := 'Irish St. Patrick day';
    SpecialDayName[ItalianFreedom]        := 'Italian liberty';
    SpecialDayName[NetherlandQueen]       := 'Netherlands'' Queen''s day';
    SpecialDayName[MondayBeforeLent]      := 'Monday before lent';
    SpecialDayName[SanJose]               := 'St. Josef';
    SpecialDayName[PalmSunday]            := 'Palm sunday';
    SpecialDayName[MaundyThursday]        := 'Maundy thursday';
    SpecialDayName[GoodFriday]            := 'Good friday';
    SpecialDayName[EasterSunday]          := 'Easter sunday';
    SpecialDayName[EasterMonday]          := 'Easter monday';
    SpecialDayName[ChristAscension]       := 'Christ''s ascension';
    SpecialDayName[LabourDay]             := 'Labour day';
    SpecialDayName[MothersDay]            := 'Mother''s day';
    SpecialDayName[Armistice1945]         := 'Armistice 1945';
    SpecialDayName[NorwegianConstitution] := 'Norwegian constitution day';
    SpecialDayName[WhitSunday]            := 'Whitsunday';
    SpecialDayName[WhitMonday]            := 'Whitmonday';
    SpecialDayName[CorpusChristi]         := 'Corpus Christi';
    SpecialDayName[Santiago]              := 'St. Jacob the elder';
    SpecialDayName[SwissDay]              := 'Swiss national day';
    SpecialDayName[AsumptionDay]          := 'Asumption day';
    SpecialDayName[AustrianDay]           := 'Austrian national day';
    SpecialDayName[USAindependence]       := 'USA independence day';
    SpecialDayName[GuadalupeVirgin]       := 'Virgin of Guadalupe''s day';
    SpecialDayName[RussianConstitution]   := 'Russian constitution day';
    SpecialDayName[SthAfrReconc]          := 'South African reconciliation';
    SpecialDayName[JapaneseEmperor]       := 'Japanese emperor Akihito''s birtday';
    SpecialDayName[BelgiumDay]            := 'Belgium national day';
    SpecialDayName[FrenchDay]             := 'French national day';
    SpecialDayName[GermanUnity]           := 'Day of German unity';
    SpecialDayName[Hispanidad]            := 'Discovering America';
    SpecialDayName[AllHallows]            := 'All hallows';
    SpecialDayName[Armistice1918]         := 'Armistice 1918';
    SpecialDayName[DayOfPrayer]           := 'Day of prayer';
    SpecialDayName[ImmaculateConception]  := 'Immaculate conception';
    SpecialDayName[SpanishConstitution]   := 'Spanish constitution';
    SpecialDayName[Advent1]               := '1st Advent';
    SpecialDayName[Advent2]               := '2nd Advent';
    SpecialDayName[Advent3]               := '3rd Advent';
    SpecialDayName[Advent4]               := '4th Advent';
    SpecialDayName[ChristmasEve]          := 'Christmas eve';
    SpecialDayName[ChristmasDay1]         := 'Christmasday';
    SpecialDayName[ChristmasDay2]         := 'St. Stephen day';
    MoonsPhaseName[NewMoon]               := 'New moon';
    MoonsPhaseName[FirstQuarter]          := 'First quarter';
    MoonsPhaseName[FullMoon]              := 'Full moon';
    MoonsPhaseName[LastQuarter]           := 'Last quarter';
    case s.PriLangID of
      Lang_German:
        begin
          SpecialDayName[Newyear]               := 'Neujahr';
          SpecialDayName[Epiphany]              := CP1252toSystem('Heilige drei Knige');
          SpecialDayName[PatrickDay]            := 'Irischer St. Patrick Tag';
          SpecialDayName[ItalianFreedom]        := 'Italienischer Freiheitstag';
          SpecialDayName[NetherlandQueen]       := CP1252toSystem('Niederlnder Kniginnentag');
          SpecialDayName[MondayBeforeLent]      := 'Rosenmontag';
          SpecialDayName[PalmSunday]            := 'Palmsonntag';
          SpecialDayName[MaundyThursday]        := CP1252toSystem('Grndonnerstag');
          SpecialDayName[GoodFriday]            := 'Karfreitag';
          SpecialDayName[EasterSunday]          := 'Ostersonntag';
          SpecialDayName[EasterMonday]          := 'Ostermontag';
          SpecialDayName[ChristAscension]       := 'Christi Himmelfahrt';
          SpecialDayName[LabourDay]             := 'Tag der Arbeit';
          SpecialDayName[MothersDay]            := 'Muttertag';
          SpecialDayName[Armistice1945]         := 'Ende des 2. Weltkriegs';
          SpecialDayName[NorwegianConstitution] := 'Norwegischer Verfassungstag';
          SpecialDayName[WhitSunday]            := 'Pfingstsonntag';
          SpecialDayName[WhitMonday]            := 'Pfingstmontag';
          SpecialDayName[CorpusChristi]         := 'Fronleichnam';
          SpecialDayName[Santiago]              := CP1252toSystem('St. Jakobus der ltere');
          SpecialDayName[SwissDay]              := 'Schweizer Bundesfeiertag';
          SpecialDayName[AsumptionDay]          := CP1252toSystem('Mari Himmelfahrt');
          SpecialDayName[AustrianDay]           := CP1252toSystem('sterreichischer Nationalfeiertag');
          SpecialDayName[USAindependence]       := CP1252toSystem('Unabhngigkeitstag der USA');
          SpecialDayName[GuadalupeVirgin]       := 'Tag der Jungfrau von Guadalupe';
          SpecialDayName[RussianConstitution]   := 'Tag der russischen Verfassung';
          SpecialDayName[SthAfrReconc]          := CP1252toSystem('Sdafrikanische Ausshnung');
          SpecialDayName[JapaneseEmperor]       := 'Geburtstag des japanischen Kaisers Akihito';
          SpecialDayName[BelgiumDay]            := 'Belgischer Nationalfeiertag';
          SpecialDayName[FrenchDay]             := CP1252toSystem('Franzsischer Nationalfeiertag');
          SpecialDayName[GermanUnity]           := 'Tag der deutschen Einheit';
          SpecialDayName[Hispanidad]            := 'Entdeckung Amerikas';
          SpecialDayName[AllHallows]            := 'Allerheiligen';
          SpecialDayName[Armistice1918]         := 'Ende des 1. Weltkriegs';
          SpecialDayName[DayOfPrayer]           := CP1252toSystem('Bu- und Bettag');
          SpecialDayName[ImmaculateConception]  := CP1252toSystem('Mari Empfngnis');
          SpecialDayName[SpanishConstitution]   := 'Spanischer Verfassungstag';
          SpecialDayName[Advent1]               := '1. Advent';
          SpecialDayName[Advent2]               := '2. Advent';
          SpecialDayName[Advent3]               := '3. Advent';
          SpecialDayName[Advent4]               := '4. Advent';
          SpecialDayName[ChristmasEve]          := 'Heiliger Abend';
          SpecialDayName[ChristmasDay1]         := '1. Weihnachtsfeiertag';
          SpecialDayName[ChristmasDay2]         := '2. Weihnachtsfeiertag';
          MoonsPhaseName[NewMoon]               := 'Neumond';
          MoonsPhaseName[FirstQuarter]          := 'Zunehmender Mond';
          MoonsPhaseName[FullMoon]              := 'Vollmond';
          MoonsPhaseName[LastQuarter]           := 'Abnehmender Mond';
          case s.SubLangID of
            SubLang_German_Austrian:
              begin
                SpecialDayName[LabourDay]     := 'Staatsfeiertag';
                SpecialDayName[AustrianDay]   := 'Nationalfeiertag';
                SpecialDayName[ChristmasDay1] := 'Christtag';
                SpecialDayName[ChristmasDay2] := 'Stephanitag';
              end;
            SubLang_German_Swiss,
            SubLang_German_Liechtenstein:
              begin
                SpecialDayName[MondayBeforeLent] := CP1252toSystem('Gdismontag');
                SpecialDayName[MaundyThursday]   := 'Hoher Donnerstag';
                SpecialDayName[ChristAscension]  := 'Auffahrt';
                SpecialDayName[SwissDay]         := 'Bundesfeiertag';
                SpecialDayName[ChristmasDay1]    := 'Weihnachten';
                SpecialDayName[ChristmasDay2]    := 'Stefanstag';
              end;
          end;
        end;
      Lang_Dutch:
        begin
          SpecialDayName[Newyear]         := 'Nieuwjaarsdag';
          SpecialDayName[NetherlandQueen] := 'Koninginnedag';
          SpecialDayName[PalmSunday]      := 'Palmzondag';
          SpecialDayName[MaundyThursday]  := 'Witte Donderdag';
          SpecialDayName[GoodFriday]      := 'Goede Vrijdag';
          SpecialDayName[EasterSunday]    := 'Eerste Paasdag';
          SpecialDayName[EasterMonday]    := 'Tweede Paasdag';
          SpecialDayName[ChristAscension] := 'Hemelvaartsdag';
          SpecialDayName[LabourDay]       := 'Dag van de Arbeid';
          SpecialDayName[MothersDay]      := 'Moederdag';
          SpecialDayName[Armistice1945]   := 'Wapenstilstandsdag 1945';
          SpecialDayName[WhitSunday]      := 'Eerste Pinksterdag';
          SpecialDayName[WhitMonday]      := 'Tweede Pinksterdag';
          SpecialDayName[Armistice1918]   := 'Wapenstilstandsdag 1918';
          SpecialDayName[Advent1]         := 'Eerste Advent';
          SpecialDayName[Advent2]         := 'Tweede Advent';
          SpecialDayName[Advent3]         := 'Derde Advent';
          SpecialDayName[Advent4]         := 'Vierde Advent';
          SpecialDayName[ChristmasEve]    := 'Kerstavond';
          SpecialDayName[ChristmasDay1]   := 'Eerste Kerstdag';
          SpecialDayName[ChristmasDay2]   := 'Tweede Kerstdag';
          MoonsPhaseName[FullMoon]        := 'Volle maan';
        end;
      Lang_French:
        begin
          SpecialDayName[Newyear]               := 'Jour de l''An';
          SpecialDayName[Epiphany]              := CP1252toSystem('piphanie');
          SpecialDayName[SanAntonio]            := 'Saint Antonio';
          SpecialDayName[PatrickDay]            := CP1252toSystem('Fte St. Patrick irlandais');
          SpecialDayName[ItalianFreedom]        := CP1252toSystem('Libration italien');
          SpecialDayName[NetherlandQueen]       := CP1252toSystem('Fte royal nerlandais');
          SpecialDayName[MondayBeforeLent]      := 'Lundi gras';
          SpecialDayName[SanJose]               := 'Saint Josef';
          SpecialDayName[MaundyThursday]        := 'Jeudi saint';
          SpecialDayName[GoodFriday]            := 'Vendredi saint';
          SpecialDayName[PalmSunday]            := CP1252toSystem('Pques fleuries');
          SpecialDayName[EasterSunday]          := CP1252toSystem('Pques');
          SpecialDayName[EasterMonday]          := CP1252toSystem('Lundi de Pques');
          SpecialDayName[ChristAscension]       := 'Ascension';
          SpecialDayName[LabourDay]             := CP1252toSystem('Fte du travail');
          SpecialDayName[MothersDay]            := CP1252toSystem('Fte des  mres');
          SpecialDayName[NorwegianConstitution] := CP1252toSystem('Constitution norvgien');
          SpecialDayName[WhitSunday]            := CP1252toSystem('Pentecte');
          SpecialDayName[WhitMonday]            := CP1252toSystem('Lundi de Pentecte');
          SpecialDayName[CorpusChristi]         := CP1252toSystem('Fte-Dieu');
          SpecialDayName[SwissDay]              := CP1252toSystem('Fte suisse');
          SpecialDayName[AsumptionDay]          := 'Assomption';
          SpecialDayName[AustrianDay]           := CP1252toSystem('Fte autrichien');
          SpecialDayName[USAindependence]       := CP1252toSystem('Indpendance USA');
          SpecialDayName[GuadalupeVirgin]       := CP1252toSystem('Fte de Guadalupe vierge');
          SpecialDayName[RussianConstitution]   := 'Constitution russe';
          SpecialDayName[SthAfrReconc]          := CP1252toSystem('Rconciliation sud africaine');
          SpecialDayName[JapaneseEmperor]       := 'Anniversaire empereur Akihito';
          SpecialDayName[BelgiumDay]            := CP1252toSystem('Fte nationale belge');
          SpecialDayName[FrenchDay]             := CP1252toSystem('Fte nationale');
          SpecialDayName[GermanUnity]           := CP1252toSystem('Unit allemand');
          SpecialDayName[AllHallows]            := 'Toussaint';
          SpecialDayName[DayOfPrayer]           := CP1252toSystem('Jour de pnitence et de prires');
          SpecialDayName[ImmaculateConception]  := CP1252toSystem('Conception immacul');
          SpecialDayName[Advent1]               := 'Avent 1';
          SpecialDayName[Advent2]               := 'Avent 2';
          SpecialDayName[Advent3]               := 'Avent 3';
          SpecialDayName[Advent4]               := 'Avent 4';
          SpecialDayName[ChristmasEve]          := CP1252toSystem('Veille de Nol');
          SpecialDayName[ChristmasDay1]         := CP1252toSystem('Nol');
          SpecialDayName[ChristmasDay2]         := CP1252toSystem('Saint Stphane');
          MoonsPhaseName[NewMoon]               := 'Nouvelle lune';
          MoonsPhaseName[FirstQuarter]          := 'Lune croissant';
          MoonsPhaseName[FullMoon]              := 'Pleine lune';
          MoonsPhaseName[LastQuarter]           := CP1252toSystem('Lune dcroissant');
        end;
      Lang_Italian:
        begin
          SpecialDayName[Newyear]              := 'Capodanno';
          SpecialDayName[Epiphany]             := 'Epifania';
          SpecialDayName[SanAntonio]           := 'S. Antonio';
          SpecialDayName[ItalianFreedom]       := 'Anniversario della Liberazione';
          SpecialDayName[SanJose]              := 'S. Josef';
          SpecialDayName[PalmSunday]           := 'Domenica delle palme';
          SpecialDayName[MaundyThursday]       := 'Gioved santo';
          SpecialDayName[GoodFriday]           := 'Venerdi santo';
          SpecialDayName[EasterSunday]         := 'Pasquetta';
          SpecialDayName[EasterMonday]         := 'Pasquetta Lunedi';
          SpecialDayName[ChristAscension]      := 'Ascensione';
          SpecialDayName[LabourDay]            := 'Festa del Lavoro';
          SpecialDayName[MothersDay]           := 'Festa della mamma';
          SpecialDayName[Armistice1945]        := 'Armistizio 1945';
          SpecialDayName[WhitSunday]           := 'Pentecoste';
          SpecialDayName[WhitMonday]           := 'Pentecoste Lunedi';
          SpecialDayName[AsumptionDay]         := 'Ferragosto';
          SpecialDayName[AllHallows]           := 'Ognissanti';
          SpecialDayName[Armistice1918]        := 'Armistizio 1918';
          SpecialDayName[ImmaculateConception] := 'Immacolata Concezione';
          SpecialDayName[Advent1]              := 'Avvento 1';
          SpecialDayName[Advent2]              := 'Avvento 2';
          SpecialDayName[Advent3]              := 'Avvento 3';
          SpecialDayName[Advent4]              := 'Avvento 4';
          SpecialDayName[ChristmasEve]         := 'Vigilia di natale';
          SpecialDayName[ChristmasDay1]        := 'Natale';
          SpecialDayName[ChristmasDay2]        := 'S. Stefano';
          MoonsPhaseName[FullMoon]             := 'Luna piena';
        end;
      Lang_Spanish:
        begin
          SpecialDayName[Newyear]              := CP1252toSystem('Ao Nuevo');
          SpecialDayName[Epiphany]             := CP1252toSystem('Epifana');
          SpecialDayName[SanAntonio]           := 'San Antonio';
          SpecialDayName[SanJose]              := CP1252toSystem('San Jos');
          SpecialDayName[MaundyThursday]       := 'Jueves Santo';
          SpecialDayName[PalmSunday]           := 'Domingo de Ramos';
          SpecialDayName[GoodFriday]           := 'Viernes santo';
          SpecialDayName[EasterSunday]         := 'Pascua';
          SpecialDayName[EasterMonday]         := 'Lunes de Pascua';
          SpecialDayName[ChristAscension]      := CP1252toSystem('Ascensin');
          SpecialDayName[LabourDay]            := CP1252toSystem('Da del Trabajo');
          SpecialDayName[MothersDay]           := CP1252toSystem('Da de las Madres');
          SpecialDayName[Armistice1945]        := 'Armisticio 1945';
          SpecialDayName[WhitSunday]           := CP1252toSystem('Pentecosts');
          SpecialDayName[WhitMonday]           := CP1252toSystem('Lunes de Pentecosts');
          SpecialDayName[Santiago]             := CP1252toSystem('Da del Apstol Santiago');
          SpecialDayName[AsumptionDay]         := CP1252toSystem('Asuncin de la Virgen');
          SpecialDayName[Hispanidad]           := CP1252toSystem('Da de la Hispanidad');
          SpecialDayName[AllHallows]           := 'Todos los Santos';
          SpecialDayName[Armistice1918]        := 'Armisticio 1918';
          SpecialDayName[ImmaculateConception] := CP1252toSystem('Inmaculada Concepcin');
          SpecialDayName[SpanishConstitution]  := CP1252toSystem('Da de la Constitucin');
          SpecialDayName[Advent1]              := 'Adviento 1';
          SpecialDayName[Advent2]              := 'Adviento 2';
          SpecialDayName[Advent3]              := 'Adviento 3';
          SpecialDayName[Advent4]              := 'Adviento 4';
          SpecialDayName[ChristmasEve]         := 'Nochebuena';
          SpecialDayName[ChristmasDay1]        := 'Navidad';
          SpecialDayName[ChristmasDay2]        := 'San Esteban';
          MoonsPhaseName[FullMoon]             := 'Luna llena';
        end;
      Lang_Portuguese:
        begin
          SpecialDayName[Newyear]              := 'Ano novo';
          SpecialDayName[PalmSunday]           := 'Domingo de ramos';
          SpecialDayName[MaundyThursday]       := 'Quinta-feira santa';
          SpecialDayName[GoodFriday]           := 'Sexta-feira santa';
          SpecialDayName[EasterSunday]         := CP1252toSystem('Pscoa');
          SpecialDayName[EasterMonday]         := CP1252toSystem('Segunda-feira de Pscoa');
          SpecialDayName[ChristAscension]      := CP1252toSystem('Ascenso');
          SpecialDayName[LabourDay]            := 'Dia do trabalho';
          SpecialDayName[MothersDay]           := CP1252toSystem('Dia das mes');
          SpecialDayName[Armistice1945]        := CP1252toSystem('Armistcio 1945');
          SpecialDayName[WhitSunday]           := 'Pentecostes';
          SpecialDayName[WhitMonday]           := 'Segunda-feira de Pentecostes';
          SpecialDayName[ImmaculateConception] := CP1252toSystem('Imaculada Conceio');
          SpecialDayName[Armistice1918]        := CP1252toSystem('Armistcio 1918');
          SpecialDayName[Advent1]              := 'Advento 1';
          SpecialDayName[Advent2]              := 'Advento 2';
          SpecialDayName[Advent3]              := 'Advento 3';
          SpecialDayName[Advent4]              := 'Advento 4';
          SpecialDayName[ChristmasEve]         := CP1252toSystem('Vspera de Natal');
          SpecialDayName[ChristmasDay1]        := 'Natal';
          MoonsPhaseName[FullMoon]             := 'Lua cheia';
        end;
      Lang_Swedish:
        begin
          SpecialDayName[Newyear]              := CP1252toSystem('Nyr');
          SpecialDayName[PalmSunday]           := CP1252toSystem('Palmsndag');
          SpecialDayName[MaundyThursday]       := CP1252toSystem('Skrtorsdag');
          SpecialDayName[GoodFriday]           := CP1252toSystem('Lngfredag');
          SpecialDayName[EasterSunday]         := CP1252toSystem('Psk');
          SpecialDayName[EasterMonday]         := CP1252toSystem('Pskmndag');
          SpecialDayName[ChristAscension]      := CP1252toSystem('Himmelsfrd');
          SpecialDayName[LabourDay]            := 'Arbetarnas dag';
          SpecialDayName[MothersDay]           := 'Mors dag';
          SpecialDayName[Armistice1945]        := CP1252toSystem('Vapenstillestnd 1945');
          SpecialDayName[WhitSunday]           := 'Pingstdag';
          SpecialDayName[WhitMonday]           := CP1252toSystem('Pingstmndag');
          SpecialDayName[ImmaculateConception] := CP1252toSystem('Obeflckade Avlelse');
          SpecialDayName[Armistice1918]        := CP1252toSystem('Vapenstillestnd 1918');
          SpecialDayName[Advent1]              := 'Advent 1';
          SpecialDayName[Advent2]              := 'Advent 2';
          SpecialDayName[Advent3]              := 'Advent 3';
          SpecialDayName[Advent4]              := 'Advent 4';
          SpecialDayName[ChristmasEve]         := 'Julafton';
          SpecialDayName[ChristmasDay1]        := 'Jul';
          MoonsPhaseName[FullMoon]             := CP1252toSystem('Fullmne');
        end;
    end;
  end;

function YearText(Year : Integer) : String;
  var    s : String;
  begin
    if Year > 0 then
      YearText := IntToStr(Year)
    else
      begin
        str(-Year, s);
        case SysLocale.PriLangID of
          LANG_GERMAN:
            YearText := s + ' v.Chr.';
        else
          YearText := s + ' b.Chr.';
        end;
      end;
  end;

function LanguageInLanguages(Language : tLanguage;
                             List : tLanguageList) : Boolean;
  var    b : Boolean;
         j : Integer;
  begin
    b := False;
    if List.Count = 0 then
      b := True
    else
      if Language.PriLangID = LANG_NEUTRAL then
        b := True
      else
        begin
          j := 0;
          while (j < List.Count) and not b do
            begin
              if     (Language.PriLangID = List[j].PriLangID)
                 and (Language.SubLangID = List[j].SubLangID)
                 or  (List[j].PriLangID  = LANG_NEUTRAL)
                 or  (Language.PriLangID = List[j].PriLangID)
                 and (Language.SubLangID = SUBLANG_NEUTRAL)
                 or  (Language.PriLangID = List[j].PriLangID)
                 and (List[j].SubLangID  = SUBLANG_NEUTRAL)
              then
                b := True;
              inc(j);
            end;
        end;
    LanguageInLanguages := b;
  end;

function LanguagesInLanguages(Languages, List : tLanguageList) : Boolean;
  var    b : Boolean;
         i : Integer;
  begin
    b := False;
    if Languages.Count = 0 then
      b := True
    else
      begin
        i := 0;
        while (i < Languages.Count) and not b do
          begin
            if LanguageInLanguages(Languages[i], List) then
              b := True;
            inc(i);
          end;
      end;
    LanguagesInLanguages := b;
  end;

function DifferentLanguages(a, b : tLanguageList) : Boolean;
  var    f : Boolean;
         i : Integer;
  begin
    f := False;
    i := 0;
    while (i < a.Count) and not f do
      begin
        if not LanguageInLanguages(a[i], b) then
          f := True;
        inc(i);
      end;
    i := 0;
    while (i < b.Count) and not f do
      begin
        if not LanguageInLanguages(b[i], a) then
          f := True;
        inc(i);
      end;
    DifferentLanguages := f;
  end;

procedure ChristianHolidays(var Languages : tLanguageList);
  begin
    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH_ARGENTINA);
    Languages.Add(LANG_GERMAN,     SUBLANG_GERMAN_AUSTRIAN);
    Languages.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_AUS);
    Languages.Add(LANG_DUTCH,      SUBLANG_DUTCH_BELGIAN);
    Languages.Add(LANG_FRENCH,     SUBLANG_FRENCH_BELGIAN);
    Languages.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE_BRAZILIAN);
    Languages.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_CAN);
    Languages.Add(LANG_FRENCH,     SUBLANG_FRENCH_CANADIAN);
    Languages.Add(LANG_GERMAN,     SUBLANG_GERMAN_SWISS);
    Languages.Add(LANG_FRENCH,     SUBLANG_FRENCH_SWISS);
    Languages.Add(LANG_ITALIAN,    SUBLANG_ITALIAN_SWISS);
    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH_CHILE);
    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH_COLOMBIA);
    Languages.Add(LANG_CZECH,      SUBLANG_NEUTRAL);
    Languages.Add(LANG_GERMAN,     SUBLANG_GERMAN);
    Languages.Add(LANG_DANISH,     SUBLANG_NEUTRAL);
    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH);
    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH_MODERN);
    Languages.Add(LANG_FINNISH,    SUBLANG_NEUTRAL);
    Languages.Add(LANG_FRENCH,     SUBLANG_FRENCH);
    Languages.Add(LANG_GREEK,      SUBLANG_NEUTRAL);
    Languages.Add(LANG_HUNGARIAN,  SUBLANG_NEUTRAL);
    Languages.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_EIRE);
    Languages.Add(LANG_ITALIAN,    SUBLANG_ITALIAN);
    Languages.Add(LANG_FRENCH,     SUBLANG_FRENCH_LUXEMBOURG);
    Languages.Add(LANG_GERMAN,     SUBLANG_GERMAN_LUXEMBOURG);
    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH_MEXICAN);
    Languages.Add(LANG_DUTCH,      SUBLANG_DUTCH);
    Languages.Add(LANG_NORWEGIAN,  SUBLANG_NEUTRAL);
    Languages.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_NZ);
    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH_PERU);
    Languages.Add(LANG_POLISH,     SUBLANG_NEUTRAL);
    Languages.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE);
    Languages.Add(LANG_SWEDISH,    SUBLANG_SWEDISH);
    Languages.Add(LANG_CHINESE,    SUBLANG_CHINESE_SINGAPORE);
    Languages.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_UK);
    Languages.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_US);
    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH_VENEZUELA);
    Languages.Add(LANG_AFRIKAANS,  SUBLANG_NEUTRAL);
    Languages.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_SOUTH_AFRICA);
  end;

function GetOffsetSpecialDay(SpecialDay    : tSpecialDay;
                             WeekDayShift  : Integer;
                             Easter        : tJulianDay;
                             Year          : Integer;
                             var Date      : tJulianDay;
                             var Languages : tLanguageList) : Boolean;
  { GetOffsetSpecialDay returns true, if SpecialDay is a Holiday }
  const FirstEaster = 532;
  var   c, l        : tLanguageList;
  begin
    GetOffsetSpecialDay := False;
    try
      c := tLanguageList.Create;
    except
      c := Nil;
    end;
    try
      l := tLanguageList.Create;
    except
      l := Nil;
    end;
    if (c <> Nil) and (l <> Nil) then
      try
        c.Assign(Languages);
        Date := -1;
        Languages.Clear;
        case SpecialDay of
          Newyear :
            if Year >= 1691 then
              begin
                Date                := LocalJulianDay(Year, 1, 1.5);
                GetOffsetSpecialDay := True;
              end;
          Epiphany :
            if Year > FirstEaster then
              begin
                Date := LocalJulianDay(Year, 1, 6);
                l.Add(LANG_GERMAN,  SUBLANG_GERMAN_AUSTRIAN);
                l.Add(LANG_GERMAN,  SUBLANG_GERMAN_SWISS);
                l.Add(LANG_FRENCH,  SUBLANG_FRENCH_SWISS);
                l.Add(LANG_ITALIAN, SUBLANG_ITALIAN_SWISS);
                l.Add(LANG_SPANISH, SUBLANG_SPANISH_COLOMBIA);
                l.Add(LANG_GERMAN,  SUBLANG_GERMAN);
                l.Add(LANG_SPANISH, SUBLANG_SPANISH);
                l.Add(LANG_SPANISH, SUBLANG_SPANISH_MODERN);
                l.Add(LANG_FINNISH, SUBLANG_NEUTRAL);
                l.Add(LANG_GREEK,   SUBLANG_NEUTRAL);
                l.Add(LANG_ITALIAN, SUBLANG_ITALIAN);
                l.Add(LANG_SWEDISH, SUBLANG_SWEDISH);
                GetOffsetSpecialDay := LanguagesInLanguages(c, l);
                ChristianHolidays(Languages);
              end;
          SanAntonio :
            if Year > FirstEaster then
              begin
                Date := LocalJulianDay(Year, 1, 17.5);
                Languages.Add(LANG_SPANISH, SUBLANG_SPANISH);
                Languages.Add(LANG_SPANISH, SUBLANG_SPANISH_MODERN);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          PatrickDay:
            if year >= 461 then
              begin
                Date := LocalJulianDay(Year, 3, 17.5);
                Languages.Add(LANG_ENGLISH, SUBLANG_ENGLISH_EIRE);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          ItalianFreedom:
            if year >= 1946 then
              begin
                Date := LocalJulianDay(Year, 4, 25.5);
                Languages.Add(LANG_ITALIAN, SUBLANG_ITALIAN);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          NetherlandQueen:
            if year >= 1980 then
              begin
                Date := LocalJulianDay(Year, 4, 30.5);
                Languages.Add(LANG_DUTCH, SUBLANG_DUTCH);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          MondayBeforeLent :
            if Year > FirstEaster Then
              Date := Easter - 48;
          SanJose :
            if Year > FirstEaster Then
              begin
                Date := LocalJulianDay(Year, 3, 19.5);
                Languages.Add(LANG_SPANISH, SUBLANG_SPANISH);
                Languages.Add(LANG_SPANISH, SUBLANG_SPANISH_MODERN);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          PalmSunday :
            if Year > FirstEaster Then
              begin
                Date                := Easter - 7;
                GetOffsetSpecialDay := True;
                ChristianHolidays(Languages);
              end;
          MaundyThursday:
            if Year > FirstEaster Then
              begin
                Date := Easter - 3;
                l.Add(LANG_NORWEGIAN, SUBLANG_NEUTRAL);
                l.Add(LANG_SPANISH,   SUBLANG_SPANISH);
                l.Add(LANG_SPANISH,   SUBLANG_SPANISH_MODERN);
                GetOffsetSpecialDay := LanguagesInLanguages(c, l);
                ChristianHolidays(Languages);
              end;
          GoodFriday :
            if Year > FirstEaster Then
              begin
                Date := Easter - 2;
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_ARGENTINA);
                l.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_AUS);
                l.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE_BRAZILIAN);
                l.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_CAN);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_CANADIAN);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_SWISS);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_SWISS);
                l.Add(LANG_ITALIAN,    SUBLANG_ITALIAN_SWISS);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_CHILE);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_COLOMBIA);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN);
                l.Add(LANG_DANISH,     SUBLANG_NEUTRAL);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_MODERN);
                l.Add(LANG_FINNISH,    SUBLANG_NEUTRAL);
                l.Add(LANG_GREEK,      SUBLANG_NEUTRAL);
                l.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_EIRE);
                l.Add(LANG_NORWEGIAN,  SUBLANG_NEUTRAL);
                l.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_NZ);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_PERU);
                l.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE);
                l.Add(LANG_SWEDISH,    SUBLANG_SWEDISH);
                l.Add(LANG_CHINESE,    SUBLANG_CHINESE_SINGAPORE);
                l.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_UK);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_VENEZUELA);
                l.Add(LANG_AFRIKAANS,  SUBLANG_NEUTRAL);
                l.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_SOUTH_AFRICA);
                GetOffsetSpecialDay := LanguagesInLanguages(c, l);
                ChristianHolidays(Languages);
              end;
          EasterSunday :
            if Year >= FirstEaster Then
              begin
                Date                := Easter;
                GetOffsetSpecialDay := True;
                ChristianHolidays(Languages);
              end;
          EasterMonday :
            if Year >= FirstEaster Then
              begin
                Date := Easter + 1;
                l.Add(LANG_GERMAN,    SUBLANG_GERMAN_AUSTRIAN);
                l.Add(LANG_ENGLISH,   SUBLANG_ENGLISH_AUS);
                l.Add(LANG_DUTCH,     SUBLANG_DUTCH_BELGIAN);
                l.Add(LANG_FRENCH,    SUBLANG_FRENCH_BELGIAN);
                l.Add(LANG_ENGLISH,   SUBLANG_ENGLISH_CAN);
                l.Add(LANG_FRENCH,    SUBLANG_FRENCH_CANADIAN);
                l.Add(LANG_GERMAN,    SUBLANG_GERMAN_SWISS);
                l.Add(LANG_FRENCH,    SUBLANG_FRENCH_SWISS);
                l.Add(LANG_ITALIAN,   SUBLANG_ITALIAN_SWISS);
                l.Add(LANG_CZECH,     SUBLANG_NEUTRAL);
                l.Add(LANG_GERMAN,    SUBLANG_GERMAN);
                l.Add(LANG_DANISH,    SUBLANG_NEUTRAL);
                l.Add(LANG_FINNISH,   SUBLANG_NEUTRAL);
                l.Add(LANG_FRENCH,    SUBLANG_FRENCH);
                l.Add(LANG_HUNGARIAN, SUBLANG_NEUTRAL);
                l.Add(LANG_ENGLISH,   SUBLANG_ENGLISH_EIRE);
                l.Add(LANG_ITALIAN,   SUBLANG_ITALIAN);
                l.Add(LANG_FRENCH,    SUBLANG_FRENCH_LUXEMBOURG);
                l.Add(LANG_GERMAN,    SUBLANG_GERMAN_LUXEMBOURG);
                l.Add(LANG_DUTCH,     SUBLANG_DUTCH);
                l.Add(LANG_NORWEGIAN, SUBLANG_NEUTRAL);
                l.Add(LANG_ENGLISH,   SUBLANG_ENGLISH_NZ);
                l.Add(LANG_POLISH,    SUBLANG_NEUTRAL);
                l.Add(LANG_SWEDISH,   SUBLANG_SWEDISH);
                l.Add(LANG_ENGLISH,   SUBLANG_ENGLISH_UK);
                l.Add(LANG_SPANISH,   SUBLANG_SPANISH_VENEZUELA);
                l.Add(LANG_AFRIKAANS, SUBLANG_NEUTRAL);
                l.Add(LANG_ENGLISH,   SUBLANG_ENGLISH_SOUTH_AFRICA);
                GetOffsetSpecialDay := LanguagesInLanguages(c, l);
                ChristianHolidays(Languages);
              end;
          ChristAscension :
            if Year >= FirstEaster Then
              begin
                Date := Easter + 39;
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_AUSTRIAN);
                l.Add(LANG_DUTCH,      SUBLANG_DUTCH_BELGIAN);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_BELGIAN);
                l.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE_BRAZILIAN);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_SWISS);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_SWISS);
                l.Add(LANG_ITALIAN,    SUBLANG_ITALIAN_SWISS);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN);
                l.Add(LANG_DANISH,     SUBLANG_NEUTRAL);
                l.Add(LANG_FINNISH,    SUBLANG_NEUTRAL);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_LUXEMBOURG);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_LUXEMBOURG);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_MEXICAN);
                l.Add(LANG_DUTCH,      SUBLANG_DUTCH);
                l.Add(LANG_NORWEGIAN,  SUBLANG_NEUTRAL);
                l.Add(LANG_SWEDISH,    SUBLANG_SWEDISH);
                GetOffsetSpecialDay := LanguagesInLanguages(c, l);
                ChristianHolidays(Languages);
              end;
          LabourDay :
            if Year >= 1882 then
              begin
                Languages.Add(LANG_ENGLISH, SUBLANG_ENGLISH_US);
                Languages.Add(LANG_ENGLISH, SUBLANG_ENGLISH_CAN);
                Languages.Add(LANG_FRENCH,  SUBLANG_FRENCH_CANADIAN);
                Languages.Add(LANG_SPANISH, SUBLANG_SPANISH_PUERTO_RICO);
                Languages.Add(LANG_SPANISH, SUBLANG_SPANISH_PANAMA);
                if LanguagesInLanguages(c, Languages) then
                  begin
                    case Year of
                      1882,
                      1883: Date := LocalJulianDay(Year, 9, 5.5);
                    else
                      Date := LocalJulianDay(Year, 9, 11.5) - WeekDayShift;
                    end;
                    GetOffsetSpecialDay := True;
                  end
                else
                  begin
                    Languages.Clear;
                    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH_ARGENTINA);
                    Languages.Add(LANG_GERMAN,     SUBLANG_GERMAN_AUSTRIAN);
                    Languages.Add(LANG_DUTCH,      SUBLANG_DUTCH_BELGIAN);
                    Languages.Add(LANG_FRENCH,     SUBLANG_FRENCH_BELGIAN);
                    Languages.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE_BRAZILIAN);
                    Languages.Add(LANG_GERMAN,     SUBLANG_GERMAN_SWISS);
                    Languages.Add(LANG_FRENCH,     SUBLANG_FRENCH_SWISS);
                    Languages.Add(LANG_ITALIAN,    SUBLANG_ITALIAN_SWISS);
                    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH_CHILE);
                    Languages.Add(LANG_CHINESE,    SUBLANG_CHINESE_SIMPLIFIED);
                    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH_COLOMBIA);
                    Languages.Add(LANG_CZECH,      SUBLANG_NEUTRAL);
                    Languages.Add(LANG_GERMAN,     SUBLANG_GERMAN);
                    Languages.Add(LANG_DANISH,     SUBLANG_NEUTRAL);
                    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH);
                    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH_MODERN);
                    Languages.Add(LANG_FINNISH,    SUBLANG_NEUTRAL);
                    Languages.Add(LANG_FRENCH,     SUBLANG_FRENCH);
                    Languages.Add(LANG_GREEK,      SUBLANG_NEUTRAL);
                    Languages.Add(LANG_HUNGARIAN,  SUBLANG_NEUTRAL);
                    Languages.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_EIRE);
                    Languages.Add(LANG_ITALIAN,    SUBLANG_ITALIAN);
                    Languages.Add(LANG_FRENCH,     SUBLANG_FRENCH_LUXEMBOURG);
                    Languages.Add(LANG_GERMAN,     SUBLANG_GERMAN_LUXEMBOURG);
                    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH_MEXICAN);
                    Languages.Add(LANG_DUTCH,      SUBLANG_DUTCH);
                    Languages.Add(LANG_NORWEGIAN,  SUBLANG_NEUTRAL);
                    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH_PERU);
                    Languages.Add(LANG_POLISH,     SUBLANG_NEUTRAL);
                    Languages.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE);
                    Languages.Add(LANG_RUSSIAN,    SUBLANG_NEUTRAL);
                    Languages.Add(LANG_SWEDISH,    SUBLANG_SWEDISH);
                    Languages.Add(LANG_CHINESE,    SUBLANG_CHINESE_SINGAPORE);
                    Languages.Add(LANG_SPANISH,    SUBLANG_SPANISH_VENEZUELA);
                    Languages.Add(LANG_AFRIKAANS,  SUBLANG_NEUTRAL);
                    Languages.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_SOUTH_AFRICA);
                    if LanguagesInLanguages(c, Languages) then
                      begin
                        Date := LocalJulianDay(Year, 5, 1.5);
                        l.Add(LANG_SPANISH,    SUBLANG_SPANISH_ARGENTINA);
                        l.Add(LANG_GERMAN,     SUBLANG_GERMAN_AUSTRIAN);
                        l.Add(LANG_DUTCH,      SUBLANG_DUTCH_BELGIAN);
                        l.Add(LANG_FRENCH,     SUBLANG_FRENCH_BELGIAN);
                        l.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE_BRAZILIAN);
                        l.Add(LANG_GERMAN,     SUBLANG_GERMAN_SWISS);
                        l.Add(LANG_FRENCH,     SUBLANG_FRENCH_SWISS);
                        l.Add(LANG_ITALIAN,    SUBLANG_ITALIAN_SWISS);
                        l.Add(LANG_SPANISH,    SUBLANG_SPANISH_CHILE);
                        l.Add(LANG_CHINESE,    SUBLANG_CHINESE_SIMPLIFIED);
                        l.Add(LANG_SPANISH,    SUBLANG_SPANISH_COLOMBIA);
                        l.Add(LANG_CZECH,      SUBLANG_NEUTRAL);
                        l.Add(LANG_GERMAN,     SUBLANG_GERMAN);
                        l.Add(LANG_SPANISH,    SUBLANG_SPANISH);
                        l.Add(LANG_SPANISH,    SUBLANG_SPANISH_MODERN);
                        l.Add(LANG_FINNISH,    SUBLANG_NEUTRAL);
                        l.Add(LANG_FRENCH,     SUBLANG_FRENCH);
                        l.Add(LANG_GREEK,      SUBLANG_NEUTRAL);
                        l.Add(LANG_HUNGARIAN,  SUBLANG_NEUTRAL);
                        l.Add(LANG_ITALIAN,    SUBLANG_ITALIAN);
                        l.Add(LANG_FRENCH,     SUBLANG_FRENCH_LUXEMBOURG);
                        l.Add(LANG_GERMAN,     SUBLANG_GERMAN_LUXEMBOURG);
                        l.Add(LANG_SPANISH,    SUBLANG_SPANISH_MEXICAN);
                        l.Add(LANG_NORWEGIAN,  SUBLANG_NEUTRAL);
                        l.Add(LANG_SPANISH,    SUBLANG_SPANISH_PERU);
                        l.Add(LANG_POLISH,     SUBLANG_NEUTRAL);
                        l.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE);
                        l.Add(LANG_RUSSIAN,    SUBLANG_NEUTRAL);
                        l.Add(LANG_SWEDISH,    SUBLANG_SWEDISH);
                        l.Add(LANG_CHINESE,    SUBLANG_CHINESE_SINGAPORE);
                        l.Add(LANG_SPANISH,    SUBLANG_SPANISH_VENEZUELA);
                        l.Add(LANG_AFRIKAANS,  SUBLANG_NEUTRAL);
                        l.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_SOUTH_AFRICA);
                        GetOffsetSpecialDay := LanguagesInLanguages(c, l);
                      end
                    else
                      Languages.Clear;
                  end;
              end;
          MothersDay :
            if Year >= 1922 Then
              begin
                Date                := LocalJulianDay(Year, 5, 14.5) - WeekDayShift;
                GetOffsetSpecialDay := True;
              end;
          Armistice1945 :
            if Year >= 1945 then
              begin
                Date := LocalJulianDay(Year, 5, 8.5);
                Languages.Add(LANG_FRENCH, SUBLANG_FRENCH);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          NorwegianConstitution:
            if year >= 1814 then
              begin
                Date := LocalJulianDay(Year, 5, 17.5);
                Languages.Add(LANG_NORWEGIAN, SUBLANG_NEUTRAL);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          WhitSunday :
            if Year >= FirstEaster Then
              begin
                Date                := Easter + 49;
                GetOffsetSpecialDay := True;
                ChristianHolidays(Languages);
              end;
          WhitMonday :
            if Year >= FirstEaster Then
              begin
                Date := Easter + 50;
                l.Add(LANG_GERMAN,    SUBLANG_GERMAN_AUSTRIAN);
                l.Add(LANG_DUTCH,     SUBLANG_DUTCH_BELGIAN);
                l.Add(LANG_FRENCH,    SUBLANG_FRENCH_BELGIAN);
                l.Add(LANG_GERMAN,    SUBLANG_GERMAN_SWISS);
                l.Add(LANG_FRENCH,    SUBLANG_FRENCH_SWISS);
                l.Add(LANG_ITALIAN,   SUBLANG_ITALIAN_SWISS);
                l.Add(LANG_GERMAN,    SUBLANG_GERMAN);
                l.Add(LANG_DANISH,    SUBLANG_NEUTRAL);
                l.Add(LANG_FRENCH,    SUBLANG_FRENCH);
                l.Add(LANG_HUNGARIAN, SUBLANG_NEUTRAL);
                l.Add(LANG_FRENCH,    SUBLANG_FRENCH_LUXEMBOURG);
                l.Add(LANG_GERMAN,    SUBLANG_GERMAN_LUXEMBOURG);
                l.Add(LANG_DUTCH,     SUBLANG_DUTCH);
                l.Add(LANG_NORWEGIAN, SUBLANG_NEUTRAL);
                l.Add(LANG_SWEDISH,   SUBLANG_SWEDISH);
                GetOffsetSpecialDay := LanguagesInLanguages(c, l);
                ChristianHolidays(Languages);
              end;
          CorpusChristi :
            if Year >= FirstEaster Then
              begin
                Date := Easter + 60;
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_AUSTRIAN);
                l.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE_BRAZILIAN);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_SWISS);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_SWISS);
                l.Add(LANG_ITALIAN,    SUBLANG_ITALIAN_SWISS);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_CHILE);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN);
                l.Add(LANG_POLISH,     SUBLANG_NEUTRAL);
                l.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE);
                GetOffsetSpecialDay := LanguagesInLanguages(c, l);
                ChristianHolidays(Languages);
              end;
          Santiago :
            if Year >= FirstEaster Then
              begin
                Date := LocalJulianDay(Year, 7, 25.5);
                Languages.Add(LANG_SPANISH, SUBLANG_SPANISH);
                Languages.Add(LANG_SPANISH, SUBLANG_SPANISH_MODERN);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          SwissDay :
            if year >= 1848 then
              begin
                Date := LocalJulianDay(Year, 8, 1.5);
                Languages.Add(LANG_GERMAN,  SUBLANG_GERMAN_SWISS);
                Languages.Add(LANG_ITALIAN, SUBLANG_ITALIAN_SWISS);
                Languages.Add(LANG_FRENCH,  SUBLANG_FRENCH_SWISS);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          AsumptionDay :
            if Year >= FirstEaster Then
              begin
                Date := LocalJulianDay(Year, 8, 15.5);
                l.Add(LANG_GERMAN, SUBLANG_GERMAN);
                if (Year <= 1993) and LanguagesInLanguages(c, l) then
                  GetOffsetSpecialDay := True;
                l.Clear;
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_AUSTRIAN);
                l.Add(LANG_DUTCH,      SUBLANG_DUTCH_BELGIAN);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_BELGIAN);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_SWISS);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_SWISS);
                l.Add(LANG_ITALIAN,    SUBLANG_ITALIAN_SWISS);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_CHILE);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_COLOMBIA);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_MODERN);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH);
                l.Add(LANG_GREEK,      SUBLANG_NEUTRAL);
                l.Add(LANG_ITALIAN,    SUBLANG_ITALIAN);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_LUXEMBOURG);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_LUXEMBOURG);
                l.Add(LANG_POLISH,     SUBLANG_NEUTRAL);
                l.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE);
                if LanguagesInLanguages(c, l) then
                  GetOffsetSpecialDay := True;
                ChristianHolidays(Languages);
              end;
          AustrianDay:
            if year >= 1955 then
              begin
                Date := LocalJulianDay(Year, 10, 26.5);
                Languages.Add(LANG_GERMAN, SUBLANG_GERMAN_AUSTRIAN);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          USAindependence:
            if year >= 1776 then
              begin
                Date := LocalJulianDay(Year, 7, 4.5);
                Languages.Add(LANG_ENGLISH, SUBLANG_ENGLISH_US);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          GuadalupeVirgin:
            if year >= 1531 then
              begin
                Date := LocalJulianDay(Year, 12, 12.5);
                Languages.Add(LANG_SPANISH, SUBLANG_SPANISH_MEXICAN);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          RussianConstitution:
            if year >= 1993 then
              begin
                Date := LocalJulianDay(Year, 12, 12.5);
                Languages.Add(LANG_RUSSIAN, SUBLANG_NEUTRAL);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          SthAfrReconc:
            if year >= 1995 then
              begin
                Date := LocalJulianDay(Year, 12, 16.5);
                Languages.Add(LANG_AFRIKAANS, SUBLANG_NEUTRAL);
                Languages.Add(LANG_ENGLISH,   SUBLANG_ENGLISH_SOUTH_AFRICA);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          JapaneseEmperor:
            if year >= 1989 then
              begin
                Date := LocalJulianDay(Year, 12, 23.5);
                Languages.Add(LANG_JAPANESE, SUBLANG_NEUTRAL);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          BelgiumDay:
            if year >= 1831 then
              begin
                Date := LocalJulianDay(Year, 7, 21.5);
                Languages.Add(LANG_DUTCH,  SUBLANG_DUTCH_BELGIAN);
                Languages.Add(LANG_FRENCH, SUBLANG_FRENCH_BELGIAN);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          FrenchDay:
            if year >= 1790 then
              begin
                Date := LocalJulianDay(Year, 7, 14.5);
                Languages.Add(LANG_FRENCH, SUBLANG_FRENCH);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          GermanUnity :
            if Year > 1953 Then
              begin
                { since the rebellion in divided eastern germany }
                if (Year <= 1990) then
                  { before reunion }
                  Date := LocalJulianDay(Year, 6, 17.5)
                else
                  { after reunion }
                  Date := LocalJulianDay(Year, 10, 3.5);
                Languages.Add(LANG_GERMAN, SUBLANG_GERMAN);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          Hispanidad :
            if Year >= 1942 then
              begin
                Date := LocalJulianDay(Year, 10, 12.5);
                Languages.Add(LANG_SPANISH, SUBLANG_SPANISH);
                Languages.Add(LANG_SPANISH, SUBLANG_SPANISH_MODERN);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          AllHallows :
            if Year >= FirstEaster Then
              begin
                Date := LocalJulianDay(Year, 11, 1.5);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_AUSTRIAN);
                l.Add(LANG_DUTCH,      SUBLANG_DUTCH_BELGIAN);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_BELGIAN);
                l.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE_BRAZILIAN);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_SWISS);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_SWISS);
                l.Add(LANG_ITALIAN,    SUBLANG_ITALIAN_SWISS);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_CHILE);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_MODERN);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH);
                l.Add(LANG_ITALIAN,    SUBLANG_ITALIAN);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_LUXEMBOURG);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_LUXEMBOURG);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_PERU);
                l.Add(LANG_POLISH,     SUBLANG_NEUTRAL);
                l.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE);
                l.Add(LANG_CHINESE,    SUBLANG_CHINESE_SINGAPORE);
                GetOffsetSpecialDay := LanguagesInLanguages(c, l);
                ChristianHolidays(Languages);
              end;
          Armistice1918:
            if year >= 1918 then
              begin
                Date := LocalJulianDay(Year, 11, 11.5);
                Languages.Add(LANG_DUTCH,  SUBLANG_DUTCH_BELGIAN);
                Languages.Add(LANG_FRENCH, SUBLANG_FRENCH_BELGIAN);
                Languages.Add(LANG_FRENCH, SUBLANG_FRENCH);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          DayOfPrayer :
            if Year >= FirstEaster Then
              begin
                Date := LocalJulianDay(Year, 11, 22.5) - WeekDayShift;
                l.Add(LANG_GERMAN, SUBLANG_GERMAN);
                GetOffsetSpecialDay := (Year <= 1994) and LanguagesInLanguages(c, l);
                ChristianHolidays(Languages);
              end;
          ImmaculateConception :
            if Year >= FirstEaster Then
              begin
                Date := LocalJulianDay(Year, 12, 8.5);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_ARGENTINA);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_AUSTRIAN);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_SWISS);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_SWISS);
                l.Add(LANG_ITALIAN,    SUBLANG_ITALIAN_SWISS);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_CHILE);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_COLOMBIA);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_MODERN);
                l.Add(LANG_ITALIAN,    SUBLANG_ITALIAN);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_PERU);
                l.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE);
                GetOffsetSpecialDay := LanguagesInLanguages(c, l);
                ChristianHolidays(Languages);
              end;
          SpanishConstitution :
            if Year >= 1978 then
              begin
                Date := LocalJulianDay(Year, 12, 6.5);
                Languages.Add(LANG_SPANISH, SUBLANG_SPANISH);
                Languages.Add(LANG_SPANISH, SUBLANG_SPANISH_MODERN);
                GetOffsetSpecialDay := LanguagesInLanguages(c, Languages);
              end;
          Advent1 :
            if Year >= FirstEaster Then
              begin
                Date                := LocalJulianDay(Year, 12, 3.5) - WeekDayShift;
                GetOffsetSpecialDay := True;
                ChristianHolidays(Languages);
              end;
          Advent2 :
            if Year >= FirstEaster Then
              begin
                Date                := LocalJulianDay(Year, 12, 10.5) - WeekDayShift;
                GetOffsetSpecialDay := True;
                ChristianHolidays(Languages);
              end;
          Advent3 :
            if Year >= FirstEaster Then
              begin
                Date                := LocalJulianDay(Year, 12, 17.5) - WeekDayShift;
                GetOffsetSpecialDay := True;
                ChristianHolidays(Languages);
              end;
          Advent4 :
            if Year >= FirstEaster Then
              begin
                Date                := LocalJulianDay(Year, 12, 24.5) - WeekDayShift;
                GetOffsetSpecialDay := True;
                ChristianHolidays(Languages);
              end;
          ChristmasEve:
            if Year >= FirstEaster Then
              begin
                Date := LocalJulianDay(Year, 12, 24.5);
                l.Add(LANG_CZECH,   SUBLANG_NEUTRAL);
                l.Add(LANG_FINNISH, SUBLANG_NEUTRAL);
                l.Add(LANG_SWEDISH, SUBLANG_SWEDISH_FINLAND);
                GetOffsetSpecialDay := LanguagesInLanguages(c, l);
                ChristianHolidays(Languages);
              end;
          ChristmasDay1 :
            if Year >= FirstEaster Then
              begin
                Date := LocalJulianDay(Year, 12, 25.5);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_ARGENTINA);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_AUSTRIAN);
                l.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_AUS);
                l.Add(LANG_DUTCH,      SUBLANG_DUTCH_BELGIAN);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_BELGIAN);
                l.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE_BRAZILIAN);
                l.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_CAN);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_CANADIAN);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_SWISS);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_SWISS);
                l.Add(LANG_ITALIAN,    SUBLANG_ITALIAN_SWISS);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_CHILE);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_COLOMBIA);
                l.Add(LANG_CZECH,      SUBLANG_NEUTRAL);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN);
                l.Add(LANG_DANISH,     SUBLANG_NEUTRAL);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_MODERN);
                l.Add(LANG_FINNISH,    SUBLANG_NEUTRAL);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH);
                l.Add(LANG_GREEK,      SUBLANG_NEUTRAL);
                l.Add(LANG_HUNGARIAN,  SUBLANG_NEUTRAL);
                l.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_EIRE);
                l.Add(LANG_ITALIAN,    SUBLANG_ITALIAN);
                l.Add(LANG_FRENCH,     SUBLANG_FRENCH_LUXEMBOURG);
                l.Add(LANG_GERMAN,     SUBLANG_GERMAN_LUXEMBOURG);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_MEXICAN);
                l.Add(LANG_DUTCH,      SUBLANG_DUTCH);
                l.Add(LANG_NORWEGIAN,  SUBLANG_NEUTRAL);
                l.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_NZ);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_PERU);
                l.Add(LANG_POLISH,     SUBLANG_NEUTRAL);
                l.Add(LANG_PORTUGUESE, SUBLANG_PORTUGUESE);
                l.Add(LANG_SWEDISH,    SUBLANG_SWEDISH);
                l.Add(LANG_CHINESE,    SUBLANG_CHINESE_SINGAPORE);
                l.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_UK);
                l.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_US);
                l.Add(LANG_SPANISH,    SUBLANG_SPANISH_VENEZUELA);
                l.Add(LANG_AFRIKAANS,  SUBLANG_NEUTRAL);
                l.Add(LANG_ENGLISH,    SUBLANG_ENGLISH_SOUTH_AFRICA);
                GetOffsetSpecialDay := LanguagesInLanguages(c, l);
                ChristianHolidays(Languages);
              end;
          ChristmasDay2 :
            if Year >= FirstEaster Then
              begin
                Date := LocalJulianDay(Year, 12, 26.5);
                l.Add(LANG_GERMAN,    SUBLANG_GERMAN_AUSTRIAN);
                l.Add(LANG_ENGLISH,   SUBLANG_ENGLISH_AUS);
                l.Add(LANG_ENGLISH,   SUBLANG_ENGLISH_CAN);
                l.Add(LANG_FRENCH,    SUBLANG_FRENCH_CANADIAN);
                l.Add(LANG_GERMAN,    SUBLANG_GERMAN_SWISS);
                l.Add(LANG_FRENCH,    SUBLANG_FRENCH_SWISS);
                l.Add(LANG_ITALIAN,   SUBLANG_ITALIAN_SWISS);
                l.Add(LANG_FRENCH,    SUBLANG_FRENCH);
                l.Add(LANG_CZECH,     SUBLANG_NEUTRAL);
                l.Add(LANG_GERMAN,    SUBLANG_GERMAN);
                l.Add(LANG_DANISH,    SUBLANG_NEUTRAL);
                l.Add(LANG_FINNISH,   SUBLANG_NEUTRAL);
                l.Add(LANG_GREEK,     SUBLANG_NEUTRAL);
                l.Add(LANG_HUNGARIAN, SUBLANG_NEUTRAL);
                l.Add(LANG_ENGLISH,   SUBLANG_ENGLISH_EIRE);
                l.Add(LANG_ITALIAN,   SUBLANG_ITALIAN);
                l.Add(LANG_FRENCH,    SUBLANG_FRENCH_LUXEMBOURG);
                l.Add(LANG_GERMAN,    SUBLANG_GERMAN_LUXEMBOURG);
                l.Add(LANG_DUTCH,     SUBLANG_DUTCH);
                l.Add(LANG_NORWEGIAN, SUBLANG_NEUTRAL);
                l.Add(LANG_ENGLISH,   SUBLANG_ENGLISH_NZ);
                l.Add(LANG_POLISH,    SUBLANG_NEUTRAL);
                l.Add(LANG_SWEDISH,   SUBLANG_SWEDISH);
                l.Add(LANG_ENGLISH,   SUBLANG_ENGLISH_UK);
                l.Add(LANG_AFRIKAANS, SUBLANG_NEUTRAL);
                l.Add(LANG_ENGLISH,   SUBLANG_ENGLISH_SOUTH_AFRICA);
                GetOffsetSpecialDay := LanguagesInLanguages(c, l);
                ChristianHolidays(Languages);
              end;
        end;
      except
      end;
    c.Free;
    l.Free;
  end;

function JulianDayAfterChrist(jd : Double; y : LongInt) : tJulianDay;
  var    a, b : Integer;
  begin
    a := y div 100;
    b := 2 - a + a div 4;
    JulianDayAfterChrist := jd + b;
  end;

function JulianDay(Year : Integer; Month: Word; Day : double) : tJulianDay;
  var    y, m  : LongInt;
         jd    : double;
  begin
    if Year < 0 then
      inc(Year);
    if Month > 2 then
      begin
        m := Month;
        y := Year;
      end
    else
      begin
        m := Month + 12;
        y := Year  - 1;
      end;
    jd := y * 365 + y div 4 + 30 * (m + 1) + (6001 * (m + 1)) div 10000 + Day
          + 1720994.5;
    if y mod 4 < 0 then
      jd := jd - 1;
    if (Year > 1582) then
      JulianDay := JulianDayAfterChrist(jd, y)
    else
      if (Year = 1582) and (Month > 10) then
        JulianDay := JulianDayAfterChrist(jd, y)
      else
        if (Year = 1582) and (Month = 10) and (Day >= 15) then
          JulianDay := JulianDayAfterChrist(jd, y)
        else
          JulianDay := jd;
  end;

Procedure JulianDate(J         : Double;
                     var Year  : Integer;
                     var Month : Word;
                     var Day   : double);
  const   Monat10000    = 306001;
  var     Z, Alpha,
          A, B, C, D, E : LongInt;
          F             : double;
  begin
    F := J + 0.5;
    Z := trunc(F);
    F := F - Z;
    if Z < 2299161 then
      A := Z
    else
      begin
        Alpha := (4 * Z - 7468865) div 146097
                 { trunc((Z - 1867216.25) / 36524.25) };
        A     := Z + 1 + Alpha - Alpha div 4;
      end;
    B   := A + 1524;
    C   := (20 * B - 2442) div 7305
           { trunc((B - 122.1) / 365.25) };
    D   := (C * 1461) div 4
           { trunc(365.25 * C) };
    E   := ((B - D) * 10000) div Monat10000;
    Day := B - D - (E * Monat10000) div 10000 + F;
    if E <= 13 { < 13.5 } then
      Month := E - 1
    else
      Month := E - 13;
    If Month > 2 { > 2.5 } Then
      Year := C - 4716
    else
      Year := C - 4715;
    if Year <= 0 then
      Dec(Year);
  end;

function JulDayOfWeek(JD : Double) : Word;
  var    d : Integer;
  begin
    d := trunc(JD + 1.5) mod 7 + 1;
    if d <= 0 then
      JulDayOfWeek := d + 7
    else
      JulDayOfWeek := d;
  end;

function FirstWeek(Year : Integer) : Integer;
  begin
    if Week0(Year) + 7 > JulianDay(Year, 1, 1.5) then
      FirstWeek := 0
    else
      FirstWeek := 1;
  end;

function LastWeek(Year : Integer) : Integer;
  begin
    if Week0(Year) + (52 * 7 + 6) < JulianDay(Year, 12, 31.5) then
      LastWeek := 53
    else
      LastWeek := 52;
  end;

function Week0(Year : Integer) : tJulianDay;
  {
  Week0 returns monday of week 0.
  Since 1976 the week starts with monday.
  According to DIN 1355 (since 1974), ISO 8601 (since 1988),
  EN 28601 (since 1992) week 1 contains January 4th.
  January 4th is a                          Mo   Tu   We   Th   Fr   Sa   Su
  January 2nd is a                          Sa   Su   Mo   Tu   We   Th   Fr
  JulDayOfWeek(January 2nd) mod 7            0    1    2    3    4    5    6
  Week 1 starts on monday                  4th  3rd  2nd  1st 31st 30th 29th
  Week 0 starts 7 days before on December 28th 27th 26th 25th 24th 23rd 22nd
  Week 0 starts                              5    6    7    8    9   10   11
     days before January 2nd
  }
  var Date : tJulianDay;
  begin
    Date  := JulianDay(Year, 1, 2.5);
    Week0 := Date - JulDayOfWeek(Date) mod 7 - 5;
  end;

function WeekDayShift(Year : Integer) : Word;
  begin
    WeekDayShift := JulDayOfWeek(JulianDay(Year, 11, 18.5)) mod 7;
  end;

Function GetSpecialDay(SpecialDay    : tSpecialDay;
                       Year          : Integer;
                       var JD        : tJulianDay;
                       var Languages : tLanguageList) : Boolean;
  begin
    GetSpecialDay :=
      GetOffsetSpecialDay(SpecialDay, WeekDayShift(Year), Easter(Year),
                          Year, JD, Languages);
  end;

function tHolidaySort.MayContinue : Boolean;
  begin
    MayContinue := True;
  end;

function tHolidaySort.Compare(a, b : LongInt) : Integer;
  begin
   if SpecialDayList[a].JDay = SpecialDayList[b].JDay then
     Compare := 0
   else
     if SpecialDayList[a].JDay < SpecialDayList[b].JDay then
       Compare := -1
     else
       Compare :=  1;
  end;

procedure tHolidaySort.Swap(a, b : LongInt);
  begin
    SpecialDayList.Swap(a, b);
  end;

procedure CalcSpecialDays(Year : Integer; Languages : tLanguageList);
  var     WDShift     : Integer;
          EasterDate  : tJulianDay;
          i           : Word;
          Sort        : tSort;
          rSpecialDay : tRSpecialDay;
  begin
    if (SpecialDayList = Nil) or (LanguageList = Nil) then
      raise Exception.Create('SpecialDayList or LanguageList are not created')
    else
      if (Year <> SpecialDaysYear)
         or DifferentLanguages(Languages, LanguageList)
      then
        begin
          WDShift    := WeekDayShift(Year);
          EasterDate := Easter(Year);
          LanguageList.Assign(Languages);
          SpecialDayList.Clear;
          for i := 1 to ord(high(tSpecialDay)) do
            begin
              rSpecialDay.JDay := 0;
              try
                rSpecialDay.Languages := tLanguageList.Create;
              except
                rSpecialDay.Languages := Nil;
              end;
              if rSpecialDay.Languages <> Nil then
                begin
                  rSpecialDay.Languages.Assign(LanguageList);
                  rSpecialDay.IsHoliday :=
                    GetOffsetSpecialDay(tSpecialDay(i),
                                        WDShift,
                                        EasterDate,
                                        Year,
                                        rSpecialDay.JDay,
                                        rSpecialDay.Languages);
                  if rSpecialDay.JDay >= 0 then
                    begin
                      rSpecialDay.SpecialDay := tSpecialDay(i);
                      SpecialDayList.Add(rSpecialDay);
                    end
                  else
                    rSpecialDay.Languages.Free;
                end;
            end;
          try
            sort := tHolidaySort.Create;
          except
            sort := Nil;
          end;
          if Sort <> Nil then
            try
              Sort.Sort(0, SpecialDayList.Count - 1);
            finally
              Sort.Free;
            end;
        end;
    SpecialDaysYear := Year;
  end;

procedure GetHolidays(JD : tJulianDay; Languages : tLanguageList;
                      var l : tSpecialDayList);
  var     Day   : tJulianDay;
          i,
          Month : Word;
          Year  : Integer;
  begin
    l.Clear;
    if SpecialDayList <> Nil then
      begin
        Year  := 0;
        Month := 0;
        Day   := 0;
        JulianDate(JD, Year, Month, Day);
        CalcSpecialDays(Year, Languages);
        for i := 0 to SpecialDayList.Count - 1 do
          if SpecialDayList[i].JDay = JD Then
            l.Add(SpecialDayList[i]);
      end;
  end;

function JulianFormatDateTime(Format : String; Datum : Double) : String;
  var    Jahr       : Integer;
         T          : Double;
         Monat,
         Tag,
         Stunde,
         Minute,
         Sekunde,
         MSec,
         Wochentag  : Word;
         TimeIsNull : Boolean;

  function JulianFormat(Format : String) : String;
    const  am_pm     = 'am/pm';
           a_p       = 'a/p';
           ampm      = 'ampm';
    var    i         : Integer;
           Ausgabe,
           s         : String;
           M_is_Min,
           bampm     : Boolean;
           c         : Char;
    begin
      Ausgabe    := '';
      s          := '';
      bampm      := False;
      M_is_Min   := False;
      TimeIsNull := True;
      while Format > '' do
        begin
          if StripString(am_pm, Format) then
            begin
              s     := s + am_pm;
              bampm := True;
            end
          else
            if StripString(a_p, Format) then
              begin
                s     := s + a_p;
                bampm := True;
              end
            else
              if StripString(ampm, Format) then
                begin
                  s     := s + ampm;
                  bampm := True;
                end
              else
                case Format[1] of
                  '''',
                  '"' : begin
                          c      := Format[1];
                          s      := s + c;
                          Format := Copy(Format, 2, length(Format));
                          i      := Pos(c, Format);
                          if i = 0 then
                            i    := length(Format) + 1;
                          s      := s + Copy(Format, 1, i);
                          Format := Copy(Format, i + 1, length(Format));
                        end;
                else
                  i      := FormatLength(Format, Format[1]);
                  s      := s + Copy(Format, 1, i);
                  Format := Copy(Format, i + 1, length(Format));
                end;
        end;
      Format := s;
      while Format > '' do
        case UpperCase(Format[1])[1] of
          'C' : begin
                  Ausgabe := Ausgabe + JulianFormat(GetShortDateFormat);
                  s := JulianFormat(GetLongTimeFormat);
                  if not TimeIsNull then
                    Ausgabe := Ausgabe + ' ' + s;
                  M_is_Min  := False;
                  Format    := StripFormat(Format);
                end;
          'T' : begin
                  if FormatLength(Format, Format[1]) > 1 then
                    Ausgabe := Ausgabe + JulianFormat(GetLongTimeFormat)
                  else
                    Ausgabe := Ausgabe + JulianFormat(GetShortTimeFormat);
                  M_is_Min := False;
                  Format   := StripFormat(Format);
                end;
          'D' : begin
                  case FormatLength(Format, Format[1]) of
                    1 : Ausgabe := Ausgabe + IntToStr(Tag);
                    2 : Ausgabe := Ausgabe + IntToDez(Tag, 2, False);
                    3 : Ausgabe := Ausgabe + GetShortDayName(Wochentag);
                    4 : Ausgabe := Ausgabe + GetLongDayName(Wochentag);
                    5 : Ausgabe := Ausgabe + JulianFormat(GetShortDateFormat);
                  else
                    Ausgabe := Ausgabe + JulianFormat(GetLongDateFormat);
                  end;
                  M_is_Min := False;
                  Format   := StripFormat(Format);
                end;
          'M' : begin
                  if M_is_Min then
                    begin
                      if FormatLength(Format, Format[1]) > 1 then
                        Ausgabe := Ausgabe + IntToDez(Minute, 2, False)
                      else
                        Ausgabe := Ausgabe + IntToStr(Minute);
                      if Minute <> 0 then
                        TimeIsNull := False;
                    end
                  else
                    case FormatLength(Format, Format[1]) of
                      1 : Ausgabe := Ausgabe + IntToStr(Monat);
                      2 : Ausgabe := Ausgabe + IntToDez(Monat, 2, False);
                      3 : Ausgabe := Ausgabe + GetShortMonthName(Monat);
                    else
                      Ausgabe := Ausgabe + GetLongMonthName(Monat);
                    end;
                  M_is_Min := False;
                  Format   := StripFormat(Format);
                end;
          'Y' : begin
                  if Jahr < 0 then
                    Ausgabe := Ausgabe + YearText(Jahr)
                  else
                    if FormatLength(Format, Format[1]) > 2 then
                      Ausgabe := Ausgabe + IntToDez(Jahr, 4, False)
                    else
                      Ausgabe := Ausgabe + IntToDez(Jahr mod 100, 2, False);
                  M_is_Min := False;
                  Format   := StripFormat(Format);
                end;
          'H' : begin
                  if FormatLength(Format, Format[1]) > 1 then
                    if bampm then
                      if (Stunde = 0) or (Stunde = 12) then
                        Ausgabe := Ausgabe + '12'
                      else
                        Ausgabe := Ausgabe + IntToDez(Stunde mod 12, 2, False)
                    else
                      Ausgabe := Ausgabe + IntToDez(Stunde, 2, False)
                  else
                    if bampm then
                      if (Stunde = 0) or (Stunde = 12) then
                        Ausgabe := Ausgabe + '12'
                      else
                        Ausgabe := Ausgabe + IntToStr(Stunde mod 12)
                    else
                      Ausgabe := Ausgabe + IntToStr(Stunde);
                  if Stunde <> 0 then
                    TimeIsNull := False;
                  M_is_Min    := True;
                  Format      := StripFormat(Format);
                end;
          'N' : begin
                  if FormatLength(Format, Format[1]) > 1 then
                    Ausgabe := Ausgabe + IntToDez(Minute, 2, False)
                  else
                    Ausgabe := Ausgabe + IntToStr(Minute);
                  if Minute <> 0 then
                    TimeIsNull := False;
                  M_is_Min  := False;
                  Format    := StripFormat(Format);
                end;
          'S' : begin
                  if FormatLength(Format, Format[1]) > 1 then
                    Ausgabe := Ausgabe + IntToDez(Sekunde, 2, False)
                  else
                    Ausgabe := Ausgabe + IntToStr(Sekunde);
                  if Sekunde <> 0 then
                    TimeIsNull := False;
                  M_is_Min  := False;
                  Format    := StripFormat(Format);
                end;
          'Z' : begin
                  i := FormatLength(Format, Format[1]);
                  if i > 3 then
                    i := 3;
                  {$ifdef fpc}
                    if i > 1 then
                      i := 3;
                  {$endif}
                  Ausgabe := Ausgabe + IntToDez(MSec, i, False);
                  if MSec <> 0 then
                    TimeIsNull := False;
                  M_is_Min     := False;
                  Format       := StripFormat(Format);
                end;
          'A' : begin
                  if StripString(am_pm, Format) then
                    if Stunde >= 12 then
                      Ausgabe := Ausgabe + 'pm'
                    else
                      Ausgabe := Ausgabe + 'am'
                  else
                    if StripString(a_p, Format) then
                      if Stunde >= 12 then
                        Ausgabe := Ausgabe + 'p'
                      else
                        Ausgabe := Ausgabe + 'a'
                    else
                      if StripString(ampm, Format) then
                        if Stunde >= 12 then
                          Ausgabe := Ausgabe + GetTimePMString
                        else
                          Ausgabe := Ausgabe + GetTimeAMString
                      else
                        begin
                          Ausgabe := Ausgabe + Format[1];
                          Format  := Copy(Format, 2, length(Format));
                        end;
                end;
          '/' : begin
                  Ausgabe := Ausgabe + GetDateSeparator;
                  Format  := Copy(Format, 2, length(Format));
                end;
          ':' : begin
                  Ausgabe := Ausgabe + GetTimeSeparator;
                  Format  := Copy(Format, 2, length(Format));
                end;
          '''',
          '"' : begin
                  c       := Format[1];
                  Format  := Copy(Format, 2, length(Format));
                  i       := Pos(c, Format);
                  if i = 0 then
                    i     := length(Format) + 1;
                  Ausgabe := Ausgabe + Copy(Format, 1, i - 1);
                  Format  := Copy(Format, i + 1, length(Format));
                end;
        else
          Ausgabe := Ausgabe + Format[1];
          Format  := Copy(Format, 2, length(Format));
        end;
      JulianFormat := Ausgabe;
    end;

  begin
    Datum      := Datum + 0.5 / 24 / 60 / 60 / 1000;
    Jahr       := 0;
    Monat      := 0;
    T          := 0;
    JulianDate(Datum, Jahr, Monat, T);
    Tag        := Trunc(t);
    t          := Frac(t) * 24;
    Stunde     := Trunc(t);
    t          := Frac(t) * 60;
    Minute     := Trunc(t);
    t          := Frac(t) * 60;
    Sekunde    := Trunc(t);
    MSec       := Trunc(Frac(t) * 1000);
    Wochentag  := JulDayOfWeek(Datum);
    TimeIsNull := True;
    if Trim(Format) = '' then
      Format   := 'c';
    JulianFormatDateTime := JulianFormat(Format);
  end;

procedure SwapLong(var l : LongInt);
  type    tLongBytes = array[0 .. 3] of Byte;
          pLongBytes = ^tLongBytes;
  var     pEingabe,
          pAusgabe   : pLongBytes;
          Ausgabe    : LongInt;
  begin
    pEingabe     := @l;
    pAusgabe     := @Ausgabe;
    pAusgabe^[0] := pEingabe^[3];
    pAusgabe^[1] := pEingabe^[2];
    pAusgabe^[2] := pEingabe^[1];
    pAusgabe^[3] := pEingabe^[0];
    l            := Ausgabe;
  end;

{$ifdef Unix}
  procedure SearchTimeZoneInformation(Year : Integer;
                                      var DaylightDate,
                                          StandardDate,
                                          DoubleSummerInfo,
                                          Daylight2Info     : tSystemTime);
    var    lTimeVon,
           lTimeBis  : time_t;
           t         : tDateTime;
           i, j      : Integer;
    begin
      FillChar(DaylightDate,     sizeof(DaylightDate),     0);
      FillChar(StandardDate,     sizeof(StandardDate),     0);
      FillChar(DoubleSummerInfo, sizeof(DoubleSummerInfo), 0);
      FillChar(Daylight2Info,    sizeof(Daylight2Info),    0);
      if (tzhead <> Nil) and (transitions <> Nil) and
         (1916 <= Year) and (Year <= 2037)
      then
        begin
          lTimeVon := Trunc(EncodeDate(Year,  1,  2) - EncodeDate(1970, 1, 1))
                      * 24 * 60 * 60;
          lTimeBis := Trunc(EncodeDate(Year, 12, 31) - EncodeDate(1970, 1, 1))
                      * 24 * 60 * 60;
          j        := 0;
          i        := 0;
          while i < tzhead^.tzh_timecnt do
            begin
              {$R-}
                if transitions^[i] > lTimeBis then
                  i := tzhead^.tzh_timecnt
                else
                  if transitions^[i] > lTimeVon then
                    begin
                      t := EncodeDate(1970, 1, 1)
                           + transitions^[i] / 60 / 60 / 24;
                      if (ttinfoind <> Nil) and (ttinfo <> Nil) then
                        if ttinfoind^[i] < tzhead^.tzh_typecnt then
                          begin
                            if i - 1 >= 0 then
                              begin
                                if ttinfoind^[i - 1] < tzhead^.tzh_typecnt
                                then
                                  t := t + ttinfo^[ttinfoind^[i - 1]].
                                             tt_gmtoff / 60 / 60 / 24;
                              end
                            else
                              if i + 1 < tzhead^.tzh_timecnt then
                                begin
                                  if ttinfoind^[i + 1] < tzhead^.tzh_typecnt
                                  then
                                    t := t + ttinfo^[ttinfoind^[i + 1]].
                                               tt_gmtoff / 60 / 60 / 24;
                                end;
                            if ttinfo^[ttinfoind^[i]].tt_isdst then
                              begin
                                inc(j);
                                case j of
                                  1: begin
                                       DecodeDate(t,
                                         DaylightDate.wYear,
                                         DaylightDate.wMonth,
                                         DaylightDate.wDay);
                                       DecodeTime(t,
                                         DaylightDate.wHour,
                                         DaylightDate.wMinute,
                                         DaylightDate.wSecond,
                                         DaylightDate.wMilliseconds);
                                     end;
                                  2: begin
                                       DecodeDate(t,
                                         DoubleSummerInfo.wYear,
                                         DoubleSummerInfo.wMonth,
                                         DoubleSummerInfo.wDay);
                                       DecodeTime(t,
                                         DoubleSummerInfo.wHour,
                                         DoubleSummerInfo.wMinute,
                                         DoubleSummerInfo.wSecond,
                                         DoubleSummerInfo.wMilliseconds);
                                       SavedDoubleSummerBias    :=
                                         - ttinfo^[ttinfoind^[i]].tt_gmtoff
                                           div 60;
                                     end;
                                  3: begin
                                       DecodeDate(t,
                                         Daylight2Info.wYear,
                                         Daylight2Info.wMonth,
                                         Daylight2Info.wDay);
                                       DecodeTime(t,
                                         Daylight2Info.wHour,
                                         Daylight2Info.wMinute,
                                         Daylight2Info.wSecond,
                                         Daylight2Info.wMilliseconds);
                                     end;
                                end;
                              end
                            else
                              begin
                                DecodeDate(t,
                                  StandardDate.wYear,
                                  StandardDate.wMonth,
                                  StandardDate.wDay);
                                DecodeTime(t,
                                  StandardDate.wHour,
                                  StandardDate.wMinute,
                                  StandardDate.wSecond,
                                  StandardDate.wMilliseconds);
                              end;
                          end;
                    end;
              {$R+}
              inc(i);
            end;
        end;
    end;

  function GetTimeZoneInformation(var TimeZoneInformation :
                                  TTimeZoneInformation): DWORD;
    var    pTime        : ptm;
           sTime        : tm;
           sTimeVal     : tTimeVal;
           sTimeZone    : tTimeZone;
           DaylightDate,
           StandardDate,
           DoubleSummerInfo,
           Daylight2Info : tSystemTime;
           i            : Integer;
           bDaylight,
           bStandard    : Boolean;
    begin
      GetTimeZoneInformation := TIME_ZONE_ID_UNKNOWN;
      TimeZoneInformation    := SavedTimeZoneInfo;
      sTime.tm_year          := 0;
      sTimeVal.tv_sec        := 0;
      sTimeZone.tz_dsttime   := 0;
      DoubleSummerInfo.wYear := 0;
      Daylight2Info.wYear    := 0;
      fpGetTimeOfDay(@sTimeVal, @sTimeZone);
      pTime := localtime(sTimeVal.tv_sec);
      if pTime <> Nil then
        begin
          SearchTimeZoneInformation(pTime^.tm_year + 1900,
                                    TimeZoneInformation.DaylightDate,
                                    TimeZoneInformation.StandardDate,
                                    DoubleSummerInfo,
                                    Daylight2Info);
          if TimeZoneInformation.DaylightDate.wYear = 0 then
            bDaylight := False
          else
            begin
              FillChar(sTime, sizeof(sTime), 0);
              sTime.tm_year  := TimeZoneInformation.DaylightDate.wYear  - 1900;
              sTime.tm_mon   := TimeZoneInformation.DaylightDate.wMonth -    1;
              sTime.tm_mday  := TimeZoneInformation.DaylightDate.wDay   +    7;
              sTime.tm_hour  := 12;
              sTime.tm_isdst := -1;
              mktime(@sTime);
              bDaylight :=
                sTime.tm_mon <> TimeZoneInformation.DaylightDate.wMonth - 1;
              TimeZoneInformation.DaylightDate.wYear := 0;
              TimeZoneInformation.DaylightDate.wDay  :=
                (TimeZoneInformation.DaylightDate.wDay - 1) div 7 + 1;
              if TimeZoneInformation.DaylightDate.wDay = 5 then
                bDaylight := False;
            end;
          if TimeZoneInformation.StandardDate.wYear = 0 then
            bStandard := False
          else
            begin
              FillChar(sTime, sizeof(sTime), 0);
              sTime.tm_year  := TimeZoneInformation.StandardDate.wYear  - 1900;
              sTime.tm_mon   := TimeZoneInformation.StandardDate.wMonth -    1;
              sTime.tm_mday  := TimeZoneInformation.StandardDate.wDay   +    7;
              sTime.tm_hour  := 12;
              sTime.tm_isdst := -1;
              mktime(@sTime);
              bStandard :=
                sTime.tm_mon <> TimeZoneInformation.StandardDate.wMonth - 1;
              TimeZoneInformation.StandardDate.wYear := 0;
              TimeZoneInformation.StandardDate.wDay  :=
                (TimeZoneInformation.StandardDate.wDay - 1) div 7 + 1;
              if TimeZoneInformation.StandardDate.wDay = 5 then
                bStandard := False;
            end;
          DaylightDate.wYear := 0;
          StandardDate.wYear := 0;
          i                  := 1;
          while bStandard or bDaylight do
            begin
              if i > 10 then
                begin
                  bStandard := False;
                  bDaylight := False;
                end
              else
                if pTime^.tm_year - i < 70 then
                  begin
                    bStandard := False;
                    bDaylight := False;
                  end
                else
                  begin
                    SearchTimeZoneInformation(pTime^.tm_year + 1900 - i,
                                              DaylightDate,
                                              StandardDate,
                                              DoubleSummerInfo,
                                              Daylight2Info);
                    if bDaylight then
                      begin
                        FillChar(sTime, sizeof(sTime), 0);
                        sTime.tm_year  := DaylightDate.wYear  - 1900;
                        sTime.tm_mon   := DaylightDate.wMonth -    1;
                        sTime.tm_mday  := DaylightDate.wDay   +    7;
                        sTime.tm_hour  := 12;
                        sTime.tm_isdst := -1;
                        mktime(@sTime);
                        if sTime.tm_mon = DaylightDate.wMonth - 1 then
                          bDaylight := False
                        else
                          begin
                            DaylightDate.wDay :=
                              (DaylightDate.wDay - 1) div 7 + 1;
                            if DaylightDate.wDay >
                               TimeZoneInformation.DaylightDate.wDay
                            then
                              begin
                                TimeZoneInformation.DaylightDate.wDay :=
                                  DaylightDate.wDay;
                                bDaylight := False;
                              end;
                          end;
                      end;
                    if bStandard then
                      begin
                        FillChar(sTime, sizeof(sTime), 0);
                        sTime.tm_year  := StandardDate.wYear  - 1900;
                        sTime.tm_mon   := StandardDate.wMonth -    1;
                        sTime.tm_mday  := StandardDate.wDay   +    7;
                        sTime.tm_hour  := 12;
                        sTime.tm_isdst := -1;
                        mktime(@sTime);
                        if sTime.tm_mon = StandardDate.wMonth - 1 then
                          bStandard := False
                        else
                          begin
                            StandardDate.wDay :=
                              (StandardDate.wDay - 1) div 7 + 1;
                            if StandardDate.wDay >
                               TimeZoneInformation.StandardDate.wDay
                            then
                              begin
                                TimeZoneInformation.StandardDate.wDay :=
                                  StandardDate.wDay;
                                bStandard := False;
                              end;
                          end;
                      end;
                    inc(i);
                  end;
            end;
        end;
      if pTime <> Nil then
        if pTime^.tm_isdst = 0 then
          pTime := Nil;
      if pTime = Nil then
        begin
          if SavedTimeZoneInfo.DaylightBias <> 0 then
            GetTimeZoneInformation := TIME_ZONE_ID_STANDARD;
        end
      else
        GetTimeZoneInformation := TIME_ZONE_ID_DAYLIGHT;
    end;
{$endif}

{$ifdef Ver80}
  function WideCharToString(Source: TimeZoneName): string;
    var    s : String;
           i : LongInt;
    begin
      s := '';
      i := 0;
      while i <= High(Source) do
        if Source[i] = 0 then
          i := High(Source) + 1
        else
          begin
            if Source[i] > $ff then
              s := s + '#'
            else
              s := s + chr(Source[i]);
            inc(i);
          end;
      WideCharToString := s;
    end;

  function SystemTimeToDateTime(Year       : Word;
                                SystemTime : tSystemTime) : tDateTime;
    var    t      : tDateTime;
           Month,
           Day    : Word;
    begin
      if (SystemTime.wMonth <= 0) or
         (SystemTime.wMonth > 12) or
         (SystemTime.wDay   <= 0) or
         (SystemTime.wDay   > 31)
      then
        t := 0
      else
        if SystemTime.wYear <> 0 then
          try
            t := EncodeDate(SystemTime.wYear,
                            SystemTime.wMonth,
                            SystemTime.wDay)
                 + EncodeTime(SystemTime.wHour,
                              SystemTime.wMinute,
                              SystemTime.wSecond,
                              SystemTime.wMilliseconds);
          except
            t := 0;
          end
        else
          try
            { tSystemTime.wDayOfWeek: Sun .. Sat = 0 .. 6
              DayOfWeek:              Sun .. Sat = 1 .. 7 }
            t := EncodeDate(Year, SystemTime.wMonth, 1) - 1
                 + EncodeTime(SystemTime.wHour,
                              SystemTime.wMinute,
                              SystemTime.wSecond,
                              SystemTime.wMilliseconds);
            t := t + (SystemTime.wDayOfWeek + 8 - DayOfWeek(t)) mod 7
                   + (SystemTime.wDay - 1) * 7;
            DecodeDate(t, Year, Month, Day);
            while Month > SystemTime.wMonth do
              begin
                t := t - 7;
                DecodeDate(t, Year, Month, Day);
              end;
          except
            t := 0;
          end;
      SystemTimeToDateTime := t;
    end;

  function GetTimeZoneInformation(var TimeZoneInformation :
                                  TTimeZoneInformation): DWORD;
    var    Jetzt  : tDateTime;
           Year,
           Month,
           Day    : Word;
    begin
      GetTimeZoneInformation := TIME_ZONE_ID_UNKNOWN;
      TimeZoneInformation    := SavedTimeZoneInfo;
      if     (TimeZoneInformation.DaylightDate.wMonth <> 0)
         and (TimeZoneInformation.DaylightDate.wDay   <> 0)
         and (TimeZoneInformation.StandardDate.wMonth <> 0)
         and (TimeZoneInformation.StandardDate.wDay   <> 0)
      then
        begin
          Jetzt := Now;
          DecodeDate(Jetzt, Year, Month, Day);
          if (SystemTimeToDateTime(Year, TimeZoneInformation.DayLightDate)
              < Jetzt) and (Jetzt <
              SystemTimeToDateTime(Year, TimeZoneInformation.StandardDate))
          then
            GetTimeZoneInformation := TIME_ZONE_ID_DAYLIGHT
          else
            GetTimeZoneInformation := TIME_ZONE_ID_STANDARD;
        end;
    end;
{$endif}

procedure AT_HU_DE_Daylight(Year             : Integer;
                            var TimeZoneInfo : TTimeZoneInformation);
  begin
    { http://de.wikipedia.org/wiki/Sommerzeit
      and Linux tzfile
    }
    case Year of
      1916:
        begin
          TimeZoneInfo.DaylightDate.wYear  := Year;
          TimeZoneInfo.DaylightDate.wMonth :=  4;
          TimeZoneInfo.DaylightDate.wDay   := 30;
          TimeZoneInfo.DaylightDate.wHour  := 23;
          TimeZoneInfo.StandardDate.wYear  := Year;
          TimeZoneInfo.StandardDate.wMonth := 10;
          TimeZoneInfo.StandardDate.wDay   :=  1;
          TimeZoneInfo.StandardDate.wHour  :=  1;
        end;
      1917 .. 1920:
        begin
          TimeZoneInfo.DaylightDate.wMonth     := 4;
          TimeZoneInfo.DaylightDate.wDay       := 3;
          TimeZoneInfo.DaylightDate.wDayOfWeek := 1;
          TimeZoneInfo.DaylightDate.wHour      := 2;
          TimeZoneInfo.StandardDate.wMonth     := 9;
          TimeZoneInfo.StandardDate.wDay       := 3;
          TimeZoneInfo.StandardDate.wDayOfWeek := 1;
          TimeZoneInfo.StandardDate.wHour      := 3;
        end;
    end;
  end;

procedure AT_DE_Daylight(Year                  : Integer;
                         var TimeZoneInfo      : TTimeZoneInformation;
                         var DoubleSummerInfo,
                             Daylight2Info     : tSystemTime);
  begin
    { http://de.wikipedia.org/wiki/Sommerzeit
      and Linux tzfile
    }
    case Year of
      1940:
        begin
          TimeZoneInfo.DaylightDate.wYear  := Year;
          TimeZoneInfo.DaylightDate.wMonth := 4;
          TimeZoneInfo.DaylightDate.wDay   := 1;
          TimeZoneInfo.DaylightDate.wHour  := 2;
          TimeZoneInfo.StandardDate.wYear  := Year + 1;
          TimeZoneInfo.StandardDate.wMonth := 1;
          TimeZoneInfo.StandardDate.wDay   := 2;
        end;
      1941:
        begin
          TimeZoneInfo.DaylightDate.wYear  := Year - 1;
          TimeZoneInfo.DaylightDate.wMonth := 12;
          TimeZoneInfo.DaylightDate.wDay   := 31;
          TimeZoneInfo.StandardDate.wYear  := Year + 1;
          TimeZoneInfo.StandardDate.wMonth :=  1;
          TimeZoneInfo.StandardDate.wDay   :=  2;
        end;
      1942:
        begin
          TimeZoneInfo.DaylightDate.wYear   := Year - 1;
          TimeZoneInfo.DaylightDate.wMonth  := 12;
          TimeZoneInfo.DaylightDate.wDay    := 31;
          TimeZoneInfo.StandardDate.wYear   := Year;
          TimeZoneInfo.StandardDate.wMonth  := 11;
          TimeZoneInfo.StandardDate.wDay    :=  2;
          TimeZoneInfo.StandardDate.wHour   :=  3;
        end;
      1943:
        begin
          TimeZoneInfo.DaylightDate.wYear   := Year;
          TimeZoneInfo.DaylightDate.wMonth  :=  3;
          TimeZoneInfo.DaylightDate.wDay    := 29;
          TimeZoneInfo.DaylightDate.wHour   :=  2;
          TimeZoneInfo.StandardDate.wYear   := Year;
          TimeZoneInfo.StandardDate.wMonth  := 10;
          TimeZoneInfo.StandardDate.wDay    :=  4;
          TimeZoneInfo.StandardDate.wHour   :=  3;
        end;
      1944:
        begin
          TimeZoneInfo.DaylightDate.wYear   := Year;
          TimeZoneInfo.DaylightDate.wMonth  :=  4;
          TimeZoneInfo.DaylightDate.wDay    :=  3;
          TimeZoneInfo.DaylightDate.wHour   :=  2;
          TimeZoneInfo.StandardDate.wYear   := Year;
          TimeZoneInfo.StandardDate.wMonth  := 10;
          TimeZoneInfo.StandardDate.wDay    :=  2;
          TimeZoneInfo.StandardDate.wHour   :=  3;
        end;
      1945:
        begin
          TimeZoneInfo.DaylightDate.wYear   := Year;
          TimeZoneInfo.DaylightDate.wMonth  :=  4;
          TimeZoneInfo.DaylightDate.wDay    :=  2;
          TimeZoneInfo.DaylightDate.wHour   :=  2;
          { Western zone
          TimeZoneInfo.StandardDate.wYear   := Year;
          TimeZoneInfo.StandardDate.wMonth  :=  9;
          TimeZoneInfo.StandardDate.wDay    := 16;
          TimeZoneInfo.StandardDate.wHour   :=  2;
          }
          { Soviet zone }
          TimeZoneInfo.StandardDate.wYear   := Year;
          TimeZoneInfo.StandardDate.wMonth  := 11;
          TimeZoneInfo.StandardDate.wDay    := 18;
          TimeZoneInfo.StandardDate.wHour   :=  3;
          { only Soviet zone }
          DoubleSummerInfo.wYear  := Year;
          DoubleSummerInfo.wMonth :=  5;
          DoubleSummerInfo.wDay   := 24;
          DoubleSummerInfo.wHour  :=  2;
          Daylight2Info.wYear     := Year;
          Daylight2Info.wMonth    :=  9;
          Daylight2Info.wDay      := 24;
          Daylight2Info.wHour     :=  3;
        end;
      1946:
        begin
          TimeZoneInfo.DaylightDate.wYear   := Year;
          TimeZoneInfo.DaylightDate.wMonth  :=  4;
          TimeZoneInfo.DaylightDate.wDay    := 14;
          TimeZoneInfo.DaylightDate.wHour   :=  2;
          TimeZoneInfo.StandardDate.wYear   := Year;
          TimeZoneInfo.StandardDate.wMonth  := 10;
          TimeZoneInfo.StandardDate.wDay    :=  7;
          TimeZoneInfo.StandardDate.wHour   :=  3;
        end;
      1947:
        begin
          TimeZoneInfo.DaylightDate.wYear   := Year;
          TimeZoneInfo.DaylightDate.wMonth  :=  4;
          TimeZoneInfo.DaylightDate.wDay    :=  6;
          TimeZoneInfo.DaylightDate.wHour   :=  3;
          TimeZoneInfo.StandardDate.wYear   := Year;
          TimeZoneInfo.StandardDate.wMonth  := 10;
          TimeZoneInfo.StandardDate.wDay    :=  5;
          TimeZoneInfo.StandardDate.wHour   :=  3;
          DoubleSummerInfo.wYear            := Year;
          DoubleSummerInfo.wMonth           :=  5;
          DoubleSummerInfo.wDay             := 11;
          DoubleSummerInfo.wHour            :=  3;
          Daylight2Info.wYear               := Year;
          Daylight2Info.wMonth              :=  6;
          Daylight2Info.wDay                := 29;
          Daylight2Info.wHour               :=  3;
        end;
      1948:
        begin
          TimeZoneInfo.DaylightDate.wYear   := Year;
          TimeZoneInfo.DaylightDate.wMonth  :=  4;
          TimeZoneInfo.DaylightDate.wDay    := 18;
          TimeZoneInfo.DaylightDate.wHour   :=  2;
          TimeZoneInfo.StandardDate.wYear   := Year;
          TimeZoneInfo.StandardDate.wMonth  := 10;
          TimeZoneInfo.StandardDate.wDay    :=  3;
          TimeZoneInfo.StandardDate.wHour   :=  3;
        end;
      1949:
        begin
          TimeZoneInfo.DaylightDate.wYear   := Year;
          TimeZoneInfo.DaylightDate.wMonth  :=  4;
          TimeZoneInfo.DaylightDate.wDay    := 10;
          TimeZoneInfo.DaylightDate.wHour   :=  2;
          TimeZoneInfo.StandardDate.wYear   := Year;
          TimeZoneInfo.StandardDate.wMonth  := 10;
          TimeZoneInfo.StandardDate.wDay    :=  2;
          TimeZoneInfo.StandardDate.wHour   :=  3;
        end;
    end;
  end;

function SystemTimeToJulianDay(Year : Integer; SystemTime : tSystemTime) :
           Double;
  var    Month : Word;
         Day,
         d     : Double;
  begin
    if (SystemTime.wMonth <= 0) or
       (SystemTime.wMonth > 12) or
       (SystemTime.wDay   <= 0) or
       (SystemTime.wDay   > 31)
    then
      d := 0
    else
      if SystemTime.wYear <> 0 then
        try
          d := JulianDay(SystemTime.wYear,
                         SystemTime.wMonth,
                         SystemTime.wDay
                         + SystemTime.wHour         / 24
                         + SystemTime.wMinute       / 24 / 60
                         + SystemTime.wSecond       / 24 / 60 / 60
                         + SystemTime.wMilliseconds / 24 / 60 / 60 / 1000);
        except
          d := 0;
        end
      else
        try
          Day   := 1;
          Month := 1;
          { tSystemTime.wDayOfWeek: Sun .. Sat = 0 .. 6
            JulDayOfWeek:           Sun .. Sat = 1 .. 7 }
          d := JulianDay(Year,
                         SystemTime.wMonth,
                         SystemTime.wHour           / 24
                         + SystemTime.wMinute       / 24 / 60
                         + SystemTime.wSecond       / 24 / 60 / 60
                         + SystemTime.wMilliseconds / 24 / 60 / 60 / 1000);
          d := d + (SystemTime.wDayOfWeek + 8 - JulDayOfWeek(d)) mod 7
                 + (SystemTime.wDay - 1) * 7;
          JulianDate(d, Year, Month, Day);
          while Month > SystemTime.wMonth do
            begin
              d := d - 7;
              JulianDate(d, Year, Month, Day);
            end;
        except
          d := 0;
        end;
    SystemTimeToJulianDay := d;
  end;

procedure GetDayLightDates(Year                  : Integer;
                           var DaylightDate1,
                               DoubleSummerDate,
                               DaylightDate2,
                               StandardDate      : tJulianDay;
                           var DaylightBias,
                               DoubleSummerBias,
                               StandardBias      : Double);
  var     UseTZInfo        : Boolean;
          DoubleSummerInfo,
          Daylight2Info    : tSystemTime;
          TimeZoneInfo     : tTimeZoneInformation;
  begin
    TimeZoneInfo := SavedTimeZoneInfo;
    if Year = DaylightYear then
      begin
        DaylightDate1    := SavedDaylightDate1;
        DoubleSummerDate := SavedDoubleSummerDate;
        DaylightDate2    := SavedDaylightDate2;
        StandardDate     := SavedStandardDate;
        DaylightBias     := (TimeZoneInfo.Bias
                             + TimeZoneInfo.DaylightBias) / 60 / 24;
        StandardBias     := (TimeZoneInfo.Bias
                             + TimeZoneInfo.StandardBias) / 60 / 24;
        DoubleSummerBias := SavedDoubleSummerBias / 60 / 24;
      end
    else
      begin
        DayLightDate1          := 0;
        DoubleSummerDate       := 0;
        DayLightDate2          := 0;
        StandardDate           := 0;
        DoubleSummerInfo.wYear := 0;
        Daylight2Info.wYear    := 0;
        {$ifdef Unix}
          UseTZInfo := True;
          SearchTimeZoneInformation(Year,
                                    TimeZoneInfo.DaylightDate,
                                    TimeZoneInfo.StandardDate,
                                    DoubleSummerInfo,
                                    Daylight2Info);
        {$else}
          UseTZInfo := Year > 1995;
        {$endif}
        if not UseTZInfo then
          begin
            FillChar(TimeZoneInfo,     sizeof(TimeZoneInfo),     0);
            FillChar(DoubleSummerInfo, sizeof(DoubleSummerInfo), 0);
            FillChar(Daylight2Info,    sizeof(Daylight2Info),    0);
            TimeZoneInfo.Bias         := SavedTimeZoneInfo.Bias;
            TimeZoneInfo.StandardBias := SavedTimeZoneInfo.StandardBias;
            TimeZoneInfo.DaylightBias := SavedTimeZoneInfo.DaylightBias;
            { http://de.wikipedia.org/wiki/Sommerzeit
              and Linux tzfile
            }
            case SysLocale.PriLangID of
              Lang_German:
                case SysLocale.SubLangID of
                  SubLang_German:
                    case Year of
                      1916 .. 1918:
                        AT_HU_DE_Daylight(Year, TimeZoneInfo);
                      1940 .. 1949:
                        AT_DE_Daylight(Year, TimeZoneInfo,
                                             DoubleSummerInfo,
                                             Daylight2Info);
                      1980:
                        begin
                          TimeZoneInfo.DaylightDate.wYear   := Year;
                          TimeZoneInfo.DaylightDate.wMonth  :=  4;
                          TimeZoneInfo.DaylightDate.wDay    :=  6;
                          TimeZoneInfo.DaylightDate.wHour   :=  2;
                          TimeZoneInfo.StandardDate.wYear   := Year;
                          TimeZoneInfo.StandardDate.wMonth  :=  9;
                          TimeZoneInfo.StandardDate.wDay    := 28;
                          TimeZoneInfo.StandardDate.wHour   :=  3;
                        end;
                      1981 .. 1995:
                        begin
                          TimeZoneInfo.DaylightDate.wMonth := 3;
                          TimeZoneInfo.DaylightDate.wDay   := 5;
                          TimeZoneInfo.DaylightDate.wHour  := 2;
                          TimeZoneInfo.StandardDate.wMonth := 9;
                          TimeZoneInfo.StandardDate.wDay   := 5;
                          TimeZoneInfo.StandardDate.wHour  := 3;
                        end;
                    end;
                  SubLang_German_Austrian:
                    case Year of
                      1916 .. 1920:
                        AT_HU_DE_Daylight(Year, TimeZoneInfo);
                      1940 .. 1948:
                        AT_DE_Daylight(Year, TimeZoneInfo,
                                             DoubleSummerInfo,
                                             Daylight2Info);
                    end;
                end;
              Lang_Hungarian:
                case Year of
                  1916 .. 1919:
                    AT_HU_DE_Daylight(Year, TimeZoneInfo);
                end;
            end;
          end;
        StandardDate     :=
          SystemTimeToJulianDay(Year, TimeZoneInfo.StandardDate);
        DayLightDate1    :=
          SystemTimeToJulianDay(Year, TimeZoneInfo.DayLightDate);
        DayLightDate2    :=
          SystemTimeToJulianDay(Year, Daylight2Info);
        DoubleSummerDate :=
          SystemTimeToJulianDay(Year, DoubleSummerInfo);
        DaylightBias          := (TimeZoneInfo.Bias
                                  + TimeZoneInfo.DaylightBias) / 60 / 24;
        StandardBias          := (TimeZoneInfo.Bias
                                  + TimeZoneInfo.StandardBias) / 60 / 24;
        DoubleSummerBias      := SavedDoubleSummerBias / 60 / 24;

        DaylightDate1         := DaylightDate1    + StandardBias;
        DoubleSummerDate      := DoubleSummerDate + DaylightBias;
        DaylightDate2         := DaylightDate2    + DaylightBias;
        StandardDate          := StandardDate     + StandardBias;
        SavedDaylightDate1    := DaylightDate1;
        SavedDoubleSummerDate := DoubleSummerDate;
        SavedDaylightDate2    := DaylightDate2;
        SavedStandardDate     := StandardDate;
        DaylightYear          := Year;
      end;
  end;

function  DateTimeToJulianDay(DateTime : tDateTime) : tJulianDay;
  var     Year, Month, Day,
          Hour, Min,   Sec,
          MSec              : Word;
  begin
    DecodeDate(DateTime, Year, Month, Day);
    DecodeTime(DateTime, Hour, Min,   Sec, MSec);
    DateTimeToJulianDay :=
      LocalJulianDay(Year, Month, Day
                                  + Hour / 24
                                  + Min  / 24 / 60
                                  + Sec  / 24 / 60 / 60
                                  + MSec / 24 / 60 / 60 / 1000);
  end;

function  JulianDayToDateTime(JulianDay : tJulianDay) : tDateTime;
  var     Year  : Integer;
          Month,
          Day,
          Hour,
          Min,
          Sec,
          MSec  : Word;
          d     : Double;
  begin
    Year  := 0;
    Month := 0;
    d     := 0;
    LocalJulianDate(JulianDay, Year, Month, d);
    Day   := Trunc(d);
    d     := Frac(d) * 24;
    Hour  := Trunc(d);
    d     := Frac(d) * 60;
    Min   := Trunc(d);
    d     := Frac(d) * 60;
    Sec   := Trunc(d);
    MSec  := Trunc(Frac(d) * 1000);
    d     := EncodeDate(Year, Month, Day);
    if d < 0 then
      JulianDayToDateTime := d - EncodeTime(Hour, Min, Sec, MSec)
    else
      JulianDayToDateTime := d + EncodeTime(Hour, Min, Sec, MSec);
  end;

function GMTDayToLocalDay(JD,
                          DaylightDate1,
                          DoubleSummerDate,
                          DaylightDate2,
                          StandardDate     : tJulianDay;
                          DaylightBias,
                          DoubleSummerBias,
                          StandardBias     : Double) : Double;
  begin
    if (DayLightDate1 < JD) and
       (JD < StandardDate - StandardBias + DaylightBias)
    then
      if (DoubleSummerDate < JD) and
         (JD < DaylightDate2 - DaylightBias + DoubleSummerBias)
      then
        GMTDayToLocalDay := JD - DoubleSummerBias
      else
        GMTDayToLocalDay := JD - DaylightBias
    else
      GMTDayToLocalDay := JD - StandardBias;
  end;

function LocalDayToGMTDay(Day               : Double;
                          DaylightDate1,
                          DoubleSummerDate,
                          DaylightDate2,
                          StandardDate      : tJulianDay;
                          DaylightBias,
                          DoubleSummerBias,
                          StandardBias      : Double) : tJulianDay;
  begin
    if (DayLightDate1 - StandardBias < Day)
       and (Day < StandardDate - StandardBias)
    then
      if (DoubleSummerDate - DaylightBias < Day)
         and (Day < DayLightDate2 - DaylightBias)
      then
        LocalDayToGMTDay := Day + DoubleSummerBias
      else
        LocalDayToGMTDay := Day + DaylightBias
    else
      LocalDayToGMTDay := Day + StandardBias;
  end;

function GMTJulianFormatDateTime(Format : String; Datum : tJulianDay) : String;
  begin
    GMTJulianFormatDateTime := JulianFormatDateTime(Format, Datum);
  end;

function LocalJulianFormatDateTime(Format : String; Datum : tJulianDay)
           : String;
  var    Year             : Integer;
         Month            : Word;
         Day,
         DaylightDate1,
         DoubleSummerDate,
         DaylightDate2,
         StandardDate     : tJulianDay;
         DaylightBias,
         DoubleSummerBias,
         StandardBias     : Double;
  begin
    DaylightDate1    := 0;
    DoubleSummerDate := 0;
    DaylightDate2    := 0;
    StandardDate     := 0;
    DaylightBias     := 0;
    DoubleSummerBias := 0;
    StandardBias     := 0;
    Year             := 0;
    Month            := 0;
    Day              := 0;
    JulianDate(Datum, Year, Month, Day);
    GetDayLightDates(Year,
                     DaylightDate1,
                     DoubleSummerDate,
                     DaylightDate2,
                     StandardDate,
                     DaylightBias,
                     DoubleSummerBias,
                     StandardBias);
    Day := GMTDayToLocalDay(Datum,
                            DaylightDate1,
                            DoubleSummerDate,
                            DaylightDate2,
                            StandardDate,
                            DaylightBias,
                            DoubleSummerBias,
                            StandardBias);
    LocalJulianFormatDateTime := JulianFormatDateTime(Format, Day);
  end;

Procedure LocalDateToGMTDate(var Year  : Integer;
                             var Month : Word;
                             var Day   : double);
  var     {$ifdef Unix}
            sTime          : tm;
            pTime          : ptm;
            lTime          : time_t;
            d              : Double;
          {$endif}
          b                : Boolean;
          DaylightDate1,
          DoubleSummerDate,
          DaylightDate2,
          StandardDate     : tJulianDay;
          DaylightBias,
          DoubleSummerBias,
          StandardBias     : Double;
  begin
    b := True;
    {$ifdef Unix}
      if Year >= 1970 then
        begin
          sTime.tm_year := 0;
          FillChar(sTime, sizeof(sTime), 0);
          sTime.tm_isdst := -1;
          sTime.tm_year  := Year  - 1900;
          sTime.tm_mon   := Month -    1;
          sTime.tm_mday  := trunc(Day);
          d              := frac(Day) * 24;
          sTime.tm_hour  := trunc(d);
          d              := frac(d)   * 60;
          sTime.tm_min   := trunc(d);
          d              := frac(d)   * 60;
          sTime.tm_sec   := trunc(d);
          d              := frac(d)   / 60 / 60 / 24;
          lTime          := mktime(@sTime);
          if lTime >= 0 then
            begin
              pTime := gmtime(lTime);
              if pTime <> Nil then
                begin
                  Day   :=   pTime^.tm_mday
                           + pTime^.tm_hour / 24
                           + pTime^.tm_min  / 24 / 60
                           + pTime^.tm_sec  / 24 / 60 / 60
                           + d;
                  Month := pTime^.tm_mon  +    1;
                  Year  := pTime^.tm_year + 1900;
                  b     := False;
                end;
            end;
        end;
    {$endif}
    if b then
      begin
        DaylightDate1    := 0;
        DoubleSummerDate := 0;
        DaylightDate2    := 0;
        StandardDate     := 0;
        DaylightBias     := 0;
        DoubleSummerBias := 0;
        StandardBias     := 0;
        GetDayLightDates(Year,
                         DaylightDate1,
                         DoubleSummerDate,
                         DaylightDate2,
                         StandardDate,
                         DaylightBias,
                         DoubleSummerBias,
                         StandardBias);
        JulianDate(LocalDayToGMTDay(JulianDay(Year, Month, Day),
                                    DaylightDate1,
                                    DoubleSummerDate,
                                    DaylightDate2,
                                    StandardDate,
                                    DaylightBias,
                                    DoubleSummerBias,
                                    StandardBias),
                   Year, Month, Day);
      end;
  end;

Procedure GMTDateToLocalDate(var Year  : Integer;
                             var Month : Word;
                             var Day   : double);
  var   DaylightDate1,
        DoubleSummerDate,
        DaylightDate2,
        StandardDate     : tJulianDay;
        DaylightBias,
        DoubleSummerBias,
        StandardBias     : Double;
  begin
    DaylightDate1    := 0;
    DoubleSummerDate := 0;
    DaylightDate2    := 0;
    StandardDate     := 0;
    DaylightBias     := 0;
    DoubleSummerBias := 0;
    StandardBias     := 0;
    GetDayLightDates(Year,
                     DaylightDate1,
                     DoubleSummerDate,
                     DaylightDate2,
                     StandardDate,
                     DaylightBias,
                     DoubleSummerBias,
                     StandardBias);
    JulianDate(GMTDayToLocalDay(JulianDay(Year, Month, Day),
                                DaylightDate1,
                                DoubleSummerDate,
                                DaylightDate2,
                                StandardDate,
                                DaylightBias,
                                DoubleSummerBias,
                                StandardBias),
               Year, Month, Day);
  end;

Function LocalJulDayOfWeek(JD : tJulianDay) : Word;
  var    d, Year          : Integer;
         Month            : Word;
         DaylightDate1,
         DoubleSummerDate,
         DaylightDate2,
         StandardDate     : tJulianDay;
         Day,
         DaylightBias,
         DoubleSummerBias,
         StandardBias     : Double;
  begin
    Year             := 0;
    Month            := 0;
    Day              := 0;
    DaylightDate1    := 0;
    DoubleSummerDate := 0;
    DaylightDate2    := 0;
    StandardDate     := 0;
    DaylightBias     := 0;
    DoubleSummerBias := 0;
    StandardBias     := 0;
    JulianDate(JD, Year, Month, Day);
    GetDayLightDates(Year,
                     DaylightDate1,
                     DoubleSummerDate,
                     DaylightDate2,
                     StandardDate,
                     DaylightBias,
                     DoubleSummerBias,
                     StandardBias);
    d := trunc(GMTDayToLocalDay(JD,
                                DaylightDate1,
                                DoubleSummerDate,
                                DaylightDate2,
                                StandardDate,
                                DaylightBias,
                                DoubleSummerBias,
                                StandardBias) + 1.5) mod 7 + 1;
    if d <= 0 then
      LocalJulDayOfWeek := d + 7
    else
      LocalJulDayOfWeek := d;
  end;

function GMTJulDayOfWeek(JD : tJulianDay) : Word;
  begin
    GMTJulDayOfWeek := JulDayOfWeek(JD);
  end;

Function LocalJulianDay(Year : Integer; Month: Word; Day : double)
           : tJulianDay;
  var    DaylightDate1,
         DoubleSummerDate,
         DaylightDate2,
         StandardDate     : tJulianDay;
         DaylightBias,
         DoubleSummerBias,
         StandardBias     : Double;
  begin
    DaylightDate1    := 0;
    DoubleSummerDate := 0;
    DaylightDate2    := 0;
    StandardDate     := 0;
    DaylightBias     := 0;
    DoubleSummerBias := 0;
    StandardBias     := 0;
    GetDayLightDates(Year,
                     DaylightDate1,
                     DoubleSummerDate,
                     DaylightDate2,
                     StandardDate,
                     DaylightBias,
                     DoubleSummerBias,
                     StandardBias);
    LocalJulianDay := LocalDayToGMTDay(JulianDay(Year, Month, Day),
                                       DaylightDate1,
                                       DoubleSummerDate,
                                       DaylightDate2,
                                       StandardDate,
                                       DaylightBias,
                                       DoubleSummerBias,
                                       StandardBias);
  end;

function GMTJulianDay(Year : Integer; Month: Word; Day : double) : tJulianDay;
  begin
    GMTJulianDay := JulianDay(Year, Month, Day);
  end;

Procedure LocalJulianDate(JD        : tJulianDay;
                          var Year  : Integer;
                          var Month : Word;
                          var Day   : double);
  var   DaylightDate1,
        DoubleSummerDate,
        DaylightDate2,
        StandardDate     : tJulianDay;
        DaylightBias,
        DoubleSummerBias,
        StandardBias     : Double;
  begin
    DaylightDate1    := 0;
    DoubleSummerDate := 0;
    DaylightDate2    := 0;
    StandardDate     := 0;
    DaylightBias     := 0;
    DoubleSummerBias := 0;
    StandardBias     := 0;
    JulianDate(JD, Year, Month, Day);
    GetDayLightDates(Year,
                     DaylightDate1,
                     DoubleSummerDate,
                     DaylightDate2,
                     StandardDate,
                     DaylightBias,
                     DoubleSummerBias,
                     StandardBias);
    JulianDate(GMTDayToLocalDay(JD,
                                DaylightDate1,
                                DoubleSummerDate,
                                DaylightDate2,
                                StandardDate,
                                DaylightBias,
                                DoubleSummerBias,
                                StandardBias), Year, Month, Day);
  end;

Procedure GMTJulianDate(JD        : tJulianDay;
                        var Year  : Integer;
                        var Month : Word;
                        var Day   : double);
  begin
    JulianDate(JD, Year, Month, Day);
  end;

function GMTSameDate(Date1, Date2 : tJulianDay) : Boolean;
  var    Year1,
         Year2  : Integer;
         Month1,
         Month2 : Word;
         Day1,
         Day2   : Double;
  begin
    Year1  := 0;
    Year2  := 0;
    Month1 := 0;
    Month2 := 0;
    Day1   := 0;
    Day2   := 0;
    GMTJulianDate(Date1, Year1, Month1, Day1);
    GMTJulianDate(Date2, Year2, Month2, Day2);
    GMTSameDate := (Year1 = Year2) and (Month1 = Month2)
                   and (Trunc(Day1) = Trunc(Day2));
  end;

function LocalSameDate(Date1, Date2 : tJulianDay) : Boolean;
  var    Year1,
         Year2  : Integer;
         Month1,
         Month2 : Word;
         Day1,
         Day2   : Double;
  begin
    Year1  := 0;
    Year2  := 0;
    Month1 := 0;
    Month2 := 0;
    Day1   := 0;
    Day2   := 0;
    LocalJulianDate(Date1, Year1, Month1, Day1);
    LocalJulianDate(Date2, Year2, Month2, Day2);
    LocalSameDate := (Year1 = Year2) and (Month1 = Month2)
                     and (Trunc(Day1) = Trunc(Day2));
  end;

Function MoonsPhase(Phase : tMoonsPhase; JDay : tJulianDay) : tJulianDay;
  var    Y     : Integer;
         Month : Word;
         Year,
         k,
         T  { julian centuries from 1900 },
         M  { sun's  mean anomaly },
         M1 { moon's mean anomaly },
         F  { moon's argument of latitude  },
         Pi180 : Double;
         Day,
         JD { julian ephemeris days }
            : tJulianDay;
  begin
    Y     := 0;
    Month := 0;
    Day   := 0;
    JulianDate(JDay, Y, Month, Day);
    if y < 0 then
      Year := Y + 1 + (JDay - JulianDay(Y, 1, 1)) / 365.25
    else
      Year := Y + (JDay - JulianDay(Y, 1, 1)) / 365.25;
    Pi180 := ArcTan(1) * 4 / 180;
    k     := (Year - 1900) * 12.3685;
    k     := round(k - ord(Phase) / 4) + ord(Phase) / 4;
    T     := k / 1236.85;
    JD    :=   2415020.75933
             + 29.53058868 * k
             + 0.0001178   * sqr(T) { sqr is the square,
                                      but not the square root }
             - 0.000000155 * sqr(T) * T
             + 0.00033
               * sin(  2.90702
                     + 2.31902 * T
                     - 0.000160099 * sqr(T)
                    {  16656
                     + 13287 * T
                     - 0009173 * sqr(T) } );
    M     := (  359.2242
              + 29.10535608 * k
              - 0.0000333   * sqr(T)
              - 0.00000347  * sqr(T) * T)
             * Pi180;
    M1    := (  306.0253
              + 385.81691806 * k
              - 0.0107306    * sqr(T)
              - 0.00001236   * sqr(T) * T)
             * Pi180;
    F     := (  21.2964
              + 390.67050646 * k
              - 0.0016528    * sqr(T)
              - 0.00000239   * sqr(T) * T)
             * Pi180;
    case Phase of
      NewMoon,
      FullMoon:
        JD := JD + (0.1734 - 0.000393 * T) * sin(M)
                 +  0.0021 * sin(2 * M)
                 -  0.4068 * sin(M1)
                 +  0.0161 * sin(2 * M1)
                 -  0.0004 * sin(3 * M1)
                 +  0.0104 * sin(2 * F)
                 -  0.0051 * sin(M + M1)
                 -  0.0074 * sin(M - M1)
                 +  0.0004 * sin(2 * F + M)
                 -  0.0004 * sin(2 * F - M)
                 -  0.0006 * sin(2 * F + M1)
                 +  0.0010 * sin(2 * F - M1)
                 +  0.0005 * sin(M + 2 * M1);
      FirstQuarter,
      LastQuarter:
        JD := JD + (0.1721 - 0.0004 * T) * sin(M)
                 +  0.0021 * sin(2 * M)
                 -  0.6280 * sin(M1)
                 +  0.0089 * sin(2 * M1)
                 -  0.0004 * sin(3 * M1)
                 +  0.0079 * sin(2 * F)
                 -  0.0119 * sin(M + M1)
                 -  0.0074 * sin(M - M1)
                 +  0.0003 * sin(2 * F + M)
                 -  0.0004 * sin(2 * F - M)
                 -  0.0006 * sin(2 * F + M1)
                 +  0.0021 * sin(2 * F - M1)
                 +  0.0003 * sin(M + 2 * M1)
                 +  0.0004 * sin(M - 2 * M1)
                 -  0.0003 * sin(2 * M + M1);
    end;
    case Phase of
      FirstQuarter:
        JD := JD + 0.0028
                 - 0.0004 * cos(M)
                 + 0.0003 * cos(M1);
      LastQuarter:
        JD := JD - 0.0028
                 + 0.0004 * cos(M)
                 - 0.0003 * cos(M1);
    end;
    MoonsPhase := JD;
  end;

Procedure DivMod(A, B : Integer; var Quote, Rest : Integer);
  begin
    Quote := A div B;
    Rest  := A mod B;
  end;

Function Easter(Year : Integer) : tJulianDay;
  Var    A, B, C, D, E, F, G, H, I, K, L, M, N, P : Integer;
  begin
    if Year < 1583 then
      begin
        A := Year mod 4;
        B := Year mod 7;
        C := Year mod 19;
        D := (19 * C + 15) mod 30;
        E := (2 * A + 4 * B - D + 34) mod 7;
        F := 0;
        G := 0;
        DivMod(D + E + 114, 31, F, G);
        Easter := LocalJulianDay(Year, F, G + 1.5);
      end
    else
      begin
        A := Year mod 19;
        DivMod(Year , 100, B, C);
        DivMod(B, 4, D, E);
        F := (B + 8) div 25;
        G := (B - F + 1) div 3;
        H := (19 * A + B - D - G + 15) mod 30;
        I := 0;
        K := 0;
        DivMod(C, 4, I, K);
        L := (32 + 2 * E + 2 * I - H - K) mod 7;
        M := (A + 11 * H + 22 * L) div 451;
        N := 0;
        P := 0;
        DivMod(H + L - 7 * M + 114, 31, N, P);
        Easter := LocalJulianDay(Year, N, P + 1.5);
      end;
  end;

{$ifdef Ver80}
  procedure Win16Initialization;
    const   WinIni         = 'Win.ini';
            Mapi           = 'MAPI 1.0 Time Zone';
            sBias          = 'Bias';
            sStandardBias  = 'StandardBias';
            sStandardStart = 'StandardStart';
            sStandardName  = 'StandardName';
            sDaylightBias  = 'DaylightBias';
            sDaylightStart = 'DaylightStart';
            sDaylightName  = 'DaylightName';
    var     s              : String;
            IniFile        : tIniFile;
            i, h           : LongInt;
    begin
      s := '';
      try
        IniFile := tIniFile.Create(WinIni);
      except
        IniFile := Nil;
      end;
      if IniFile <> Nil then
        try
          { Maybe Mapi is available,
            otherwise I don't know how to get the information }
          SavedTimeZoneInfo.Bias :=
            HexToInt(IniFile.ReadString(Mapi, sBias,         '0'));
          SavedTimeZoneInfo.StandardBias :=
            HexToInt(IniFile.ReadString(Mapi, sStandardBias, '0'));
          SavedTimeZoneInfo.DayLightBias :=
            HexToInt(IniFile.ReadString(Mapi, sDayLightBias, '0'));
          s := IniFile.ReadString(Mapi, sStandardName, '');
          h := length(s) - 1;
          if h >= High(SavedTimeZoneInfo.StandardName) then
            h := High(SavedTimeZoneInfo.StandardName) - 1;
          for i := low(SavedTimeZoneInfo.StandardName) to h do
            SavedTimeZoneInfo.StandardName[i] := ord(s[i + 1]);
          s := IniFile.ReadString(Mapi, sDaylightName, '');
          h := length(s) - 1;
          if h >= High(SavedTimeZoneInfo.DaylightName) then
            h := High(SavedTimeZoneInfo.DaylightName) - 1;
          for i := low(SavedTimeZoneInfo.DaylightName) to h do
            SavedTimeZoneInfo.DaylightName[i] := ord(s[i + 1]);
          s := IniFile.ReadString(Mapi, sStandardStart, '');
          SavedTimeZoneInfo.StandardDate.wMonth  := HexToInt(Copy(s,  3, 4));
          SavedTimeZoneInfo.StandardDate.wDay    := HexToInt(Copy(s,  7, 4));
          SavedTimeZoneInfo.StandardDate.wHour   := HexToInt(Copy(s, 11, 4));
          SavedTimeZoneInfo.StandardDate.wMinute := HexToInt(Copy(s, 15, 4));
          SavedTimeZoneInfo.StandardDate.wSecond := HexToInt(Copy(s, 19, 4));
          SavedTimeZoneInfo.StandardDate.wMilliseconds :=
            HexToInt(Copy(s, 23, 4));
          SavedTimeZoneInfo.StandardDate.wDayOfWeek :=
            HexToInt(Copy(s, 27, 4));
          s := IniFile.ReadString(Mapi, sDaylightStart, '');
          SavedTimeZoneInfo.DaylightDate.wMonth  := HexToInt(Copy(s,  3, 4));
          SavedTimeZoneInfo.DaylightDate.wDay    := HexToInt(Copy(s,  7, 4));
          SavedTimeZoneInfo.DaylightDate.wHour   := HexToInt(Copy(s, 11, 4));
          SavedTimeZoneInfo.DaylightDate.wMinute := HexToInt(Copy(s, 15, 4));
          SavedTimeZoneInfo.DaylightDate.wSecond := HexToInt(Copy(s, 19, 4));
          SavedTimeZoneInfo.DaylightDate.wMilliseconds :=
            HexToInt(Copy(s, 23, 4));
          SavedTimeZoneInfo.DaylightDate.wDayOfWeek :=
            HexToInt(Copy(s, 27, 4));
          SavedDoubleSummerBias := SavedTimeZoneInfo.Bias
                                   + SavedTimeZoneInfo.DaylightBias - 60;
        finally
          IniFile.Free;
        end;
    end;
{$endif}

{$ifdef Unix}
  procedure UnixInitialization;
    const   sTimeZoneLocationFile = '/etc/timezone';
            sTZName               = 'localtime';
            sTimeZoneFile         = '/etc/'              + sTZName;
            sAltTimeZoneFile      = '/usr/lib/zoneinfo/' + sTZName;
            sTZDIR                = 'TZDIR';
            sTZPath               = '/usr/share/zoneinfo';
            sTZif                 = 'TZif';
            fmReadOnly            = 0;
    var     SearchRec             : tSearchRec;
            TextDatei             : Text;
            TZDatei               : File;
            b                     : Boolean;
            s, t                  : String;
            OldFileMode           : Byte;
            i                     : LongInt;
            Pos                   : Int64;
            qTime                 : ptm;
            pTime,
            sTime                 : tm;
            lTime                 : time_t;
            sTimeVal              : tTimeVal;
            sTimeZone             : tTimeZone;
    begin
      TTextRec(TextDatei).bufend     := 0;
      SearchRec.Size                 := 0;
      SavedTimeZoneInfo.Bias         := - TZSeconds div 60;
      SavedTimeZoneInfo.StandardBias := 0;
      SavedTimeZoneInfo.DaylightBias := 0;
      sTimeVal.tv_sec                := 0;
      sTimeZone.tz_dsttime           := 0;
      fpGetTimeOfDay(@sTimeVal, @sTimeZone);
      qTime         := localtime(sTimeVal.tv_sec);
      pTime.tm_year := 0;
      Move(qtime^, ptime, sizeof(ptime));
      sTime.tm_year := 0;
      FillChar(sTime, sizeof(sTime), 0);
      sTime.tm_year   := pTime.tm_year;
      sTime.tm_mon    :=   0;
      sTime.tm_mday   :=   1;
      sTime.tm_hour   :=  12;
      sTime.tm_isdst  :=  -1;
      lTime           := mktime(@sTime);
      if lTime >= 0 then
        begin
          inc(lTime, 182 * 24 * 60 * 60);
          qTime := localtime(lTime);
          if qTime <> Nil then
            SavedTimeZoneInfo.DaylightBias := - (qTime^.tm_hour - 12) * 60
                                              -  qTime^.tm_min;
        end;
      if pTime.tm_isdst <> 0 then
        dec(SavedTimeZoneInfo.Bias, SavedTimeZoneInfo.DaylightBias);
      SavedDoubleSummerBias := SavedTimeZoneInfo.Bias
                               + SavedTimeZoneInfo.DaylightBias - 60;
      if TZName[False] <> Nil then
        StringToWideChar(TZName[False], SavedTimeZoneInfo.StandardName,
                         sizeof(SavedTimeZoneInfo.StandardName));
      if TZName[True] <> Nil then
        StringToWideChar(TZName[True],  SavedTimeZoneInfo.DaylightName,
                         sizeof(SavedTimeZoneInfo.DaylightName));
      s           := '';
      OldFileMode := FileMode;
      FileMode    := fmReadOnly;
      b           := FileExists(sTimeZoneLocationFile);
      if b then
        begin
          AssignFile(TextDatei, sTimeZoneLocationFile);
          try
            Reset(TextDatei);
          except
            b := False;
          end;
        end;
      if b then
        begin
          Readln(TextDatei, s);
          CloseFile(TextDatei);
          if s = '' then
            s := sTZName;
          if s[1] <> PathDelim then
            begin
              t := GetEnvironmentVariable(sTZDIR);
              if t = '' then
                t := sTZPath;
              if t[length(t)] <> PathDelim then
                t := t + PathDelim;
              s := t + s;
            end;
        end;
      if s > '' then
        begin
          if FindFirst(s, faAnyFile, SearchRec) <> 0 then
            SearchRec.Size := 0;
          FindClose(SearchRec);
        end;
      if SearchRec.Size = 0 then
        begin
          if FindFirst(sTimeZoneFile, faAnyFile, SearchRec) = 0 then
            s := sTimeZoneFile
          else
            SearchRec.Size := 0;
          FindClose(SearchRec);
        end;
      if SearchRec.Size = 0 then
        begin
          if FindFirst(sAltTimeZoneFile, faAnyFile, SearchRec) = 0 then
            s := sAltTimeZoneFile
          else
            SearchRec.Size := 0;
          FindClose(SearchRec);
        end;
      if SearchRec.Size > 0 then
        begin
          GetMem(tzhead, SearchRec.Size);
          if tzhead <> Nil then
            begin
              b := False;
              AssignFile(TZDatei, s);
              try
                Reset(TZDatei, 1);
              except
                b := True;
              end;
              if b then
                begin
                  FreeMem(tzhead);
                  tzhead := Nil;
                end
              else
                begin
                  i := 0;
                  Blockread(TZDatei, tzhead^, SearchRec.Size, i);
                  if i <> SearchRec.Size then
                    begin
                      FreeMem(tzhead);
                      tzhead := Nil;
                    end
                  else
                    if strlcomp(tzhead^.tzh_TZif, sTZif, length(sTZif)) <> 0 then
                      begin
                        FreeMem(tzhead);
                        tzhead := Nil;
                      end;
                  CloseFile(TZDatei);
                end;
            end;
        end;
      FileMode := OldFileMode;
      if tzhead <> Nil then
        begin
          SwapLong(tzhead^.tzh_ttisgmtcnt);
          SwapLong(tzhead^.tzh_ttisstdcnt);
          SwapLong(tzhead^.tzh_leapcnt);
          SwapLong(tzhead^.tzh_timecnt);
          SwapLong(tzhead^.tzh_typecnt);
          SwapLong(tzhead^.tzh_charcnt);
          Pos := sizeof(ttzhead);
          {$R-}
            if tzhead^.tzh_timecnt > 0 then
              begin
                transitions := ptransitions(@pByteArray(tzhead)^[Pos]);
                for i := 0 to tzhead^.tzh_timecnt - 1 do
                  SwapLong(transitions^[i]);
                inc(Pos, LongestInt(tzhead^.tzh_timecnt)
                         * sizeof(ttransitions[0]));
                ttinfoind := @pByteArray(tzhead)^[Pos];
                inc(Pos, tzhead^.tzh_timecnt);
              end;
            if tzhead^.tzh_typecnt > 0 then
              begin
                ttinfo := pttinfo(@pByteArray(tzhead)^[Pos]);
                inc(Pos, LongestInt(tzhead^.tzh_typecnt) * sizeof(tttinfo));
                for i := 0 to tzhead^.tzh_typecnt - 1 do
                  begin
                    SwapLong(ttinfo^[i].tt_gmtoff);
                    if tznames <> Nil then
                      begin
                        while tznames.count <= ttinfo^[i].tt_abbrind do
                          tznames.add('');
                        tznames[ttinfo^[i].tt_abbrind] :=
                          StrPas(@pByteArray(tzhead)^
                                  [Pos + ttinfo^[i].tt_abbrind]);
                      end;
                  end;
              end;
            inc(Pos, tzhead^.tzh_charcnt);
            if tzhead^.tzh_leapcnt > 0 then
              begin
                leap := pleap(@pByteArray(tzhead)^[Pos]);
                for i := 0 to tzhead^.tzh_leapcnt - 1 do
                  begin
                    SwapLong(leap^[i].transition);
                    SwapLong(leap^[i].change);
                  end;
                inc(Pos, LongestInt(tzhead^.tzh_leapcnt) * sizeof(tleap));
              end;
            if tzhead^.tzh_ttisstdcnt > 0 then
              begin
                std_wall := pByteBoolArray(@pByteArray(tzhead)^[Pos]);
                inc(Pos, tzhead^.tzh_ttisstdcnt);
              end;
            if tzhead^.tzh_ttisgmtcnt > 0 then
              begin
                utc_local := pByteBoolArray(@pByteArray(tzhead)^[Pos]);
                inc(Pos, tzhead^.tzh_ttisgmtcnt);
              end;
            tzMoreData := @pByteArray(tzhead)^[Pos];
            tzMoreLen  := SearchRec.Size - Pos;
          {$R+}
        end;
    end;
{$endif}

initialization;
  SpecialDaysYear        := -MaxInt;
  DaylightYear           := -MaxInt;
  SavedDaylightDate1     := 0;
  SavedDoubleSummerDate  := 0;
  SavedDaylightDate2     := 0;
  SavedStandardDate      := 0;
  SavedDoubleSummerBias  := 0;
  tzhead                 := Nil;
  transitions            := Nil;
  ttinfoind              := Nil;
  ttinfo                 := Nil;
  leap                   := Nil;
  std_wall               := Nil;
  utc_local              := Nil;
  tzMoreData             := Nil;
  tzMoreLen              := 0;
  try
    tznames              := tStringList.Create;
  except
    tznames              := nil;
  end;
  try
    LanguageList         := tLanguageList.Create;
  except
    LanguageList         := nil;
  end;
  try
    SpecialDayList       := tSpecialDayList.Create;
  except
    SpecialDayList       := nil;
  end;
  SavedTimeZoneInfo.Bias := 0;
  FillChar(SavedTimeZoneInfo, sizeof(SavedTimeZoneInfo), 0);
  {$ifdef Win32}
    GetTimeZoneInformation(SavedTimeZoneInfo);
    SavedDoubleSummerBias := SavedTimeZoneInfo.Bias
                             + SavedTimeZoneInfo.DaylightBias - 60;
  {$else}
    {$ifdef Ver80}
      { Delphi 1 }
      Win16Initialization;
    {$endif}
    {$ifdef Unix}
      UnixInitialization;
    {$endif}
  {$endif}
  SetTimeNames(SysLocale);

{$ifndef Ver80}
  { unfortunately does not exist in Delphi 1 }
  finalization
    LanguageList.Free;
    SpecialDayList.Free;
    tznames.Free;
    if tzhead <> Nil then
      FreeMem(tzhead);
{$endif}

end.


