{ $Id: wolfutil.pas 163 2012-06-30 11:59:25Z Heiner $

 *****************************************************************************

                               wolfutil.pas
                               ------------
                       Initial Revision : May 2011


 *****************************************************************************

 This file is distributed under the Library GNU General Public License
 (see the file COPYING.LGPL) with the following modification:

 As a special exception, the copyright holder of this file gives you
 permission to link this file with independent modules to produce an
 executable, regardless of the license terms of these independent modules,
 and to copy and distribute the resulting executable under terms of your choice,
 provided that you also meet, for each linked independent module, the terms
 and conditions of the license of that module. An independent module is a
 module which is not derived from or based on this file. If you modify this
 file, you may extend this exception to your version of the file, but
 you are not obligated to do so. If you do not wish to do so, delete this
 exception statement from your version.


 In order to receive a copy of the file COPYING.LGPL, contact:
      Free Software Foundation, Inc.,
      675 Mass Ave
      Cambridge, MA  02139
      USA

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 *****************************************************************************

wolfutil.pas for Lazarus or Delphi
Copyright (C) 2011 Heinrich Wolf.
http://www.Wolf-Fuerth.de/
}

unit wolfutil;

{$ifdef FPC}
  {$mode objfpc}{$H+}
{$endif}

{$ifdef MSWINDOWS}
  { Delphi 2005 and newer }
  {$define DelphiMin4}
{$endif}

{$ifdef Ver140}
  { Kylix 3 }
  {$define DelphiMin4}
{$endif}

{$ifdef Ver130}
  { Delphi 5 }
  {$define DelphiMin4}
{$endif}

{$ifdef Ver120}
  { Delphi 4 }
  {$define DelphiMin4}
{$endif}

{$ifdef FPC}
  {$define DelphiMin4}
{$endif}

interface

uses
  {$ifdef fpc}
    LConvEncoding,
  {$endif}
  {$ifdef Unix}
    GetText,
  {$endif}
  {$ifdef Win32}
    Windows,
  {$endif}
  {$ifdef Ver80}
    IniFiles,
  {$endif}
  Classes;

const
  {$ifdef DelphiMin4}
    MaxLongestInt                 = High(Int64);
  {$else}
    MaxLongestInt                 = MaxLongInt;
  {$endif}

  { Output of DayOfWeek() }
  Sunday                          = 1;
  Monday                          = 2;
  Tuesday                         = 3;
  Wednesday                       = 4;
  Thursday                        = 5;
  Friday                          = 6;
  Saturday                        = 7;

  {$ifndef Win32}
    { Primary language IDs. }
    LANG_NEUTRAL                  = $00;
    LANG_AFRIKAANS                = $36;
    LANG_ALBANIAN                 = $1c;
    LANG_ARABIC                   = $01;
    LANG_BASQUE                   = $2d;
    LANG_BELARUSIAN               = $23;
    LANG_BULGARIAN                = $02;
    LANG_CATALAN                  = $03;
    LANG_CHINESE                  = $04;
    LANG_CROATIAN                 = $1a;
    LANG_CZECH                    = $05;
    LANG_DANISH                   = $06;
    LANG_DUTCH                    = $13;
    LANG_ENGLISH                  = $09;
    LANG_ESTONIAN                 = $25;
    LANG_FAEROESE                 = $38;
    LANG_FARSI                    = $29;
    LANG_FINNISH                  = $0b;
    LANG_FRENCH                   = $0c;
    LANG_GERMAN                   = $07;
    LANG_GREEK                    = $08;
    LANG_HEBREW                   = $0d;
    LANG_HUNGARIAN                = $0e;
    LANG_ICELANDIC                = $0f;
    LANG_INDONESIAN               = $21;
    LANG_ITALIAN                  = $10;
    LANG_JAPANESE                 = $11;
    LANG_KOREAN                   = $12;
    LANG_LATVIAN                  = $26;
    LANG_LITHUANIAN               = $27;
    LANG_NORWEGIAN                = $14;
    LANG_POLISH                   = $15;
    LANG_PORTUGUESE               = $16;
    LANG_ROMANIAN                 = $18;
    LANG_RUSSIAN                  = $19;
    LANG_SERBIAN                  = $1a;
    LANG_SLOVAK                   = $1b;
    LANG_SLOVENIAN                = $24;
    LANG_SPANISH                  = $0a;
    LANG_SWEDISH                  = $1d;
    LANG_THAI                     = $1e;
    LANG_TURKISH                  = $1f;
    LANG_UKRAINIAN                = $22;
    LANG_VIETNAMESE               = $2a;

    { Sublanguage IDs. }
    SUBLANG_NEUTRAL                    = $00;    { language neutral }
    SUBLANG_DEFAULT                    = $01;    { user default }
    SUBLANG_SYS_DEFAULT                = $02;    { system default }
    SUBLANG_ARABIC_SAUDI_ARABIA        = $01;
    SUBLANG_ARABIC_IRAQ                = $02;
    SUBLANG_ARABIC_EGYPT               = $03;
    SUBLANG_ARABIC_LIBYA               = $04;
    SUBLANG_ARABIC_ALGERIA             = $05;
    SUBLANG_ARABIC_MOROCCO             = $06;
    SUBLANG_ARABIC_TUNISIA             = $07;
    SUBLANG_ARABIC_OMAN                = $08;
    SUBLANG_ARABIC_YEMEN               = $09;
    SUBLANG_ARABIC_SYRIA               = $0a;
    SUBLANG_ARABIC_JORDAN              = $0b;
    SUBLANG_ARABIC_LEBANON             = $0c;
    SUBLANG_ARABIC_KUWAIT              = $0d;
    SUBLANG_ARABIC_UAE                 = $0e;    { United arab emirates }
    SUBLANG_ARABIC_BAHRAIN             = $0f;
    SUBLANG_ARABIC_QATAR               = $10;
    SUBLANG_CHINESE_TRADITIONAL        = $01;    { Taiwan }
    SUBLANG_CHINESE_SIMPLIFIED         = $02;    { China }
    SUBLANG_CHINESE_HONGKONG           = $03;
    SUBLANG_CHINESE_SINGAPORE          = $04;
    SUBLANG_DUTCH                      = $01;
    SUBLANG_DUTCH_BELGIAN              = $02;
    SUBLANG_ENGLISH_US                 = $01;    { USA }
    SUBLANG_ENGLISH_UK                 = $02;
    SUBLANG_ENGLISH_AUS                = $03;    { Australia }
    SUBLANG_ENGLISH_CAN                = $04;    { Canada }
    SUBLANG_ENGLISH_NZ                 = $05;    { New Zealand }
    SUBLANG_ENGLISH_EIRE               = $06;    { Ireland }
    SUBLANG_ENGLISH_SOUTH_AFRICA       = $07;
    SUBLANG_ENGLISH_JAMAICA            = $08;
    SUBLANG_ENGLISH_CARIBBEAN          = $09;
    SUBLANG_ENGLISH_BELIZE             = $0a;
    SUBLANG_ENGLISH_TRINIDAD           = $0b;
    SUBLANG_FRENCH                     = $01;
    SUBLANG_FRENCH_BELGIAN             = $02;
    SUBLANG_FRENCH_CANADIAN            = $03;
    SUBLANG_FRENCH_SWISS               = $04;
    SUBLANG_FRENCH_LUXEMBOURG          = $05;
    SUBLANG_GERMAN                     = $01;
    SUBLANG_GERMAN_SWISS               = $02;
    SUBLANG_GERMAN_AUSTRIAN            = $03;
    SUBLANG_GERMAN_LUXEMBOURG          = $04;
    SUBLANG_GERMAN_LIECHTENSTEIN       = $05;
    SUBLANG_ITALIAN                    = $01;
    SUBLANG_ITALIAN_SWISS              = $02;
    SUBLANG_KOREAN                     = $01;    { Extended Wansung }
    SUBLANG_KOREAN_JOHAB               = $02;
    SUBLANG_NORWEGIAN_BOKMAL           = $01;
    SUBLANG_NORWEGIAN_NYNORSK          = $02;
    SUBLANG_PORTUGUESE                 = $02;
    SUBLANG_PORTUGUESE_BRAZILIAN       = $01;
    SUBLANG_SERBIAN_LATIN              = $02;
    SUBLANG_SERBIAN_CYRILLIC           = $03;
    SUBLANG_SPANISH                    = $01;    { Castilian }
    SUBLANG_SPANISH_MEXICAN            = $02;
    SUBLANG_SPANISH_MODERN             = $03;
    SUBLANG_SPANISH_GUATEMALA          = $04;
    SUBLANG_SPANISH_COSTA_RICA         = $05;
    SUBLANG_SPANISH_PANAMA             = $06;
    SUBLANG_SPANISH_DOMINICAN_REPUBLIC = $07;
    SUBLANG_SPANISH_VENEZUELA          = $08;
    SUBLANG_SPANISH_COLOMBIA           = $09;
    SUBLANG_SPANISH_PERU               = $0a;
    SUBLANG_SPANISH_ARGENTINA          = $0b;
    SUBLANG_SPANISH_ECUADOR            = $0c;
    SUBLANG_SPANISH_CHILE              = $0d;
    SUBLANG_SPANISH_URUGUAY            = $0e;
    SUBLANG_SPANISH_PARAGUAY           = $0f;
    SUBLANG_SPANISH_BOLIVIA            = $10;
    SUBLANG_SPANISH_EL_SALVADOR        = $11;
    SUBLANG_SPANISH_HONDURAS           = $12;
    SUBLANG_SPANISH_NICARAGUA          = $13;
    SUBLANG_SPANISH_PUERTO_RICO        = $14;
    SUBLANG_SWEDISH                    = $01;
    SUBLANG_SWEDISH_FINLAND            = $02;
  {$endif}

type
  {$ifdef DelphiMin4}
    LongestInt         = Int64;
  {$else}
    LongestInt         = LongInt;
  {$endif}
  tLongIntArray        = Array[0..0] of LongInt;
  pLongIntArray        = ^tLongIntArray;
  tLongestIntArray     = Array[0..0] of LongestInt;
  pLongestIntArray     = ^tLongestIntArray;
  tExtendedArray       = Array[0..0] of Extended;
  pExtendedArray       = ^tExtendedArray;

  tLongIntList         = class
  private
    fList              : pLongIntArray;
    fCount             : LongInt;
    function             GetLongInt(Pos : LongInt) : LongInt;
    procedure            SetLongInt(Pos : LongInt; NewVal : LongInt);
    procedure            SetCount(NewCount : LongInt);
  public
    constructor          Create;
    destructor           Destroy; override;
    property             Count : LongInt read fCount write SetCount;
    property             Items[Pos : LongInt] : LongInt
                           read GetLongInt write SetLongInt; default;
    procedure            Clear;
    procedure            Add(NewVal : LongInt);
    procedure            Delete(Pos : LongInt);
    function             IndexOf(Val : LongInt) : LongInt;
  end;

  tLongestIntList      = class
  private
    fList              : pLongestIntArray;
    fCount             : LongInt;
    function             GetLongestInt(Pos : LongInt) : LongestInt;
    procedure            SetLongestInt(Pos : LongInt; NewVal : LongestInt);
    procedure            SetCount(NewCount : LongInt);
  public
    constructor          Create;
    destructor           Destroy; override;
    property             Count : LongInt read fCount write SetCount;
    property             Items[Pos : LongInt] : LongestInt
                           read GetLongestInt write SetLongestInt; default;
    procedure            Exchange(Index, WithIndex : LongInt);
  end;

  tExtendedList        = class
  private
    fList              : pExtendedArray;
    fCount             : LongInt;
    procedure            SetCount(NewCount : LongInt);
    function             GetExtended(Pos : LongInt) : Extended;
    procedure            SetExtended(Pos : LongInt; NewVal : Extended);
  public
    constructor          Create;
    destructor           Destroy; override;
    property             Count : LongInt read fCount write SetCount;
    property             Items[Pos : LongInt] : Extended
                           read GetExtended write SetExtended; default;
    procedure            Exchange(Index, WithIndex : LongInt);
    procedure            Assign(l : tExtendedList);
  end;

  tLongestIntMatrix    = class
  private
    fList              : tList;
    function             GetCell(i, j : LongInt) : LongestInt;
    procedure            SetCell(i, j : LongInt; NewVal : LongestInt);
  public
    constructor          Create;
    destructor           Destroy; override;
    property             Cells[i, j : LongInt] : LongestInt
                           read GetCell write SetCell; default;
    procedure            AddInts(Count : LongInt);
    procedure            AddNil;
    procedure            Clear;
    procedure            Exchange(Index, WithIndex : LongInt);
  end;

  tExtendedMatrix      = class
  private
    fList              : tList;
    function             GetRowCount : LongInt;
    function             GetColCount : LongInt;
    function             GetCell(i, j : LongInt) : Extended;
    procedure            SetCell(i, j : LongInt; NewVal : Extended);
  public
    constructor          Create;
    destructor           Destroy; override;
    property             Cells[i, j : LongInt] : Extended
                           read GetCell write SetCell; default;
    procedure            AddExtended(Count : LongInt);
    procedure            AddNil;
    procedure            Clear;
    procedure            Exchange(Index, WithIndex : LongInt);
    property             ColCount : LongInt read GetColCount;
    property             RowCount : LongInt read GetRowCount;
    procedure            Dim(NewColCount, NewRowCount : LongInt);
    procedure            Assign(m : tExtendedMatrix);
  end;

  tSort                = class
  public
    function             MayContinue : Boolean; virtual; abstract;
    { MayContinue allows canceling
      and should contain Application.ProcessMessages
    }
    function             Compare(a, b : LongInt) : Integer; virtual; abstract;
    procedure            Swap(a, b : LongInt); virtual; abstract;
    procedure            Sort(von, bis : LongInt);
  end;

{$ifdef Ver80}
  tSysLocale           = Record
                           PriLangID,
                           SubLangID : Word;
                         end;

  var SysLocale        : tSysLocale;
{$endif}

function FormatLength(Format : String; c : Char) : Integer;
function StripFormat(Format : String) : String;
function StripString(Literal : String; var s : String) : Boolean;
function StrWithFormatToDateTime(s, Format : String) : tDateTime;
function GetShortDateFormat   : String;
function GetLongDateFormat    : String;
function GetLongTimeFormat    : String;
function GetShortTimeFormat   : String;
function GetTimeAMString      : String;
function GetTimePMString      : String;
function GetLongMonthName(m   : Integer) : String;
function GetShortMonthName(m  : Integer) : String;
function GetLongDayName(d     : Integer) : String;
function GetShortDayName(d    : Integer) : String;
function GetThousandSeparator : char;
function GetDateSeparator     : char;
function GetTimeSeparator     : char;
function GetCurrencyDecimals  : Byte;
function GetCurrencyFormat    : Byte;
function GetCurrencyString    : String;
function CP1252toSystem(s : String) : String;
function LNull(S : String; Feld : Integer) : String;
function HexToInt(s : String) : LongestInt;
function IntToDez(i : LongestInt; Stellen : Integer;
                  Thousands : Boolean) : String;
function DezToInt(s : String) : LongestInt;

{$ifndef DelphiMin4}
  Type     TReplaceFlags  = set of (rfReplaceAll, rfIgnoreCase);
  function StringReplace(const S, OldPattern, NewPattern: string;
                         Flags: TReplaceFlags): string;
{$endif}

{$ifdef Ver80}
  function StringOfChar(c : char; n : Byte) : String;
  function Trim(s : String) : String;
{$endif}

implementation

uses Sysutils;

constructor tLongIntList.Create;
  begin
    fList  := Nil;
    fCount := 0;
  end;

destructor tLongIntList.Destroy;
  begin
    Clear;
  end;

function tLongIntList.GetLongInt(Pos : LongInt) : LongInt;
  begin
    if (Pos < 0) or (Pos >= fCount) then
      raise Exception.Create('tLongIntList.GetLongInt(' + IntToStr(Pos)
                             + ' out of range)')
    else
      {$R-}
        GetLongInt := fList^[Pos];
      {$R+}
  end;

procedure tLongIntList.SetLongInt(Pos : LongInt; NewVal : LongInt);
  begin
    if (Pos < 0) or (Pos >= fCount) then
      raise Exception.Create('tLongIntList.SetLongInt(' + IntToStr(Pos)
                             + ' out of range)')
    else
      {$R-}
        fList^[Pos] := NewVal;
      {$R+}
  end;

procedure tLongIntList.SetCount(NewCount : LongInt);
  var     NewList : pLongIntArray;
          i, j    : LongInt;
  begin
    if NewCount < 0 then
      Raise Exception.Create('invalid tLongIntList.SetCount('
                             + IntToStr(NewCount) + ')')
    else
      if NewCount <> fCount then
        begin
          GetMem(NewList, NewCount * sizeof(LongInt));
          if NewList = Nil then
            Raise Exception.Create('out of memory in tLongIntList.SetCount')
          else
            begin
              if NewCount > fCount then
                j := fCount   - 1
              else
                j := NewCount - 1;
              for i := 0 to j do
                {$R-}
                  NewList^[i] := fList^[i];
                {$R+}
              if fList <> Nil then
                FreeMem(fList, fCount * sizeof(LongInt));
              fList  := NewList;
              fCount := NewCount;
            end;
        end;
  end;

procedure tLongIntList.Clear;
  begin
    if fList <> Nil then
      FreeMem(fList, fCount * sizeof(LongInt));
    fList  := Nil;
    fCount := 0;
  end;

procedure tLongIntList.Add(NewVal : LongInt);
  begin
    SetCount(fCount + 1);
    {$R-}
      fList^[fCount - 1] := NewVal;
    {$R+}
  end;

procedure tLongIntList.Delete(Pos : LongInt);
  var     NewList : pLongIntArray;
          i       : LongInt;
  begin
    if (Pos < 0) or (Pos >= fCount) then
      raise Exception.Create('tLongIntList.Delete(' + IntToStr(Pos)
                             + ' out of range)')
    else
      begin
        if fCount <= 1 then
          NewList := Nil
        else
          begin
            GetMem(NewList, (fCount - 1) * sizeof(LongInt));
            if NewList = Nil then
              raise Exception.Create('out of memory in tLongIntList.Delete')
            else
              begin
                {$R-}
                  for i := 0 to Pos - 1 do
                    NewList^[i] := fList^[i];
                  for i := Pos to fCount - 2 do
                    NewList^[i] := fList^[i + 1];
                {$R+}
              end;
          end;
        if fList <> Nil then
          FreeMem(fList, fCount * sizeof(LongInt));
        fList := NewList;
        dec(fCount);
      end;
  end;

function tLongIntList.IndexOf(Val : LongInt) : LongInt;
  var    i : LongInt;
  begin
    IndexOf := -1;
    i       := 0;
    {$R-}
    while i < fCount do
      if fList^[i] = Val then
        begin
          IndexOf := i;
          i       := fCount;
        end
      else
        inc(i);
    {$R+}
  end;

constructor tLongestIntList.Create;
  begin
    fList  := Nil;
    fCount := 0;
  end;

destructor tLongestIntList.Destroy;
  begin
    if fList <> Nil then
      FreeMem(fList, fCount * sizeof(LongestInt));
    fList  := Nil;
    fCount := 0;
  end;

function tLongestIntList.GetLongestInt(Pos : LongInt) : LongestInt;
  begin
    if (Pos < 0) or (Pos >= fCount) then
      Raise Exception.Create('tLongestIntList.GetLongestInt(' + IntToStr(Pos)
                             + ' out of range)')
    else
      {$R-}
        GetLongestInt := fList^[Pos];
      {$R+}
  end;

procedure tLongestIntList.SetLongestInt(Pos : LongInt; NewVal : LongestInt);
  begin
    if (Pos < 0) or (Pos >= fCount) then
      Raise Exception.Create('tLongestIntList.SetLongestInt(' + IntToStr(Pos)
                             + ' out of range)')
    else
      {$R-}
        fList^[Pos] := NewVal;
      {$R+}
  end;

procedure tLongestIntList.SetCount(NewCount : LongInt);
  begin
    if NewCount < 0 then
      Raise Exception.Create('invalid tLongestIntList.SetCount('
                             + IntToStr(NewCount) + ')')
    else
      if NewCount <> fCount then
        begin
          if fList <> Nil then
            FreeMem(fList, fCount * sizeof(LongestInt));
          fCount := 0;
          GetMem(fList, NewCount * sizeof(LongestInt));
          if fList = Nil then
            Raise Exception.Create('out of memory in tLongestIntList.SetCount');
          fCount := NewCount;
        end;
  end;

procedure tLongestIntList.Exchange(Index, WithIndex : LongInt);
  var     Tmp : LongestInt;
  begin
    if (Index < 0) or (Index >= fCount) then
      Raise Exception.Create('tLongestIntList.Exchange(Index '
                             + IntToStr(Index) + ' out of range)')
    else
      if (WithIndex < 0) or (WithIndex >= fCount) then
        Raise Exception.Create('tLongestIntList.Exchange(WithIndex '
                               + IntToStr(WithIndex) + ' out of range)')
      else
        begin
          {$R-}
            Tmp               := fList^[Index];
            fList^[Index]     := fList^[WithIndex];
            fList^[WithIndex] := Tmp;
          {$R+}
        end;
  end;

constructor tExtendedList.Create;
  begin
    fList  := Nil;
    fCount := 0;
  end;

destructor tExtendedList.Destroy;
  begin
    if fList <> Nil then
      FreeMem(fList, fCount * sizeof(Extended));
    fList  := Nil;
    fCount := 0;
  end;

function tExtendedList.GetExtended(Pos : LongInt) : Extended;
  begin
    if (Pos < 0) or (Pos >= fCount) then
      Raise Exception.Create('tExtendedList.GetExtended(' + IntToStr(Pos)
                             + ' out of range)')
    else
      {$R-}
        GetExtended := fList^[Pos];
      {$R+}
  end;

procedure tExtendedList.SetExtended(Pos : LongInt; NewVal : Extended);
  begin
    if (Pos < 0) or (Pos >= fCount) then
      Raise Exception.Create('tExtendedList.SetExtended(' + IntToStr(Pos)
                             + ' out of range)')
    else
      {$R-}
        fList^[Pos] := NewVal;
      {$R+}
  end;

procedure tExtendedList.SetCount(NewCount : LongInt);
  begin
    if NewCount < 0 then
      Raise Exception.Create('invalid tExtendedList.SetCount('
                             + IntToStr(NewCount) + ')')
    else
      if NewCount <> fCount then
        begin
          if fList <> Nil then
            FreeMem(fList, fCount * sizeof(Extended));
          fCount := 0;
          GetMem(fList, NewCount * sizeof(Extended));
          if fList = Nil then
            Raise Exception.Create('out of memory in tExtendedList.SetCount');
          fCount := NewCount;
        end;
  end;

procedure tExtendedList.Exchange(Index, WithIndex : LongInt);
  var     Tmp : Extended;
  begin
    if (Index < 0) or (Index >= fCount) then
      Raise Exception.Create('tExtendedList.Exchange(Index '
                             + IntToStr(Index) + ' out of range)')
    else
      if (WithIndex < 0) or (WithIndex >= fCount) then
        Raise Exception.Create('tExtendedList.Exchange(WithIndex '
                               + IntToStr(WithIndex) + ' out of range)')
      else
        begin
          {$R-}
            Tmp               := fList^[Index];
            fList^[Index]     := fList^[WithIndex];
            fList^[WithIndex] := Tmp;
          {$R+}
        end;
  end;

procedure tExtendedList.Assign(l : tExtendedList);
  var     i : LongInt;
  begin
    SetCount(l.Count);
    for i := 0 to fCount - 1 do
      {$R-}
        fList^[i] := l.fList^[i];
      {$r+}
  end;

constructor tLongestIntMatrix.Create;
  begin
    try
      fList := tList.Create;
    except
      fList := Nil;
    end;
    if fList = Nil then
      raise Exception.Create('Error in tLongestIntMatrix.Create');
  end;

destructor tLongestIntMatrix.Destroy;
  begin
    if fList <> Nil then
      begin
        Clear;
        fList.Free;
      end;
    fList := Nil;
  end;

function tLongestIntMatrix.GetCell(i, j : LongInt) : LongestInt;
  begin
    if fList[i] = Nil then
      Raise Exception.Create('tLongestIntMatrix.GetCell('
                             + IntToStr(i) + ' out of range)')
    else
      GetCell := tLongestIntList(fList[i])[j];
  end;

procedure tLongestIntMatrix.SetCell(i, j : LongInt; NewVal : LongestInt);
  begin
    if fList[i] = Nil then
      Raise Exception.Create('tLongestIntMatrix.SetCell('
                             + IntToStr(i) + ' out of range)')
    else
      tLongestIntList(fList[i])[j] := NewVal;
  end;

procedure tLongestIntMatrix.AddInts(Count : LongInt);
  var     NewCells : tLongestIntList;
  begin
    NewCells       := tLongestIntList.Create;
    NewCells.Count := Count;
    fList.Add(NewCells);
  end;

procedure tLongestIntMatrix.AddNil;
  begin
    fList.Add(Nil);
  end;

procedure tLongestIntMatrix.Clear;
  begin
    while fList.Count > 0 do
      begin
        tLongestIntList(fList[0]).Free;
        fList.Delete(0);
      end;
  end;

procedure tLongestIntMatrix.Exchange(Index, WithIndex : LongInt);
  var     i : LongInt;
  begin
    for i := 0 to fList.Count - 1 do
      if fList[i] <> Nil then
        tLongestIntList(fList[i]).Exchange(Index, WithIndex);
  end;

constructor tExtendedMatrix.Create;
  begin
    try
      fList := tList.Create;
    except
      fList := Nil;
    end;
    if fList = Nil then
      raise Exception.Create('Error in tExtendedMatrix.Create');
  end;

destructor tExtendedMatrix.Destroy;
  begin
    if fList <> Nil then
      begin
        Clear;
        fList.Free;
      end;
    fList := Nil;
  end;

function tExtendedMatrix.GetRowCount : LongInt;
  var    i,
         c  : LongInt;
  begin
    if fList.Count = 0 then
      GetRowCount := 0
    else
      begin
        if fList[0] = Nil then
          c := 0
        else
          c := tExtendedList(fList[0]).Count;
        for i := 1 to fList.Count - 1 do
          if fList[i] = Nil then
            begin
              if c <> 0 then
                raise Exception.Create(
                  'tExtendedMatrix.GetRowCount not rectangular')
            end
          else
            if tExtendedList(fList[i]).Count <> c then
              raise Exception.Create(
                'tExtendedMatrix.GetRowCount not rectangular');
        GetRowCount := c;
      end;
  end;

function tExtendedMatrix.GetColCount : LongInt;
  begin
    GetColCount := fList.Count;
  end;

procedure tExtendedMatrix.Dim(NewColCount, NewRowCount : LongInt);
  var     NewCells : tExtendedList;
          Col      : LongInt;
  begin
    if (NewColCount = 0) and (NewRowCount > 0) then
      raise Exception.Create('tExtendedMatrix.Dim(0, '
                             + IntToStr(NewRowCount) + ' > 0) is not possible')
    else
      begin
        while fList.Count > NewColCount do
          begin
            tExtendedList(fList[fList.Count - 1]).Free;
            fList.Delete(fList.Count - 1);
          end;
        while fList.Count < NewColCount do
          begin
            NewCells := tExtendedList.Create;
            fList.Add(NewCells);
          end;
        for Col := 0 to fList.Count - 1 do
          tExtendedList(fList[Col]).Count := NewRowCount;
      end;
  end;

function tExtendedMatrix.GetCell(i, j : LongInt) : Extended;
  begin
    if fList[i] = Nil then
      Raise Exception.Create('tExtendedMatrix.GetCell('
                             + IntToStr(i) + ' out of range)')
    else
      GetCell := tExtendedList(fList[i])[j];
  end;

procedure tExtendedMatrix.SetCell(i, j : LongInt; NewVal : Extended);
  begin
    if fList[i] = Nil then
      Raise Exception.Create('tExtendedMatrix.SetCell('
                             + IntToStr(i) + ' out of range)')
    else
      tExtendedList(fList[i])[j] := NewVal;
  end;

procedure tExtendedMatrix.AddExtended(Count : LongInt);
  var     NewCells : tExtendedList;
  begin
    NewCells       := tExtendedList.Create;
    NewCells.Count := Count;
    fList.Add(NewCells);
  end;

procedure tExtendedMatrix.AddNil;
  begin
    fList.Add(Nil);
  end;

procedure tExtendedMatrix.Clear;
  begin
    while fList.Count > 0 do
      begin
        tExtendedList(fList[0]).Free;
        fList.Delete(0);
      end;
  end;

procedure tExtendedMatrix.Exchange(Index, WithIndex : LongInt);
  var     i : LongInt;
  begin
    for i := 0 to fList.Count - 1 do
      if fList[i] <> Nil then
        tExtendedList(fList[i]).Exchange(Index, WithIndex);
  end;

procedure tExtendedMatrix.Assign(m : tExtendedMatrix);
  var i : LongInt;
  begin
    Dim(m.ColCount, m.RowCount);
    for i := 0 to m.fList.Count - 1 do
      tExtendedList(fList[i]).Assign(tExtendedList(m.fList[i]));
  end;

procedure tSort.Sort(von, bis : LongInt);
  var     Mitte,
          i, j   : LongInt;
          Done   : Boolean;
  begin
    i    := von;
    Done := False;
    {  This repeat ensures,
       that an already sorted range is not resorted
    }
    repeat
      if i >= Bis then
        Done := True
      else
        if Compare(i, i + 1) <= 0 then
          inc(i)
        else
          Done := True;
    until done;
    if (i < bis) and MayContinue then
      begin
        Mitte := i;
        i     := von;
        j     := bis;
        while (i < j) and MayContinue do
          begin
            while (i < Mitte) and (Compare(i, Mitte) < 0) and MayContinue do
              inc(i);
            while (Mitte < j) and (Compare(Mitte, j) <= 0) and MayContinue do
              dec(j);
            if i = Mitte then
              begin
                Mitte := (i + j + 1) div 2;
                Swap(i, Mitte);
              end
            else
              if Mitte = j then
                begin
                  Mitte := (i + j) div 2;
                  Swap(Mitte, j);
                end
              else
                Swap(i, j);
          end;
        if i = von then
          inc(j)
        else
          dec(i);
        Sort(von, i);
        Sort(j,   bis);
      end;
  end;

function GetShortDateFormat : String;
  begin
    {$ifdef fpc}
      {$ifdef Unix}
        GetShortDateFormat := FormatSettings.ShortDateFormat;
      {$else}
        GetShortDateFormat := ConvertEncoding(FormatSettings.ShortDateFormat,
          GetDefaultTextEncoding, 'UTF8');
      {$endif}
    {$else}
      GetShortDateFormat := ShortDateFormat;
    {$endif}
  end;

function GetLongDateFormat : String;
  begin
    {$ifdef fpc}
      {$ifdef Unix}
        GetLongDateFormat := FormatSettings.LongDateFormat;
      {$else}
        GetLongDateFormat := ConvertEncoding(FormatSettings.LongDateFormat,
          GetDefaultTextEncoding, 'UTF8');
      {$endif}
    {$else}
      GetLongDateFormat := LongDateFormat;
    {$endif}
  end;

function GetLongTimeFormat : String;
  begin
    {$ifdef fpc}
      {$ifdef Unix}
        GetLongTimeFormat := FormatSettings.LongTimeFormat;
      {$else}
        GetLongTimeFormat := ConvertEncoding(FormatSettings.LongTimeFormat,
          GetDefaultTextEncoding, 'UTF8');
      {$endif}
    {$else}
      GetLongTimeFormat := LongTimeFormat;
    {$endif}
  end;

function GetShortTimeFormat : String;
  begin
    {$ifdef fpc}
      {$ifdef Unix}
        GetShortTimeFormat := FormatSettings.ShortTimeFormat;
      {$else}
        GetShortTimeFormat := ConvertEncoding(FormatSettings.ShortTimeFormat,
          GetDefaultTextEncoding, 'UTF8');
      {$endif}
    {$else}
      GetShortTimeFormat := ShortTimeFormat;
    {$endif}
  end;

function GetTimeAMString : String;
  begin
    {$ifdef fpc}
      {$ifdef Unix}
        GetTimeAMString := FormatSettings.TimeAMString;
      {$else}
        GetTimeAMString := ConvertEncoding(FormatSettings.TimeAMString,
          GetDefaultTextEncoding, 'UTF8');
      {$endif}
    {$else}
      GetTimeAMString := TimeAMString;
    {$endif}
  end;

function GetTimePMString : String;
  begin
    {$ifdef fpc}
      {$ifdef Unix}
        GetTimePMString := FormatSettings.TimePMString;
      {$else}
        GetTimePMString := ConvertEncoding(FormatSettings.TimePMString,
          GetDefaultTextEncoding, 'UTF8');
      {$endif}
    {$else}
      GetTimePMString := TimePMString;
    {$endif}
  end;

function GetLongMonthName(m : Integer) : String;
  begin
    {$ifdef fpc}
      {$ifdef Unix}
        GetLongMonthName := FormatSettings.LongMonthNames[m];
      {$else}
        GetLongMonthName := ConvertEncoding(FormatSettings.LongMonthNames[m],
          GetDefaultTextEncoding, 'UTF8');
      {$endif}
    {$else}
      GetLongMonthName := LongMonthNames[m];
    {$endif}
  end;

function GetShortMonthName(m : Integer) : String;
  begin
    {$ifdef fpc}
      {$ifdef Unix}
        GetShortMonthName := FormatSettings.ShortMonthNames[m];
      {$else}
        GetShortMonthName := ConvertEncoding(FormatSettings.ShortMonthNames[m],
          GetDefaultTextEncoding, 'UTF8');
      {$endif}
    {$else}
      GetShortMonthName := ShortMonthNames[m];
    {$endif}
  end;

function GetLongDayName(d : Integer) : String;
  begin
    {$ifdef fpc}
      {$ifdef Unix}
        GetLongDayName := FormatSettings.LongDayNames[d];
      {$else}
        GetLongDayName := ConvertEncoding(FormatSettings.LongDayNames[d],
          GetDefaultTextEncoding, 'UTF8');
      {$endif}
    {$else}
      GetLongDayName := LongDayNames[d];
    {$endif}
  end;

function GetShortDayName(d : Integer) : String;
  begin
    {$ifdef fpc}
      {$ifdef Unix}
        GetShortDayName := FormatSettings.ShortDayNames[d];
      {$else}
        GetShortDayName := ConvertEncoding(FormatSettings.ShortDayNames[d],
          GetDefaultTextEncoding, 'UTF8');
      {$endif}
    {$else}
      GetShortDayName := ShortDayNames[d];
    {$endif}
  end;

function GetThousandSeparator : char;
  begin
    {$ifdef fpc}
      GetThousandSeparator := FormatSettings.ThousandSeparator;
    {$else}
      GetThousandSeparator := ThousandSeparator;
    {$endif}
  end;

function GetDateSeparator : char;
  begin
    {$ifdef fpc}
      GetDateSeparator := FormatSettings.DateSeparator;
    {$else}
      GetDateSeparator := DateSeparator;
    {$endif}
  end;

function GetTimeSeparator : char;
  begin
    {$ifdef fpc}
      GetTimeSeparator := FormatSettings.TimeSeparator;
    {$else}
      GetTimeSeparator := TimeSeparator;
    {$endif}
  end;

function GetCurrencyDecimals : Byte;
  begin
    {$ifdef fpc}
      GetCurrencyDecimals := FormatSettings.CurrencyDecimals;
    {$else}
      GetCurrencyDecimals := CurrencyDecimals;
    {$endif}
  end;

function GetCurrencyFormat : Byte;
  begin
    {$ifdef fpc}
      GetCurrencyFormat := FormatSettings.CurrencyFormat;
    {$else}
      GetCurrencyFormat := CurrencyFormat;
    {$endif}
  end;

function GetCurrencyString : String;
  begin
    {$ifdef fpc}
      {$ifdef Unix}
        GetCurrencyString := FormatSettings.CurrencyString;
      {$else}
        GetCurrencyString := ConvertEncoding(FormatSettings.CurrencyString,
          GetDefaultTextEncoding, 'UTF8');
      {$endif}
    {$else}
      GetCurrencyString := CurrencyString;
    {$endif}
  end;

function CP1252toSystem(s : String) : String;
  begin
    {$ifdef fpc}
      CP1252toSystem := CP1252toUTF8(s);
    {$else}
      CP1252toSystem := s;
    {$endif}
  end;

function StripString(Literal : String; var s : String) : Boolean;
  begin
    StripString := False;
    if UpperCase(Copy(s, 1, length(Literal))) = UpperCase(Literal) then
      begin
        StripString := True;
        s := Copy(s, length(Literal) + 1, length(s));
      end;
  end;

function FormatLength(Format : String; c : Char) : Integer;
  var    i, j, l : Integer;
  begin
    l := length(Format);
    i := 0;
    j := 1;
    c := UpperCase(c)[1];
    while j <= l do
      if UpperCase(Format[j]) = c then
        begin
          i := j;
          inc(j);
        end
      else
        j := l + 1;
    FormatLength := i;
  end;

function StripFormat(Format : String) : String;
  begin
    if Format = '' then
      StripFormat := ''
    else
      StripFormat :=
        Copy(Format, FormatLength(Format, Format[1]) + 1, length(Format));
  end;

function StripWord(var s : String; Max : Integer; var w : Word) : Boolean;
  var    e : Integer;
  begin
    StripWord := True;
    val(Copy(s, 1, Max), w, e);
    if e = 0 then
      s := Copy(s, Max + 1, length(s))
    else
      if e = 1 then
        StripWord := False
      else
        begin
          w := StrToInt(Copy(s, 1, e - 1));
          s := Copy(s, e, length(s));
        end;
  end;

function StrWithFormatToDateTime(s, Format : String) : tDateTime;
  var    Year,
         Month,
         Day,
         Hour,
         Minute,
         Second,
         MSec          : Word;
         DateContained,
         M_is_Min      : Boolean;
         Error         : String;
         t             : tDateTime;

  procedure Parse(var Format : String);
    const   am_pm      = 'am/pm';
            a_p        = 'a/p';
            ampm       = 'ampm';
    var     i,
            OldHour,
            OldMinute,
            OldSecond,
            OldMSec    : Word;
            pm, b      : Boolean;
            OldS,
            OldFormat,
            e, t       : String;
            c          : Char;
    begin
      pm        := False;
      OldS      := S;
      OldFormat := Format;
      OldHour   := Hour;
      OldMinute := Minute;
      OldSecond := Second;
      OldMSec   := MSec;
      While (Format > '') and (Error = '') do
        begin
          case UpperCase(Format[1])[1] of
            'C' : begin
                    e := GetShortDateFormat;
                    Parse(e);
                    if Error = '' then
                      begin
                        M_is_Min := True;
                        e        := GetLongTimeFormat;
                        t        := s;
                        Parse(e);
                        if Error > '' then
                          begin
                            s        := t;
                            M_is_Min := False;
                            Error    := '';
                          end
                        else
                          if (Hour = 0) and (Minute = 0) and (Second = 0)
                             and (MSec = 0)
                          then
                            begin
                              s        := t;
                              M_is_Min := False;
                            end;
                      end;
                    Format := StripFormat(Format);
                  end;
            'T' : begin
                    M_is_Min := True;
                    if FormatLength(Format, Format[1]) > 1 then
                      e    := GetLongTimeFormat
                    else
                      e    := GetShortTimeFormat;
                    Parse(e);
                    Format := StripFormat(Format);
                  end;
            'D' : begin
                    DateContained := True;
                    case FormatLength(Format, Format[1]) of
                      1,
                      2 : begin
                            if not StripWord(s, 2, Day) then
                              Error := 'no day';
                            M_is_Min := False;
                          end;
                      3 : begin
                            i := 1;
                            while (i <= 7) do
                              if StripString(GetShortDayName(i), s) then
                                i := 9
                              else
                                inc(i);
                            if i = 8 then
                              Error := 'no ShortDayName';
                            M_is_Min := False;
                          end;
                      4 : begin
                            i := 1;
                            while (i <= 7) do
                              if StripString(GetLongDayName(i), s) then
                                i := 9
                              else
                                inc(i);
                            if i = 8 then
                              Error := 'no LongDayName';
                            M_is_Min := False;
                          end;
                      5 : begin
                            e := GetShortDateFormat;
                            Parse(e);
                          end;
                    else
                      e := GetLongDateFormat;
                      Parse(e);
                    end;
                    Format := StripFormat(Format);
                  end;
            'M':  begin
                    if M_is_Min then
                      begin
                        if not StripWord(s, 2, Minute) then
                          Error := 'no minute';
                      end
                    else
                      begin
                        DateContained := True;
                        case FormatLength(Format, Format[1]) of
                          1,
                          2 : if not StripWord(s, 2, Month) then
                                Error := 'no month';
                          3 : begin
                                i := 1;
                                while (i <= 12) do
                                  if StripString(GetShortMonthName(i), s) then
                                    begin
                                      Month := i;
                                      i     := 14;
                                    end
                                  else
                                    inc(i);
                                if i = 13 then
                                  Error := 'no ShortMonthName';
                              end;
                        else
                          i := 1;
                          while (i <= 12) do
                            if StripString(GetLongMonthName(i), s) then
                              begin
                                Month := i;
                                i     := 14;
                              end
                            else
                              inc(i);
                          if i = 13 then
                            Error := 'no LongMonthName';
                        end;
                      end;
                    M_is_Min := False;
                    Format   := StripFormat(Format);
                  end;
            'Y' : begin
                    DateContained := True;
                    case FormatLength(Format, Format[1]) of
                      1,
                      2 : begin
                            if not StripWord(s, 2, Year) then
                              Error := 'no year';
                            Inc(Year, 1900);
                          end;
                    else
                      if not StripWord(s, 4, Year) then
                        Error := 'no year';
                    end;
                    M_is_Min := False;
                    Format   := StripFormat(Format);
                  end;
            'H' : begin
                    if not StripWord(s, 2, Hour) then
                      Error := 'no hour';
                    M_is_Min := True;
                    Format   := StripFormat(Format);
                  end;
            'N' : begin
                    if not StripWord(s, 2, Minute) then
                      Error := 'no minute';
                    M_is_Min := False;
                    Format   := StripFormat(Format);
                  end;
            'S' : begin
                    if not StripWord(s, 2, Second) then
                      Error := 'no second';
                    M_is_Min := False;
                    Format   := StripFormat(Format);
                  end;
            'Z' : begin
                    if not StripWord(s, 3, MSec) then
                      Error := 'no millisecond';
                    M_is_Min := False;
                    Format   := StripFormat(Format);
                  end;
            'A' : begin
                    if StripString(am_pm, Format) then
                      begin
                        if not StripString('am', s) then
                          if StripString('pm', s) then
                            pm := true
                          else
                            Error := 'no ' + am_pm;
                      end
                    else
                      if StripString(ampm, Format) then
                        begin
                          b := True;
                          e := GetTimeAMString;
                          if e > '' then
                            if StripString(e, s) then
                              b := False;
                          if b then
                            begin
                              e := GetTimePMString;
                              if e = '' then
                                b := False
                              else
                                if StripString(e, s) then
                                  begin
                                    pm := true;
                                    b  := False;
                                  end;
                            end;
                          if b then
                            Error := 'no ' + ampm;
                        end
                      else
                        if StripString(a_p, Format) then
                          begin
                            if not StripString('a', s) then
                              if StripString('p', s) then
                                pm := true
                              else
                                Error := 'no ' + a_p;
                          end
                        else
                          begin
                            c      := Format[1];
                            Format := Copy(Format, 2, length(Format));
                            if not StripString(c, s) then
                              Error := 'literal "' + c + '"not found';
                          end;
                    M_is_Min := False;
                  end;
            '/' : begin
                    if not StripString(LowerCase(GetDateSeparator), s) then
                      Error := 'no DateSeparator';
                    Format := Copy(Format, 2, length(Format));
                  end;
            ':' : begin
                    if not StripString(LowerCase(GetTimeSeparator), s) then
                      Error := 'no TimeSeparator';
                    Format := Copy(Format, 2, length(Format));
                  end;
            '''',
            '"' : begin
                    c      := Format[1];
                    Format := Copy(Format, 2, length(Format));
                    i      := Pos(c, Format);
                    if i = 0 then
                      i := length(Format) + 1;
                    e := Copy(Format, 1, i - 1);
                    if not StripString(e, s) then
                      Error := 'literal "' + e + '" not found';
                    Format := Copy(Format, i + 1, length(Format));
                  end;
          else
            c := GetDateSeparator;
            if StripString(c, Format) then
              begin
                if not StripString(c, s) then
                  Error := 'no DateSeparator';
              end
            else
              begin
                c := GetTimeSeparator;
                if StripString(c, s) then
                  begin
                    if not StripString(c, s) then
                      Error := 'no TimeSeparator';
                  end
                else
                  begin
                    c      := Format[1];
                    Format := Copy(Format, 2, length(Format));
                    if not StripString(c, s) then
                      Error := 'literal "' + c + '" not found';
                  end;
              end;
          end;
          Format := Trim(Format);
          s      := Trim(s);
        end;
      if pm and (Hour < 12) then
        Inc(Hour, 12);
      try
        EncodeTime(Hour, Minute, Second, MSec);
      except
        Error := 'no time';
      end;
      if Error <> '' then
        begin
          s      := OldS;
          Format := OldFormat;
          Hour   := OldHour;
          Minute := OldMinute;
          Second := OldSecond;
          MSec   := OldMSec;
        end;
    end;

  begin
    Year          := 0;
    Month         := 0;
    Day           := 0;
    Hour          := 0;
    Minute        := 0;
    Second        := 0;
    MSec          := 0;
    M_is_Min      := False;
    DateContained := False;
    Error         := '';
    Format        := Trim(Format);
    s             := Trim(s);
    if Format = '' then
      Format := 'c';
    Parse(Format);
    if Error > '' then
      raise Exception.Create('StrWithFormatToDateTime: ' + Error);
    if DateContained then
      begin
        t := EncodeDate(Year, Month, Day);
        if t < 0 then
          StrWithFormatToDateTime := t - EncodeTime(Hour, Minute, Second, MSec)
        else
          StrWithFormatToDateTime := t + EncodeTime(Hour, Minute, Second, MSec);
      end
    else
      StrWithFormatToDateTime := EncodeTime(Hour, Minute, Second, MSec);
  end;

function LNull(S : String; Feld : Integer) : String;
  var    u : String;
         t : char;
         i : Integer;
  Begin
    Trim(S);
    for i := 1 to length(s) do
      if ('0' <= s[i]) and (s[i] <= '9') then
        dec(Feld);
    if Feld > 0 then
      begin
        u := StringOfChar('0', Feld);
        if s = '' then
          s := u
        else
          begin
            t := s[1];
            case t of
              '+', '-' :
                s := t + u + Copy(s, 2, length(s))
            else
              s := u + s;
            end;
          end;
      end;
    LNull := s;
  end;

function HexToInt(s : String) : LongestInt;
  var    l : LongestInt;
         c : Char;
  begin
    l := 0;
    while s > '' do
      begin
        c := s[1];
        s := copy(s, 2, length(s));
        l := l shl 4;
        if ('0' <= c) and (c <= '9') then
          l := l or (LongestInt(ord(c)) - ord('0'))
        else
          if ('a' <= c) and (c <= 'f') then
            l := l or (LongestInt(ord(c)) - ord('a') + 10)
          else
            if ('A' <= c) and (c <= 'F') then
              l := l or (LongestInt(ord(c)) - ord('A') + 10);
      end;
    HexToInt := l;
  end;

function IntToDez(i : LongestInt; Stellen : Integer;
                  Thousands : Boolean) : String;
  var    l,
         z,
         min : Integer;
         e   : Extended;
         s   : String;
         c   : char;
  begin
    if Thousands then
      begin
        e := i;
        s := Format('%.0n', [e])
      end
    else
      s := IntToStr(i);
    s := LNull(s, Stellen);
    if Thousands then
      begin
        l := length(s);
        if l > 0 then
          begin
            z := 0;
            c := GetThousandSeparator;
            case s[1] of
              '+', '-' : Min := 3;
            else
              Min := 2;
            end;
            while l >= Min do
              begin
                if ('0' <= s[l]) and (s[l] <= '9') then
                  begin
                    inc(z);
                    if z mod 3 = 0 then
                      if s[l - 1] <> c then
                        s := Copy(s, 1, l - 1) + c + Copy(s, l, length(s));
                  end;
                dec(l);
              end;
          end;
      end;
    IntToDez := s;
  end;

function DezToInt(s : String) : LongestInt;
  begin
    {$ifdef DelphiMin4}
      DezToInt := StrToInt64(StringReplace(s, GetThousandSeparator,
                                           '', [rfReplaceAll]));
    {$else}
      DezToInt := StrToInt(StringReplace(s, GetThousandSeparator,
                                         '', [rfReplaceAll]));
    {$endif}
  end;

{$ifndef DelphiMin4}
  function StringReplace(const S, OldPattern, NewPattern: string;
                         Flags: TReplaceFlags): string;
    var    p, l, m : integer;
           t, u,
           v, Old  : String;
    begin
      l := Length(OldPattern);
      m := Length(s);
      if (rfIgnoreCase in Flags) then
        begin
          v   := AnsiUpperCase(s);
          Old := AnsiUpperCase(OldPattern);
        end
      else
        begin
          v   := s;
          Old := OldPattern;
        end;
      p := pos(Old, v);
      if p = 0 then
        begin
          t := s;
          u := '';
          v := '';
        end
      else
        begin
          t := Copy(s, 1, p - 1) + NewPattern;
          u := Copy(s, p + l, m);
          v := Copy(v, p + l, m);
        end;
      if not (rfReplaceAll in Flags) then
        t := t + u
      else
        while v > '' do
          begin
            p := pos(Old, v);
            if p = 0 then
              begin
                t := t + u;
                v := '';
              end
            else
              begin
                t := t + Copy(u, 1, p - 1) + NewPattern;
                u := Copy(u, p + l, m);
                v := Copy(v, p + l, m);
              end;
          end;
      StringReplace := t;
    end;
{$endif}

{$ifdef Ver80}
  function StringOfChar(c : char; n : Byte) : String;
    var    s : String;
           i : Byte;
    begin
      s := '';
      for i := 1 to n do
        s := s + c;
      StringOfChar := s;
    end;

  function Trim(s : String) : String;
    var    von, bis : Integer;
    begin
      if s = '' then
        Trim := s
      else
        begin
          von := 1;
          bis := length(s);
          while (s[von] <= ' ') and (von < bis) do
            inc(von);
          while (s[bis] <= ' ') and (bis >= von) do
            dec(bis);
          if bis < von then
            Trim := ''
          else
            Trim := Copy(s, von, bis - von + 1);
        end;
    end;
{$endif}

procedure EraseFormat(var Format : String; Erase, Separator : Char);
  var     EraseFrom,
          EraseTo,
          len,
          p          : Integer;
  begin
    EraseFrom := Pos(Erase, Format);
    if EraseFrom > 0 then
      begin
        EraseTo := EraseFrom;
        len     := Length(Format);
        p       := EraseTo + 1;
        while p <= Len do
          if Format[p] = Erase then
            begin
              EraseTo := p;
              inc(p);
            end
          else
            p := Len + 1;
        p := EraseTo + 1;
        while p <= Len do
          if Format[p] = Separator then
            begin
              EraseTo := p;
              inc(p);
            end
          else
            p := Len + 1;
        Format := Copy(Format, 1, EraseFrom - 1) + Copy(Format, EraseTo + 1, 1);
      end;
  end;

{$ifdef Ver80}
  procedure Win16Initialization;
    const   WinIni    = 'Win.ini';
            Intl      = 'Intl';
            sLanguage = 'sLanguage';
    var     s         : String;
            IniFile   : tIniFile;
            i, h      : Integer;
    begin
      s := '';
      try
        IniFile := tIniFile.Create(WinIni);
      except
        IniFile := Nil;
      end;
      if IniFile <> Nil then
        try
          s := UpperCase(IniFile.ReadString(Intl, sLanguage, ''));
        finally
          IniFile.Free;
        end;
      if      s = 'HRV' then
        SysLocale.PriLangId := LANG_CROATIAN
      else if s = 'CSY' then
        SysLocale.PriLangId := LANG_CZECH
      else if s = 'NLD' then
        begin
          SysLocale.PriLangId := LANG_DUTCH;
          SysLocale.SubLangId := SUBLANG_DUTCH;
        end
      else if s = 'NLB' then
        begin
          SysLocale.PriLangId := LANG_DUTCH;
          SysLocale.SubLangId := SUBLANG_DUTCH_BELGIAN;
        end
      else if s = 'ENU' then
        begin
          SysLocale.PriLangId := LANG_ENGLISH;
          SysLocale.SubLangId := SUBLANG_ENGLISH_US;
        end
      else if s = 'ENG' then
        begin
          SysLocale.PriLangId := LANG_ENGLISH;
          SysLocale.SubLangId := SUBLANG_ENGLISH_UK;
        end
      else if s = 'ENI' then
        begin
          SysLocale.PriLangId := LANG_ENGLISH;
          SysLocale.SubLangId := SUBLANG_ENGLISH_EIRE;
        end
      else if s = 'FRA' then
        begin
          SysLocale.PriLangId := LANG_FRENCH;
          SysLocale.SubLangId := SUBLANG_FRENCH;
        end
      else if s = 'FRB' then
        begin
          SysLocale.PriLangId := LANG_FRENCH;
          SysLocale.SubLangId := SUBLANG_FRENCH_BELGIAN;
        end
      else if s = 'FRC' then
        begin
          SysLocale.PriLangId := LANG_FRENCH;
          SysLocale.SubLangId := SUBLANG_FRENCH_CANADIAN;
        end
      else if s = 'FRS' then
        begin
          SysLocale.PriLangId := LANG_FRENCH;
          SysLocale.SubLangId := SUBLANG_FRENCH_SWISS;
        end
      else if s = 'FRL' then
        begin
          SysLocale.PriLangId := LANG_FRENCH;
          SysLocale.SubLangId := SUBLANG_FRENCH_LUXEMBOURG;
        end
      else if s = 'DEU' then
        begin
          SysLocale.PriLangId := LANG_GERMAN;
          SysLocale.SubLangId := SUBLANG_GERMAN;
        end
      else if s = 'DES' then
        begin
          SysLocale.PriLangId := LANG_GERMAN;
          SysLocale.SubLangId := SUBLANG_GERMAN_SWISS;
        end
      else if s = 'DEA' then
        begin
          SysLocale.PriLangId := LANG_GERMAN;
          SysLocale.SubLangId := SUBLANG_GERMAN_AUSTRIAN;
        end
      else if s = 'DEL' then
        begin
          SysLocale.PriLangId := LANG_GERMAN;
          SysLocale.SubLangId := SUBLANG_GERMAN_LUXEMBOURG;
        end
      else if s = 'DEC' then
        begin
          SysLocale.PriLangId := LANG_GERMAN;
          SysLocale.SubLangId := SUBLANG_GERMAN_LIECHTENSTEIN;
        end
      else if s = 'ELL' then
        SysLocale.PriLangId := LANG_GREEK
      else if s = 'HUN' then
        SysLocale.PriLangId := LANG_HUNGARIAN
      else if s = 'ITA' then
        begin
          SysLocale.PriLangId := LANG_ITALIAN;
          SysLocale.SubLangId := SUBLANG_ITALIAN;
        end
      else if s = 'ITS' then
        begin
          SysLocale.PriLangId := LANG_ITALIAN;
          SysLocale.SubLangId := SUBLANG_ITALIAN_SWISS;
        end
      else if s = 'NON' then
        begin
          SysLocale.PriLangId := LANG_NORWEGIAN;
          SysLocale.SubLangId := SUBLANG_NORWEGIAN_NYNORSK;
        end
      else if s = 'NOR' then
        begin
          SysLocale.PriLangId := LANG_NORWEGIAN;
          SysLocale.SubLangId := SUBLANG_NORWEGIAN_BOKMAL;
        end
      else if s = 'PLK' then
        SysLocale.PriLangId := LANG_POLISH
      else if s = 'SLV' then
        SysLocale.PriLangId := LANG_SLOVENIAN
      else if s = 'ESP' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH;
        end
      else if s = 'ESM' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_MEXICAN;
        end
      else if s = 'ESN' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_MODERN;
        end
      else if s = 'ESG' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_GUATEMALA;
        end
      else if s = 'ESC' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_COSTA_RICA;
        end
      else if s = 'ESA' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_PANAMA;
        end
      else if s = 'ESD' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_DOMINICAN_REPUBLIC;
        end
      else if s = 'ESV' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_VENEZUELA;
        end
      else if s = 'ESO' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_COLOMBIA;
        end
      else if s = 'ESR' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_PERU;
        end
      else if s = 'ESS' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_ARGENTINA;
        end
      else if s = 'ESF' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_ECUADOR;
        end
      else if s = 'ESL' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_CHILE;
        end
      else if s = 'ESY' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_URUGUAY;
        end
      else if s = 'ESZ' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_PARAGUAY;
        end
      else if s = 'ESB' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_BOLIVIA;
        end
      else if s = 'ESE' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_EL_SALVADOR;
        end
      else if s = 'ESH' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_HONDURAS;
        end
      else if s = 'ESI' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_NICARAGUA;
        end
      else if s = 'ESU' then
        begin
          SysLocale.PriLangId := LANG_SPANISH;
          SysLocale.SubLangId := SUBLANG_SPANISH_PUERTO_RICO;
        end
      else if s = 'TRK' then
        SysLocale.PriLangId := LANG_TURKISH;
      case SysLocale.PriLangID of
        Lang_German:
          begin
            LongMonthNames[ 1]      := 'Januar';
            LongMonthNames[ 2]      := 'Februar';
            LongMonthNames[ 3]      := 'Mrz';
            LongMonthNames[ 4]      := 'April';
            LongMonthNames[ 5]      := 'Mai';
            LongMonthNames[ 6]      := 'Juni';
            LongMonthNames[ 7]      := 'Juli';
            LongMonthNames[ 8]      := 'August';
            LongMonthNames[ 9]      := 'September';
            LongMonthNames[10]      := 'Oktober';
            LongMonthNames[11]      := 'November';
            LongMonthNames[12]      := 'Dezember';
            LongDayNames[Sunday]    := 'Sonntag';
            LongDayNames[Monday]    := 'Montag';
            LongDayNames[Tuesday]   := 'Dienstag';
            LongDayNames[Wednesday] := 'Mittwoch';
            LongDayNames[Thursday]  := 'Donnerstag';
            LongDayNames[Friday]    := 'Freitag';
            LongDayNames[Saturday]  := 'Samstag';
          end;
        Lang_Dutch:
          begin
            LongMonthNames[ 1]      := 'Januari';
            LongMonthNames[ 2]      := 'Februari';
            LongMonthNames[ 3]      := 'Maart';
            LongMonthNames[ 4]      := 'April';
            LongMonthNames[ 5]      := 'Mei';
            LongMonthNames[ 6]      := 'Juni';
            LongMonthNames[ 7]      := 'Juli';
            LongMonthNames[ 8]      := 'Augustus';
            LongMonthNames[ 9]      := 'September';
            LongMonthNames[10]      := 'Oktober';
            LongMonthNames[11]      := 'November';
            LongMonthNames[12]      := 'December';
            LongDayNames[Sunday]    := 'Zondag';
            LongDayNames[Monday]    := 'Maandag';
            LongDayNames[Tuesday]   := 'Dinsdag';
            LongDayNames[Wednesday] := 'Woensdag';
            LongDayNames[Thursday]  := 'Donderdag';
            LongDayNames[Friday]    := 'Vrijdag';
            LongDayNames[Saturday]  := 'Zaterdag';
          end;
        Lang_French:
          begin
            LongMonthNames[ 1]      := 'Janvier';
            LongMonthNames[ 2]      := 'Fvrier';
            LongMonthNames[ 3]      := 'Mars';
            LongMonthNames[ 4]      := 'Avril';
            LongMonthNames[ 5]      := 'Mai';
            LongMonthNames[ 6]      := 'Juin';
            LongMonthNames[ 7]      := 'Juillet';
            LongMonthNames[ 8]      := 'Aot';
            LongMonthNames[ 9]      := 'Septembre';
            LongMonthNames[10]      := 'Octobre';
            LongMonthNames[11]      := 'Novembre';
            LongMonthNames[12]      := 'Dcembre';
            LongDayNames[Sunday]    := 'Dimanche';
            LongDayNames[Monday]    := 'Lundi';
            LongDayNames[Tuesday]   := 'Mardi';
            LongDayNames[Wednesday] := 'Mercredi';
            LongDayNames[Thursday]  := 'Jeudi';
            LongDayNames[Friday]    := 'Vendredi';
            LongDayNames[Saturday]  := 'Samedi';
          end;
        Lang_Italian:
          begin
            LongMonthNames[ 1]      := 'Gennaio';
            LongMonthNames[ 2]      := 'Febbraio';
            LongMonthNames[ 3]      := 'Marzo';
            LongMonthNames[ 4]      := 'Aprile';
            LongMonthNames[ 5]      := 'Maggio';
            LongMonthNames[ 6]      := 'Giugno';
            LongMonthNames[ 7]      := 'Iuglio';
            LongMonthNames[ 8]      := 'Agosto';
            LongMonthNames[ 9]      := 'Settembre';
            LongMonthNames[10]      := 'Ottobre';
            LongMonthNames[11]      := 'Novembre';
            LongMonthNames[12]      := 'Dicembre';
            LongDayNames[Sunday]    := 'Domenica';
            LongDayNames[Monday]    := 'Luned';
            LongDayNames[Tuesday]   := 'Marted';
            LongDayNames[Wednesday] := 'Mercoled';
            LongDayNames[Thursday]  := 'Gioved';
            LongDayNames[Friday]    := 'Venerd';
            LongDayNames[Saturday]  := 'Sabato';
          end;
        Lang_Spanish:
          begin
            LongMonthNames[ 1]      := 'Enero';
            LongMonthNames[ 2]      := 'Febrero';
            LongMonthNames[ 3]      := 'Marzo';
            LongMonthNames[ 4]      := 'Abril';
            LongMonthNames[ 5]      := 'Mayo';
            LongMonthNames[ 6]      := 'Junio';
            LongMonthNames[ 7]      := 'Julio';
            LongMonthNames[ 8]      := 'Agosto';
            LongMonthNames[ 9]      := 'Septiembre';
            LongMonthNames[10]      := 'Octubre';
            LongMonthNames[11]      := 'Noviembre';
            LongMonthNames[12]      := 'Diciembre';
            LongDayNames[Sunday]    := 'Domingo';
            LongDayNames[Monday]    := 'Lunes';
            LongDayNames[Tuesday]   := 'Martes';
            LongDayNames[Wednesday] := 'Mircoles';
            LongDayNames[Thursday]  := 'Jueves';
            LongDayNames[Friday]    := 'Viernes';
            LongDayNames[Saturday]  := 'Sbado';
          end;
        Lang_Norwegian:
          begin
            LongMonthNames[ 1]      := 'Januar';
            LongMonthNames[ 2]      := 'Februar';
            LongMonthNames[ 3]      := 'Mars';
            LongMonthNames[ 4]      := 'April';
            LongMonthNames[ 5]      := 'Mai';
            LongMonthNames[ 6]      := 'Juni';
            LongMonthNames[ 7]      := 'Juli';
            LongMonthNames[ 8]      := 'August';
            LongMonthNames[ 9]      := 'September';
            LongMonthNames[10]      := 'Oktober';
            LongMonthNames[11]      := 'November';
            LongMonthNames[12]      := 'Desember';
            LongDayNames[Sunday]    := 'Sundag';
            LongDayNames[Monday]    := 'Mndag';
            LongDayNames[Tuesday]   := 'Tysdag';
            LongDayNames[Wednesday] := 'Onsdag';
            LongDayNames[Thursday]  := 'Torsdag';
            LongDayNames[Friday]    := 'Fredag';
            LongDayNames[Saturday]  := 'Laurdag';
          end;
        Lang_Portuguese:
          begin
            LongMonthNames[ 1]      := 'Janeiro';
            LongMonthNames[ 2]      := 'Fevereiro';
            LongMonthNames[ 3]      := 'Maro';
            LongMonthNames[ 4]      := 'Abril';
            LongMonthNames[ 5]      := 'Maio';
            LongMonthNames[ 6]      := 'Junho';
            LongMonthNames[ 7]      := 'Julho';
            LongMonthNames[ 8]      := 'Agosto';
            LongMonthNames[ 9]      := 'Setembro';
            LongMonthNames[10]      := 'Outubro';
            LongMonthNames[11]      := 'Novembro';
            LongMonthNames[12]      := 'Dezembro';
            LongDayNames[Sunday]    := 'Domingo';
            LongDayNames[Monday]    := 'Segunda-feira';
            LongDayNames[Tuesday]   := 'Tera-feira';
            LongDayNames[Wednesday] := 'Quarta-feira';
            LongDayNames[Thursday]  := 'Quinta-feira';
            LongDayNames[Friday]    := 'Sexta-feira';
            LongDayNames[Saturday]  := 'Sbado';
          end;
        Lang_Swedish:
          begin
            LongMonthNames[ 1]      := 'Januari';
            LongMonthNames[ 2]      := 'Februari';
            LongMonthNames[ 3]      := 'Mars';
            LongMonthNames[ 4]      := 'April';
            LongMonthNames[ 5]      := 'Maj';
            LongMonthNames[ 6]      := 'Juni';
            LongMonthNames[ 7]      := 'Juli';
            LongMonthNames[ 8]      := 'Augusti';
            LongMonthNames[ 9]      := 'September';
            LongMonthNames[10]      := 'Oktober';
            LongMonthNames[11]      := 'November';
            LongMonthNames[12]      := 'December';
            LongDayNames[Sunday]    := 'Sndag';
            LongDayNames[Monday]    := 'Mndag';
            LongDayNames[Tuesday]   := 'Tisdag';
            LongDayNames[Wednesday] := 'Onsdag';
            LongDayNames[Thursday]  := 'Torsdag';
            LongDayNames[Friday]    := 'Fredag';
            LongDayNames[Saturday]  := 'Lrdag';
          end;
      end;
      for i := Low(ShortMonthNames) to High(ShortMonthNames) do
        ShortMonthNames[i] := Copy(LongMonthNames[i], 1, 3);
      case SysLocale.PriLangId of
        Lang_German : h := 2;
      else
        h := 3;
      end;
      for i := Low(ShortDayNames) to High(ShortDayNames) do
        ShortDayNames[i] := Copy(LongDayNames[i], 1, h);
    end;
{$endif}

{$ifdef Unix}
  procedure UnixInitialization;
    var     s, t      : String;
            Separator : Char;
    begin
      s := '';
      t := '';
      GetLanguageIDs(t, s);
      t := Copy(t, 4, 2);
      if      s = 'af' then
        SysLocale.PriLangId := LANG_AFRIKAANS
      else if s = 'ar' then
        SysLocale.PriLangId := LANG_ARABIC
      else if s = 'ca' then
        SysLocale.PriLangId := LANG_CATALAN
      else if s = 'cs' then
        SysLocale.PriLangId := LANG_CZECH
      else if s = 'da' then
        SysLocale.PriLangId := LANG_DANISH
      else if s = 'de' then
        begin
          SysLocale.PriLangId   := LANG_GERMAN;
          if      t = 'DE' then
            SysLocale.SubLangId := SUBLANG_GERMAN
          else if t = 'AT' then
            SysLocale.SubLangId := SUBLANG_GERMAN_AUSTRIAN
          else if t = 'CH' then
            SysLocale.SubLangId := SUBLANG_GERMAN_SWISS;
        end
      else if s = 'el' then
        SysLocale.PriLangId := LANG_GREEK
      else if s = 'en' then
        SysLocale.PriLangId := LANG_ENGLISH
      else if s = 'es' then
        SysLocale.PriLangId := LANG_SPANISH
      else if s = 'et' then
        SysLocale.PriLangId := LANG_ESTONIAN
      else if s = 'eu' then
        SysLocale.PriLangId := LANG_BASQUE
      else if s = 'fi' then
        SysLocale.PriLangId := LANG_FINNISH
      else if s = 'fo' then
        SysLocale.PriLangId := LANG_FAEROESE
      else if s = 'fr' then
        SysLocale.PriLangId := LANG_FRENCH
      else if s = 'ga' then
        begin
          SysLocale.PriLangId := LANG_ENGLISH;
          SysLocale.SubLangId := SUBLANG_ENGLISH_EIRE;
        end
      else if s = 'he' then
        SysLocale.PriLangId := LANG_HEBREW
      else if s = 'hr' then
        SysLocale.PriLangId := LANG_CROATIAN
      else if s = 'hu' then
        SysLocale.PriLangId := LANG_HUNGARIAN
      else if s = 'id' then
        SysLocale.PriLangId := LANG_INDONESIAN
      else if s = 'in' then
        SysLocale.PriLangId := LANG_INDONESIAN
      else if s = 'is' then
        SysLocale.PriLangId := LANG_ICELANDIC
      else if s = 'it' then
        SysLocale.PriLangId := LANG_ITALIAN
      else if s = 'iw' then
        SysLocale.PriLangId := LANG_HEBREW
      else if s = 'ja' then
        SysLocale.PriLangId := LANG_JAPANESE
      else if s = 'ji' then
        SysLocale.PriLangId := LANG_HEBREW
      else if s = 'kn' then
        begin
          SysLocale.PriLangId := LANG_ENGLISH;
          SysLocale.SubLangId := SUBLANG_ENGLISH_CAN;
        end
      else if s = 'ko' then
        SysLocale.PriLangId := LANG_KOREAN
      else if s = 'lt' then
        SysLocale.PriLangId := LANG_LITHUANIAN
      else if s = 'lv' then
        SysLocale.PriLangId := LANG_LATVIAN
      else if s = 'nl' then
        SysLocale.PriLangId := LANG_DUTCH
      else if s = 'no' then
        SysLocale.PriLangId := LANG_NORWEGIAN
      else if s = 'pl' then
        SysLocale.PriLangId := LANG_POLISH
      else if s = 'pt' then
        SysLocale.PriLangId := LANG_PORTUGUESE
      else if s = 'ro' then
        SysLocale.PriLangId := LANG_ROMANIAN
      else if s = 'ru' then
        SysLocale.PriLangId := LANG_RUSSIAN
      else if s = 'sh' then
        SysLocale.PriLangId := LANG_SERBIAN
      else if s = 'sk' then
        SysLocale.PriLangId := LANG_SLOVAK
      else if s = 'sl' then
        SysLocale.PriLangId := LANG_SLOVENIAN
      else if s = 'sq' then
        SysLocale.PriLangId := LANG_ALBANIAN
      else if s = 'sr' then
        SysLocale.PriLangId := LANG_SERBIAN
      else if s = 'sv' then
        SysLocale.PriLangId := LANG_SWEDISH
      else if s = 'th' then
        SysLocale.PriLangId := LANG_THAI
      else if s = 'tr' then
        SysLocale.PriLangId := LANG_TURKISH
      else if s = 'uk' then
        SysLocale.PriLangId := LANG_UKRAINIAN
      else if s = 'vi' then
        SysLocale.PriLangId := LANG_VIETNAMESE
      else if s = 'yi' then
        SysLocale.PriLangId := LANG_HEBREW
      else if s = 'zh' then
        SysLocale.PriLangId := LANG_CHINESE;
      s         := FormatSettings.LongDateFormat;
      Separator := GetTimeSeparator;
      EraseFormat(s, 'h', Separator);
      EraseFormat(s, 'n', Separator);
      EraseFormat(s, 's', #0);
      FormatSettings.LongDateFormat := Trim(s);
    end;
{$endif}

initialization
  {$ifdef Win32}
    Windows.SetThreadLocale(LOCALE_USER_DEFAULT);
    SysUtils.GetFormatSettings;
  {$else}
    SysLocale.PriLangId := LANG_NEUTRAL;
    SysLocale.SubLangId := SUBLANG_NEUTRAL;
    {$ifdef Ver80}
      { Delphi 1 }
      Win16Initialization;
    {$endif}
    {$ifdef Unix}
      UnixInitialization;
    {$endif}
  {$endif}
end.


